@@ -32,8 +32,6 @@
 
 #include "v6util.h"
 
-#define TODO_COUNT 1
-
 #define CTRL_ID 1995
 
 static HWND hMainWnd;
@@ -79,13 +77,32 @@ static LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lpara
     return DefWindowProcA(hwnd, msg, wparam, lparam);
 }
 
-static void test_updates(int style, int flags)
+static void test_updates(int style)
 {
     HWND hStatic = create_static(style);
-    RECT r1 = {20, 20, 30, 30};
+    RECT r1 = {5, 5, 30, 30}, rcClient;
     int exp;
+    LONG exstyle;
 
     flush_events();
+    trace("Testing style 0x%x\n", style);
+
+    exstyle = GetWindowLongW(hStatic, GWL_EXSTYLE);
+    if (style == SS_ETCHEDHORZ || style == SS_ETCHEDVERT || style == SS_SUNKEN)
+        ok(exstyle == WS_EX_STATICEDGE, "expected WS_EX_STATICEDGE, got %ld\n", exstyle);
+    else
+        ok(exstyle == 0, "expected 0, got %ld\n", exstyle);
+
+    GetClientRect(hStatic, &rcClient);
+    if (style == SS_ETCHEDVERT)
+        ok(rcClient.right == 0, "expected zero width, got %ld\n", rcClient.right);
+    else
+        ok(rcClient.right > 0, "expected non-zero width, got %ld\n", rcClient.right);
+    if (style == SS_ETCHEDHORZ)
+        ok(rcClient.bottom == 0, "expected zero height, got %ld\n", rcClient.bottom);
+    else
+        ok(rcClient.bottom > 0, "expected non-zero height, got %ld\n", rcClient.bottom);
+
     g_nReceivedColorStatic = 0;
     /* during each update parent WndProc will test the WM_CTLCOLORSTATIC message */
     InvalidateRect(hMainWnd, NULL, FALSE);
@@ -101,7 +118,6 @@ static void test_updates(int style, int flags)
     {
         HDC hdc = GetDC(hStatic);
         COLORREF colour = GetPixel(hdc, 10, 10);
-        todo_wine
         ok(colour == 0, "Unexpected pixel color.\n");
         ReleaseDC(hStatic, hdc);
     }
@@ -109,15 +125,9 @@ static void test_updates(int style, int flags)
     if (style != SS_ETCHEDHORZ && style != SS_ETCHEDVERT)
         exp = 4;
     else
-        exp = 1; /* SS_ETCHED* seems to send WM_CTLCOLORSTATIC only sometimes */
+        exp = 2; /* SS_ETCHEDHORZ/SS_ETCHEDVERT have empty client rect so WM_CTLCOLORSTATIC is sent only when parent window is invalidated */
 
-    if (flags & TODO_COUNT)
-    todo_wine
-        ok(g_nReceivedColorStatic == exp, "Unexpected WM_CTLCOLORSTATIC value %d\n", g_nReceivedColorStatic);
-    else if ((style & SS_TYPEMASK) == SS_ICON || (style & SS_TYPEMASK) == SS_BITMAP)
-        ok(g_nReceivedColorStatic == exp, "Unexpected %u got %u\n", exp, g_nReceivedColorStatic);
-    else
-        ok(g_nReceivedColorStatic == exp, "Unexpected WM_CTLCOLORSTATIC value %d\n", g_nReceivedColorStatic);
+    ok(g_nReceivedColorStatic == exp, "expected %u, got %u\n", exp, g_nReceivedColorStatic);
     DestroyWindow(hStatic);
 }
 
@@ -165,7 +175,6 @@ static void test_image(HBITMAP image, BOOL is_dib, BOOL is_premult, BOOL is_alph
         ok(bm.bmBits != NULL, "bmBits is NULL\n");
         memcpy(bits, bm.bmBits, 4);
         if (is_premult)
-            todo_wine
             ok(bits[0] == 0x05 &&  bits[1] == 0x09 &&  bits[2] == 0x0e && bits[3] == 0x44,
                "bits: %02x %02x %02x %02x\n", bits[0], bits[1], bits[2], bits[3]);
         else if (is_alpha)
@@ -195,7 +204,6 @@ static void test_image(HBITMAP image, BOOL is_dib, BOOL is_premult, BOOL is_alph
     DeleteDC(hdc);
 
     if (is_premult)
-        todo_wine
         ok(bits[0] == 0x05 &&  bits[1] == 0x09 &&  bits[2] == 0x0e && bits[3] == 0x44,
            "bits: %02x %02x %02x %02x\n", bits[0], bits[1], bits[2], bits[3]);
     else if (is_alpha)
@@ -208,6 +216,8 @@ static void test_image(HBITMAP image, BOOL is_dib, BOOL is_premult, BOOL is_alph
 
 static void test_set_image(void)
 {
+    char resource[4];
+    WCHAR resource_unicode[3];
     HWND hwnd = create_static(SS_BITMAP);
     HBITMAP bmp1, bmp2, image;
 
@@ -262,6 +272,33 @@ static void test_set_image(void)
 
     test_image(image, TRUE, FALSE, FALSE);
 
+    resource[0] = '\xff';
+    resource[1] = PtrToUlong(MAKEINTRESOURCEW(IDB_BITMAP_1x1_32BPP));
+    resource[2] = PtrToUlong(MAKEINTRESOURCEW(IDB_BITMAP_1x1_32BPP)) >> 8;
+    resource[3] = 0;
+
+    resource_unicode[0] = 0xffff;
+    resource_unicode[1] = PtrToUlong(MAKEINTRESOURCEW(IDB_BITMAP_1x1_32BPP));
+    resource_unicode[2] = 0;
+
+    hwnd = CreateWindowW(L"Static", resource_unicode, WS_VISIBLE|WS_CHILD|SS_BITMAP, 5, 5, 100, 100,
+                         hMainWnd, (HMENU)CTRL_ID, GetModuleHandleW(NULL), 0);
+
+    bmp1 = (HBITMAP)SendMessageW(hwnd, STM_GETIMAGE, IMAGE_BITMAP, 0);
+    ok(bmp1 != NULL, "got NULL\n");
+    ok(bmp1 != image, "bmp == image\n");
+    test_image(bmp1, TRUE, TRUE, TRUE);
+    DestroyWindow(hwnd);
+
+    hwnd = CreateWindowA("Static", resource, WS_VISIBLE|WS_CHILD|SS_BITMAP, 5, 5, 100, 100,
+                         hMainWnd, (HMENU)CTRL_ID, GetModuleHandleW(NULL), 0);
+
+    bmp1 = (HBITMAP)SendMessageA(hwnd, STM_GETIMAGE, IMAGE_BITMAP, 0);
+    ok(bmp1 != NULL, "got NULL\n");
+    ok(bmp1 != image, "bmp == image\n");
+    test_image(bmp1, TRUE, TRUE, TRUE);
+    DestroyWindow(hwnd);
+
     DeleteObject(image);
 }
 
@@ -283,10 +320,10 @@ static void test_STM_SETIMAGE(void)
 
     for (type = SS_LEFT; type < SS_ETCHEDFRAME; type++)
     {
-        winetest_push_context("%u", type);
+        winetest_push_context("%lu", type);
 
         hwnd = create_static(type);
-        ok(hwnd != 0, "failed to create static type %#x\n", type);
+        ok(hwnd != 0, "failed to create static type %#lx\n", type);
 
         /* set icon */
         g_nReceivedColorStatic = 0;
@@ -405,15 +442,21 @@ START_TEST(static)
         GetModuleHandleA(NULL), NULL);
     ShowWindow(hMainWnd, SW_SHOW);
 
-    test_updates(0, 0);
-    test_updates(SS_SIMPLE, 0);
-    test_updates(SS_ICON, 0);
-    test_updates(SS_BITMAP, 0);
-    test_updates(SS_BITMAP | SS_CENTERIMAGE, 0);
-    test_updates(SS_BLACKRECT, TODO_COUNT);
-    test_updates(SS_WHITERECT, TODO_COUNT);
-    test_updates(SS_ETCHEDHORZ, TODO_COUNT);
-    test_updates(SS_ETCHEDVERT, TODO_COUNT);
+    test_updates(0);
+    test_updates(SS_ICON);
+    test_updates(SS_BLACKRECT);
+    test_updates(SS_WHITERECT);
+    test_updates(SS_BLACKFRAME);
+    test_updates(SS_WHITEFRAME);
+    test_updates(SS_USERITEM);
+    test_updates(SS_SIMPLE);
+    test_updates(SS_OWNERDRAW);
+    test_updates(SS_BITMAP);
+    test_updates(SS_BITMAP | SS_CENTERIMAGE);
+    test_updates(SS_ETCHEDHORZ);
+    test_updates(SS_ETCHEDVERT);
+    test_updates(SS_ETCHEDFRAME);
+    test_updates(SS_SUNKEN);
     test_set_text();
     test_set_image();
     test_STM_SETIMAGE();
