@@ -125,6 +125,7 @@ struct d3d9_device
 HRESULT device_init(struct d3d9_device *device, struct d3d9 *parent, struct wined3d *wined3d,
         UINT adapter, D3DDEVTYPE device_type, HWND focus_window, DWORD flags,
         D3DPRESENT_PARAMETERS *parameters, D3DDISPLAYMODEEX *mode) DECLSPEC_HIDDEN;
+void d3d9_device_upload_managed_textures(struct d3d9_device *device);
 
 struct d3d9_resource
 {
@@ -219,7 +220,7 @@ struct d3d9_texture
 {
     IDirect3DBaseTexture9 IDirect3DBaseTexture9_iface;
     struct d3d9_resource resource;
-    struct wined3d_texture *wined3d_texture;
+    struct wined3d_texture *wined3d_texture, *draw_texture;
     struct d3d9_device *parent_device;
     struct list rtv_list;
     DWORD usage;
@@ -228,12 +229,18 @@ struct d3d9_texture
     D3DTEXTUREFILTERTYPE autogen_filter_type;
 };
 
-HRESULT cubetexture_init(struct d3d9_texture *texture, struct d3d9_device *device,
-        UINT edge_length, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-HRESULT texture_init(struct d3d9_texture *texture, struct d3d9_device *device,
-        UINT width, UINT height, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-HRESULT volumetexture_init(struct d3d9_texture *texture, struct d3d9_device *device,
-        UINT width, UINT height, UINT depth, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
+HRESULT d3d9_texture_2d_init(struct d3d9_texture *texture, struct d3d9_device *device, unsigned int width,
+        unsigned int height, unsigned int level_count, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+HRESULT d3d9_texture_3d_init(struct d3d9_texture *texture, struct d3d9_device *device, unsigned int width,
+        unsigned int height, unsigned int depth, unsigned int level_count, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+HRESULT d3d9_texture_cube_init(struct d3d9_texture *texture, struct d3d9_device *device,
+        unsigned int edge_length, unsigned int level_count, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+
+static inline struct wined3d_texture *d3d9_texture_get_draw_texture(struct d3d9_texture *texture)
+{
+    return texture->draw_texture ? texture->draw_texture : texture->wined3d_texture;
+}
+
 struct d3d9_texture *unsafe_impl_from_IDirect3DBaseTexture9(IDirect3DBaseTexture9 *iface) DECLSPEC_HIDDEN;
 void d3d9_texture_flag_auto_gen_mipmap(struct d3d9_texture *texture) DECLSPEC_HIDDEN;
 void d3d9_texture_gen_auto_mipmap(struct d3d9_texture *texture) DECLSPEC_HIDDEN;
@@ -318,6 +325,9 @@ static inline DWORD d3dusage_from_wined3dusage(unsigned int wined3d_usage, unsig
 
 static inline D3DPOOL d3dpool_from_wined3daccess(unsigned int access, unsigned int usage)
 {
+    if (usage & WINED3DUSAGE_MANAGED)
+        return D3DPOOL_MANAGED;
+
     switch (access & (WINED3D_RESOURCE_ACCESS_GPU | WINED3D_RESOURCE_ACCESS_CPU))
     {
         default:
@@ -373,6 +383,16 @@ static inline unsigned int wined3daccess_from_d3dpool(D3DPOOL pool, unsigned int
     return access;
 }
 
+static inline unsigned int wined3d_usage_from_d3d(D3DPOOL pool, DWORD usage)
+{
+    usage &= WINED3DUSAGE_MASK;
+    if (pool == D3DPOOL_SCRATCH)
+        usage |= WINED3DUSAGE_SCRATCH;
+    else if (pool == D3DPOOL_MANAGED)
+        usage |= WINED3DUSAGE_MANAGED;
+    return usage;
+}
+
 static inline unsigned int wined3d_bind_flags_from_d3d9_usage(DWORD usage)
 {
     unsigned int bind_flags = 0;
@@ -385,11 +405,6 @@ static inline unsigned int wined3d_bind_flags_from_d3d9_usage(DWORD usage)
     return bind_flags;
 }
 
-static inline DWORD wined3dusage_from_d3dusage(unsigned int usage)
-{
-    return usage & WINED3DUSAGE_MASK;
-}
-
 static inline enum wined3d_multisample_type wined3d_multisample_type_from_d3d(D3DMULTISAMPLE_TYPE type)
 {
     return (enum wined3d_multisample_type)type;
