@@ -90,7 +90,7 @@ static ULONG WINAPI IDirectMusicDownloadedInstrumentImpl_AddRef(LPDIRECTMUSICDOW
     IDirectMusicDownloadedInstrumentImpl *This = impl_from_IDirectMusicDownloadedInstrument(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p)->(): new ref = %u\n", iface, ref);
+    TRACE("(%p): new ref = %lu\n", iface, ref);
 
     return ref;
 }
@@ -100,7 +100,7 @@ static ULONG WINAPI IDirectMusicDownloadedInstrumentImpl_Release(LPDIRECTMUSICDO
     IDirectMusicDownloadedInstrumentImpl *This = impl_from_IDirectMusicDownloadedInstrument(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p)->(): new ref = %u\n", iface, ref);
+    TRACE("(%p): new ref = %lu\n", iface, ref);
 
     if (!ref)
     {
@@ -151,7 +151,7 @@ static HRESULT WINAPI synth_port_QueryInterface(IDirectMusicPort *iface, REFIID
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    TRACE("(%p/%p)->(%s, %p)\n", iface, This, debugstr_dmguid(riid), ret_iface);
+    TRACE("(%p, %s, %p)\n", iface, debugstr_dmguid(riid), ret_iface);
 
     if (IsEqualIID(riid, &IID_IUnknown) || IsEqualGUID(riid, &IID_IDirectMusicPort))
         *ret_iface = &This->IDirectMusicPort_iface;
@@ -177,7 +177,7 @@ static ULONG WINAPI synth_port_AddRef(IDirectMusicPort *iface)
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p)->(): new ref = %u\n", This, ref);
+    TRACE("(%p): new ref = %lu\n", This, ref);
 
     DMUSIC_LockModule();
 
@@ -189,7 +189,7 @@ static ULONG WINAPI synth_port_Release(IDirectMusicPort *iface)
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p)->(): new ref = %u\n", This, ref);
+    TRACE("(%p): new ref = %lu\n", This, ref);
 
     if (!ref)
     {
@@ -218,7 +218,7 @@ static HRESULT WINAPI synth_port_PlayBuffer(IDirectMusicPort *iface, IDirectMusi
     LPBYTE data;
     DWORD size;
 
-    TRACE("(%p/%p)->(%p)\n", iface, This, buffer);
+    TRACE("(%p, %p)\n", iface, buffer);
 
     if (!buffer)
         return E_POINTER;
@@ -266,7 +266,7 @@ static HRESULT WINAPI synth_port_DownloadInstrument(IDirectMusicPort *iface, IDi
     ULONG size;
     ULONG i;
 
-    TRACE("(%p/%p)->(%p, %p, %p, %d)\n", iface, This, instrument, downloaded_instrument, note_ranges, num_note_ranges);
+    TRACE("(%p, %p, %p, %p, %ld)\n", iface, instrument, downloaded_instrument, note_ranges, num_note_ranges);
 
     if (!instrument || !downloaded_instrument || (num_note_ranges && !note_ranges))
         return E_POINTER;
@@ -339,10 +339,9 @@ static HRESULT WINAPI synth_port_DownloadInstrument(IDirectMusicPort *iface, IDi
 static HRESULT WINAPI synth_port_UnloadInstrument(IDirectMusicPort *iface,
         IDirectMusicDownloadedInstrument *downloaded_instrument)
 {
-    struct synth_port *This = synth_from_IDirectMusicPort(iface);
     IDirectMusicDownloadedInstrumentImpl *downloaded_object = unsafe_impl_from_IDirectMusicDownloadedInstrument(downloaded_instrument);
 
-    TRACE("(%p/%p)->(%p)\n", iface, This, downloaded_instrument);
+    TRACE("(%p, %p)\n", iface, downloaded_instrument);
 
     if (!downloaded_instrument)
         return E_POINTER;
@@ -361,7 +360,7 @@ static HRESULT WINAPI synth_port_GetLatencyClock(IDirectMusicPort *iface, IRefer
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    TRACE("(%p/%p)->(%p)\n", iface, This, clock);
+    TRACE("(%p, %p)\n", iface, clock);
 
     return IDirectMusicSynth8_GetLatencyClock(This->synth, clock);
 }
@@ -385,7 +384,7 @@ static HRESULT WINAPI synth_port_GetCaps(IDirectMusicPort *iface, DMUS_PORTCAPS
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    TRACE("(%p/%p)->(%p)\n", iface, This, port_caps);
+    TRACE("(%p, %p)\n", iface, port_caps);
 
     return IDirectMusicSynth_GetPortCaps(This->synth, port_caps);
 }
@@ -396,7 +395,7 @@ static HRESULT WINAPI synth_port_DeviceIoControl(IDirectMusicPort *iface, DWORD
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    FIXME("(%p/%p)->(%d, %p, %d, %p, %d, %p, %p): stub\n", iface, This, io_control_code, in_buffer, in_buffer_size, out_buffer, out_buffer_size, bytes_returned, overlapped);
+    FIXME("(%p/%p, %ld, %p, %ld, %p, %ld, %p, %p): stub\n", iface, This, io_control_code, in_buffer, in_buffer_size, out_buffer, out_buffer_size, bytes_returned, overlapped);
 
     return S_OK;
 }
@@ -405,7 +404,7 @@ static HRESULT WINAPI synth_port_SetNumChannelGroups(IDirectMusicPort *iface, DW
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    FIXME("(%p/%p)->(%d): semi-stub\n", iface, This, channel_groups);
+    FIXME("(%p, %ld): semi-stub\n", iface, channel_groups);
 
     This->nrofgroups = channel_groups;
 
@@ -416,7 +415,7 @@ static HRESULT WINAPI synth_port_GetNumChannelGroups(IDirectMusicPort *iface, DW
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    TRACE("(%p/%p)->(%p)\n", iface, This, channel_groups);
+    TRACE("(%p, %p)\n", iface, channel_groups);
 
     *channel_groups = This->nrofgroups;
 
@@ -457,11 +456,11 @@ static HRESULT WINAPI synth_port_SetChannelPriority(IDirectMusicPort *iface, DWO
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    FIXME("(%p/%p)->(%d, %d, %d): semi-stub\n", iface, This, channel_group, channel, priority);
+    FIXME("(%p/%p, %ld, %ld, %ld): semi-stub\n", iface, This, channel_group, channel, priority);
 
     if (channel > 16)
     {
-        WARN("isn't there supposed to be 16 channels (no. %d requested)?! (faking as it is ok)\n", channel);
+        WARN("isn't there supposed to be 16 channels (no. %ld requested)?! (faking as it is ok)\n", channel);
         /*return E_INVALIDARG;*/
     }
 
@@ -473,7 +472,7 @@ static HRESULT WINAPI synth_port_GetChannelPriority(IDirectMusicPort *iface, DWO
 {
     struct synth_port *This = synth_from_IDirectMusicPort(iface);
 
-    TRACE("(%p/%p)->(%u, %u, %p)\n", iface, This, channel_group, channel, priority);
+    TRACE("(%p, %lu, %lu, %p)\n", iface, channel_group, channel, priority);
 
     *priority = This->group[channel_group - 1].channel[channel].priority;
 
@@ -511,50 +510,21 @@ static HRESULT WINAPI synth_port_SetDirectSound(IDirectMusicPort *iface, IDirect
     return S_OK;
 }
 
-static HRESULT WINAPI synth_port_GetFormat(IDirectMusicPort *iface, WAVEFORMATEX *pWaveFormatEx,
-        DWORD *pdwWaveFormatExSize, DWORD *pdwBufferSize)
+static HRESULT WINAPI synth_port_GetFormat(IDirectMusicPort *iface, WAVEFORMATEX *format,
+        DWORD *fmtsize, DWORD *bufsize)
 {
-	struct synth_port *This = synth_from_IDirectMusicPort(iface);
-	WAVEFORMATEX format;
-	FIXME("(%p, %p, %p, %p): stub\n", This, pWaveFormatEx, pdwWaveFormatExSize, pdwBufferSize);
+    struct synth_port *This = synth_from_IDirectMusicPort(iface);
+    HRESULT hr;
 
-	if (pWaveFormatEx == NULL)
-	{
-		if (pdwWaveFormatExSize)
-			*pdwWaveFormatExSize = sizeof(format);
-		else
-			return E_POINTER;
-	}
-	else
-	{
-		if (pdwWaveFormatExSize == NULL)
-			return E_POINTER;
+    TRACE("(%p, %p, %p, %p)\n", This, format, fmtsize, bufsize);
 
-		/* Just fill this in with something that will not crash Direct Sound for now. */
-		/* It won't be used anyway until Performances are completed */
-		format.wFormatTag = WAVE_FORMAT_PCM;
-		format.nChannels = 2; /* This->params.dwAudioChannels; */
-		format.nSamplesPerSec = 44100; /* This->params.dwSampleRate; */
-		format.wBitsPerSample = 16;	/* FIXME: check this */
-		format.nBlockAlign = (format.wBitsPerSample * format.nChannels) / 8;
-		format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign;
-		format.cbSize = 0;
+    if (FAILED(hr = IDirectMusicSynth_GetFormat(This->synth, format, fmtsize)))
+        return hr;
 
-		if (*pdwWaveFormatExSize >= sizeof(format))
-		{
-			CopyMemory(pWaveFormatEx, &format, min(sizeof(format), *pdwWaveFormatExSize));
-			*pdwWaveFormatExSize = sizeof(format);	/* FIXME check if this is set */
-		}
-		else
-			return E_POINTER;	/* FIXME find right error */
-	}
+    if (bufsize)
+        hr = IDirectMusicSynthSink_GetDesiredBufferSize(This->synth_sink, bufsize);
 
-	if (pdwBufferSize)
-		*pdwBufferSize = 44100 * 2 * 2;
-	else
-		return E_POINTER;
-
-	return S_OK;
+    return hr;
 }
 
 static const IDirectMusicPortVtbl synth_port_vtbl = {
@@ -586,7 +556,7 @@ static HRESULT WINAPI synth_port_download_QueryInterface(IDirectMusicPortDownloa
 {
     struct synth_port *This = synth_from_IDirectMusicPortDownload(iface);
 
-    TRACE("(%p/%p)->(%s, %p)\n", iface, This, debugstr_dmguid(riid), ret_iface);
+    TRACE("(%p, %s, %p)\n", iface, debugstr_dmguid(riid), ret_iface);
 
     return IDirectMusicPort_QueryInterface(&This->IDirectMusicPort_iface, riid, ret_iface);
 }
@@ -595,7 +565,7 @@ static ULONG WINAPI synth_port_download_AddRef(IDirectMusicPortDownload *iface)
 {
     struct synth_port *This = synth_from_IDirectMusicPortDownload(iface);
 
-    TRACE("(%p/%p)->()\n", iface, This);
+    TRACE("(%p)\n", iface);
 
     return IDirectMusicPort_AddRef(&This->IDirectMusicPort_iface);
 }
@@ -604,7 +574,7 @@ static ULONG WINAPI synth_port_download_Release(IDirectMusicPortDownload *iface)
 {
     struct synth_port *This = synth_from_IDirectMusicPortDownload(iface);
 
-    TRACE("(%p/%p)->()\n", iface, This);
+    TRACE("(%p)\n", iface);
 
     return IDirectMusicPort_Release(&This->IDirectMusicPort_iface);
 }
@@ -614,7 +584,7 @@ static HRESULT WINAPI synth_port_download_GetBuffer(IDirectMusicPortDownload *if
 {
     struct synth_port *This = synth_from_IDirectMusicPortDownload(iface);
 
-    FIXME("(%p/%p)->(%u, %p): stub\n", iface, This, DLId, IDMDownload);
+    FIXME("(%p/%p, %lu, %p): stub\n", iface, This, DLId, IDMDownload);
 
     if (!IDMDownload)
         return E_POINTER;
@@ -627,7 +597,7 @@ static HRESULT WINAPI synth_port_download_AllocateBuffer(IDirectMusicPortDownloa
 {
     struct synth_port *This = synth_from_IDirectMusicPortDownload(iface);
 
-    FIXME("(%p/%p)->(%u, %p): stub\n", iface, This, size, IDMDownload);
+    FIXME("(%p/%p, %lu, %p): stub\n", iface, This, size, IDMDownload);
 
     return S_OK;
 }
@@ -636,7 +606,7 @@ static HRESULT WINAPI synth_port_download_GetDLId(IDirectMusicPortDownload *ifac
 {
     struct synth_port *This = synth_from_IDirectMusicPortDownload(iface);
 
-    FIXME("(%p/%p)->(%p, %u): stub\n", iface, This, start_DLId, count);
+    FIXME("(%p/%p, %p, %lu): stub\n", iface, This, start_DLId, count);
 
     return S_OK;
 }
@@ -686,7 +656,7 @@ static HRESULT WINAPI synth_port_thru_QueryInterface(IDirectMusicThru *iface, RE
 {
     struct synth_port *This = synth_from_IDirectMusicThru(iface);
 
-    TRACE("(%p/%p)->(%s, %p)\n", iface, This, debugstr_dmguid(riid), ret_iface);
+    TRACE("(%p, %s, %p)\n", iface, debugstr_dmguid(riid), ret_iface);
 
     return IDirectMusicPort_QueryInterface(&This->IDirectMusicPort_iface, riid, ret_iface);
 }
@@ -695,7 +665,7 @@ static ULONG WINAPI synth_port_thru_AddRef(IDirectMusicThru *iface)
 {
     struct synth_port *This = synth_from_IDirectMusicThru(iface);
 
-    TRACE("(%p/%p)->()\n", iface, This);
+    TRACE("(%p)\n", iface);
 
     return IDirectMusicPort_AddRef(&This->IDirectMusicPort_iface);
 }
@@ -704,7 +674,7 @@ static ULONG WINAPI synth_port_thru_Release(IDirectMusicThru *iface)
 {
     struct synth_port *This = synth_from_IDirectMusicThru(iface);
 
-    TRACE("(%p/%p)->()\n", iface, This);
+    TRACE("(%p)\n", iface);
 
     return IDirectMusicPort_Release(&This->IDirectMusicPort_iface);
 }
@@ -715,7 +685,7 @@ static HRESULT WINAPI synth_port_thru_ThruChannel(IDirectMusicThru *iface, DWORD
 {
     struct synth_port *This = synth_from_IDirectMusicThru(iface);
 
-    FIXME("(%p/%p)->(%d, %d, %d, %d, %p): stub\n", iface, This, source_channel_group, source_channel, destination_channel_group, destination_channel, destination_port);
+    FIXME("(%p/%p, %ld, %ld, %ld, %ld, %p): stub\n", iface, This, source_channel_group, source_channel, destination_channel_group, destination_channel, destination_port);
 
     return S_OK;
 }
@@ -754,12 +724,12 @@ static ULONG WINAPI IKsControlImpl_Release(IKsControl *iface)
 static HRESULT WINAPI IKsControlImpl_KsProperty(IKsControl *iface, KSPROPERTY *prop,
         ULONG prop_len, void *data, ULONG data_len, ULONG *ret_len)
 {
-    TRACE("(%p)->(%p, %u, %p, %u, %p)\n", iface, prop, prop_len, data, data_len, ret_len);
-    TRACE("prop = %s - %u - %u\n", debugstr_guid(&prop->u.s.Set), prop->u.s.Id, prop->u.s.Flags);
+    TRACE("(%p, %p, %lu, %p, %lu, %p)\n", iface, prop, prop_len, data, data_len, ret_len);
+    TRACE("prop = %s - %lu - %lu\n", debugstr_guid(&prop->u.s.Set), prop->u.s.Id, prop->u.s.Flags);
 
     if (prop->u.s.Flags != KSPROPERTY_TYPE_GET)
     {
-        FIXME("prop flags %u not yet supported\n", prop->u.s.Flags);
+        FIXME("prop flags %lu not yet supported\n", prop->u.s.Flags);
         return S_FALSE;
     }
 
@@ -776,7 +746,7 @@ static HRESULT WINAPI IKsControlImpl_KsProperty(IKsControl *iface, KSPROPERTY *p
 static HRESULT WINAPI IKsControlImpl_KsMethod(IKsControl *iface, KSMETHOD *method,
         ULONG method_len, void *data, ULONG data_len, ULONG *ret_len)
 {
-    FIXME("(%p)->(%p, %u, %p, %u, %p): stub\n", iface, method, method_len, data, data_len, ret_len);
+    FIXME("(%p, %p, %lu, %p, %lu, %p): stub\n", iface, method, method_len, data, data_len, ret_len);
 
     return E_NOTIMPL;
 }
@@ -784,7 +754,7 @@ static HRESULT WINAPI IKsControlImpl_KsMethod(IKsControl *iface, KSMETHOD *metho
 static HRESULT WINAPI IKsControlImpl_KsEvent(IKsControl *iface, KSEVENT *event, ULONG event_len,
         void *data, ULONG data_len, ULONG *ret_len)
 {
-    FIXME("(%p)->(%p, %u, %p, %u, %p): stub\n", iface, event, event_len, data, data_len, ret_len);
+    FIXME("(%p, %p, %lu, %p, %lu, %p): stub\n", iface, event, event_len, data, data_len, ret_len);
 
     return E_NOTIMPL;
 }
@@ -917,7 +887,7 @@ static ULONG WINAPI midi_IDirectMusicPort_AddRef(IDirectMusicPort *iface)
     struct midi_port *This = impl_from_IDirectMusicPort(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref = %u\n", iface, ref);
+    TRACE("(%p) ref = %lu\n", iface, ref);
 
     return ref;
 }
@@ -927,7 +897,7 @@ static ULONG WINAPI midi_IDirectMusicPort_Release(IDirectMusicPort *iface)
     struct midi_port *This = impl_from_IDirectMusicPort(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref = %u\n", iface, ref);
+    TRACE("(%p) ref = %lu\n", iface, ref);
 
     if (!ref) {
         if (This->clock)
@@ -966,7 +936,7 @@ static HRESULT WINAPI midi_IDirectMusicPort_DownloadInstrument(IDirectMusicPort
         IDirectMusicInstrument *instrument, IDirectMusicDownloadedInstrument **downloaded,
         DMUS_NOTERANGE *ranges, DWORD num_ranges)
 {
-    FIXME("(%p, %p, %p, %p, %u) stub!\n", iface, instrument, downloaded, ranges, num_ranges);
+    FIXME("(%p, %p, %p, %p, %lu) stub!\n", iface, instrument, downloaded, ranges, num_ranges);
 
     return E_NOTIMPL;
 }
@@ -1021,7 +991,7 @@ static HRESULT WINAPI midi_IDirectMusicPort_DeviceIoControl(IDirectMusicPort *if
         DWORD io_control_code, void *in, DWORD size_in, void *out, DWORD size_out, DWORD *ret_len,
         OVERLAPPED *overlapped)
 {
-    FIXME("(%p, %u, %p, %u, %p, %u, %p, %p) stub!\n", iface, io_control_code, in, size_in, out
+    FIXME("(%p, %lu, %p, %lu, %p, %lu, %p, %p) stub!\n", iface, io_control_code, in, size_in, out
             , size_out, ret_len, overlapped);
 
     return E_NOTIMPL;
@@ -1030,7 +1000,7 @@ static HRESULT WINAPI midi_IDirectMusicPort_DeviceIoControl(IDirectMusicPort *if
 static HRESULT WINAPI midi_IDirectMusicPort_SetNumChannelGroups(IDirectMusicPort *iface,
         DWORD cgroups)
 {
-    FIXME("(%p, %u) stub!\n", iface, cgroups);
+    FIXME("(%p, %lu) stub!\n", iface, cgroups);
 
     return E_NOTIMPL;
 }
@@ -1053,7 +1023,7 @@ static HRESULT WINAPI midi_IDirectMusicPort_Activate(IDirectMusicPort *iface, BO
 static HRESULT WINAPI midi_IDirectMusicPort_SetChannelPriority(IDirectMusicPort *iface,
         DWORD channel_group, DWORD channel, DWORD priority)
 {
-    FIXME("(%p, %u, %u, %u) stub!\n", iface, channel_group, channel, priority);
+    FIXME("(%p, %lu, %lu, %lu) stub!\n", iface, channel_group, channel, priority);
 
     return E_NOTIMPL;
 }
@@ -1061,7 +1031,7 @@ static HRESULT WINAPI midi_IDirectMusicPort_SetChannelPriority(IDirectMusicPort
 static HRESULT WINAPI midi_IDirectMusicPort_GetChannelPriority(IDirectMusicPort *iface,
         DWORD channel_group, DWORD channel, DWORD *priority)
 {
-    FIXME("(%p, %u, %u, %p) stub!\n", iface, channel_group, channel, priority);
+    FIXME("(%p, %lu, %lu, %p) stub!\n", iface, channel_group, channel, priority);
 
     return E_NOTIMPL;
 }
@@ -1135,7 +1105,7 @@ static ULONG WINAPI midi_IDirectMusicThru_Release(IDirectMusicThru *iface)
 static HRESULT WINAPI midi_IDirectMusicThru_ThruChannel(IDirectMusicThru *iface, DWORD src_group,
         DWORD src_channel, DWORD dest_group, DWORD dest_channel, IDirectMusicPort *dest_port)
 {
-    FIXME("(%p, %u, %u, %u, %u, %p) stub!\n", iface, src_group, src_channel, dest_group,
+    FIXME("(%p, %lu, %lu, %lu, %lu, %p) stub!\n", iface, src_group, src_channel, dest_group,
             dest_channel, dest_port);
 
     return S_OK;
