@@ -110,6 +110,17 @@ extern GpStatus METAFILE_DrawDriverString(GpMetafile *metafile, GDIPCONST UINT16
 extern GpStatus METAFILE_FillRegion(GpMetafile* metafile, GpBrush* brush,
     GpRegion* region) DECLSPEC_HIDDEN;
 extern void METAFILE_Free(GpMetafile *metafile) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_DrawEllipse(GpMetafile *metafile, GpPen *pen, GpRectF *rect) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_FillEllipse(GpMetafile *metafile, GpBrush *brush, GpRectF *rect) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_DrawRectangles(GpMetafile *metafile, GpPen *pen, const GpRectF *rects, INT count) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_FillPie(GpMetafile *metafile, GpBrush *brush, const GpRectF *rect,
+    REAL startAngle, REAL sweepAngle) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_DrawArc(GpMetafile *metafile, GpPen *pen, const GpRectF *rect,
+    REAL startAngle, REAL sweepAngle) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_OffsetClip(GpMetafile *metafile, REAL dx, REAL dy) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_ResetClip(GpMetafile *metafile) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_SetClipPath(GpMetafile *metafile, GpPath *path, CombineMode mode) DECLSPEC_HIDDEN;
+extern GpStatus METAFILE_SetRenderingOrigin(GpMetafile *metafile, INT x, INT y) DECLSPEC_HIDDEN;
 
 extern void calc_curve_bezier(const GpPointF *pts, REAL tension, REAL *x1,
     REAL *y1, REAL *x2, REAL *y2) DECLSPEC_HIDDEN;
@@ -196,8 +207,8 @@ extern void convert_32bppARGB_to_32bppPARGB(UINT width, UINT height,
     BYTE *dst_bits, INT dst_stride, const BYTE *src_bits, INT src_stride) DECLSPEC_HIDDEN;
 
 extern GpStatus convert_pixels(INT width, INT height,
-    INT dst_stride, BYTE *dst_bits, PixelFormat dst_format,
-    INT src_stride, const BYTE *src_bits, PixelFormat src_format, ColorPalette *palette) DECLSPEC_HIDDEN;
+    INT dst_stride, BYTE *dst_bits, PixelFormat dst_format, ColorPalette *dst_palette,
+    INT src_stride, const BYTE *src_bits, PixelFormat src_format, ColorPalette *src_palette) DECLSPEC_HIDDEN;
 
 extern PixelFormat apply_image_attributes(const GpImageAttributes *attributes, LPBYTE data,
     UINT width, UINT height, INT stride, ColorAdjustType type, PixelFormat fmt) DECLSPEC_HIDDEN;
@@ -224,6 +235,8 @@ struct GpPen{
     GpBrush *brush;
     GpPenAlignment align;
     GpMatrix transform;
+    REAL *compound_array;
+    INT compound_array_size;
 };
 
 struct GpGraphics{
@@ -360,7 +373,7 @@ struct GpImage{
     UINT frame_count, current_frame;
     ColorPalette *palette;
     REAL xres, yres;
-    LONG busy;
+    SRWLOCK lock;
 };
 
 #define EmfPlusObjectTableSize 64
@@ -532,6 +545,8 @@ struct GpFontFamily{
     WCHAR FamilyName[LF_FACESIZE];
     UINT16 em_height, ascent, descent, line_spacing; /* in font units */
     int dpi;
+    BOOL installed;
+    LONG ref;
 };
 
 /* internal use */
@@ -600,17 +615,22 @@ GpStatus gdip_format_string(HDC hdc,
 
 void get_log_fontW(const GpFont *, GpGraphics *, LOGFONTW *) DECLSPEC_HIDDEN;
 
-static inline BOOL image_lock(GpImage *image, BOOL *unlock)
+static inline BOOL image_lock(GpImage *image)
 {
-    LONG tid = GetCurrentThreadId(), owner_tid;
-    owner_tid = InterlockedCompareExchange(&image->busy, tid, 0);
-    *unlock = !owner_tid;
-    return !owner_tid || owner_tid==tid;
+    return TryAcquireSRWLockExclusive(&image->lock);
 }
 
-static inline void image_unlock(GpImage *image, BOOL unlock)
+static inline void image_unlock(GpImage *image)
 {
-    if (unlock) image->busy = 0;
+    ReleaseSRWLockExclusive(&image->lock);
+}
+
+static inline void set_rect(GpRectF *rect, REAL x, REAL y, REAL width, REAL height)
+{
+    rect->X = x;
+    rect->Y = y;
+    rect->Width = width;
+    rect->Height = height;
 }
 
 #endif
