@@ -17,6 +17,7 @@
  */
 
 #include <math.h>
+#include <locale.h>
 #include <assert.h>
 
 #include "jscript.h"
@@ -39,14 +40,16 @@ static inline NumberInstance *number_from_jsdisp(jsdisp_t *jsdisp)
     return CONTAINING_RECORD(jsdisp, NumberInstance, dispex);
 }
 
-static inline NumberInstance *number_from_vdisp(vdisp_t *vdisp)
+static inline HRESULT numberval_this(jsval_t vthis, DOUBLE *ret)
 {
-    return number_from_jsdisp(vdisp->u.jsdisp);
-}
-
-static inline NumberInstance *number_this(vdisp_t *jsthis)
-{
-    return is_vclass(jsthis, JSCLASS_NUMBER) ? number_from_vdisp(jsthis) : NULL;
+    jsdisp_t *jsdisp;
+    if(is_number(vthis))
+        *ret = get_number(vthis);
+    else if(is_object_instance(vthis) && (jsdisp = to_jsdisp(get_object(vthis))) && is_class(jsdisp, JSCLASS_NUMBER))
+        *ret = number_from_jsdisp(jsdisp)->value;
+    else
+        return JS_E_NUMBER_EXPECTED;
+    return S_OK;
 }
 
 static inline void number_to_str(double d, WCHAR *buf, int size, int *dec_point)
@@ -222,10 +225,9 @@ static inline jsstr_t *number_to_exponential(double val, int prec)
 }
 
 /* ECMA-262 3rd Edition    15.7.4.2 */
-static HRESULT Number_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Number_toString(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    NumberInstance *number;
     INT radix = 10;
     DOUBLE val;
     jsstr_t *str;
@@ -233,8 +235,9 @@ static HRESULT Number_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
 
     TRACE("\n");
 
-    if(!(number = number_this(jsthis)))
-        return JS_E_NUMBER_EXPECTED;
+    hres = numberval_this(vthis, &val);
+    if(FAILED(hres))
+        return hres;
 
     if(argc) {
         hres = to_int32(ctx, argv[0], &radix);
@@ -245,8 +248,6 @@ static HRESULT Number_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
             return JS_E_INVALIDARG;
     }
 
-    val = number->value;
-
     if(radix==10 || !isfinite(val)) {
         hres = to_string(ctx, jsval_number(val), &str);
         if(FAILED(hres))
@@ -341,17 +342,98 @@ static HRESULT Number_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     return S_OK;
 }
 
-static HRESULT Number_toLocaleString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
-        jsval_t *r)
+HRESULT localize_number(script_ctx_t *ctx, DOUBLE val, BOOL new_format, jsstr_t **ret)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    WCHAR buf[316], decimal[8], thousands[8], *numstr;
+    NUMBERFMTW *format = NULL, format_buf;
+    LCID lcid = ctx->lcid;
+    _locale_t locale;
+    unsigned convlen;
+    jsstr_t *str;
+    int len;
+
+    /* FIXME: Localize this */
+    if(!isfinite(val))
+        return to_string(ctx, jsval_number(val), ret);
+
+    /* Native never uses an exponent, even if the number is very large, it will in fact
+       return all the digits (with thousands separators). jscript.dll uses two digits for
+       fraction even if they are zero (likely default numDigits) and always returns them,
+       while mshtml's jscript uses 3 digits and trims trailing zeros (on same locale).
+       This is even for very small numbers, such as 0.0000999, which will simply be 0. */
+    if(!(locale = _create_locale(LC_ALL, "C")))
+        return E_OUTOFMEMORY;
+    len = _swprintf_l(buf, ARRAY_SIZE(buf), L"%.3f", locale, val);
+    _free_locale(locale);
+
+    if(new_format) {
+        WCHAR grouping[10];
+
+        format = &format_buf;
+        format->NumDigits = 3;
+        while(buf[--len] == '0')
+            format->NumDigits--;
+
+        /* same logic as VarFormatNumber */
+        grouping[2] = '\0';
+        if(!GetLocaleInfoW(lcid, LOCALE_SGROUPING, grouping, ARRAY_SIZE(grouping)))
+            format->Grouping = 3;
+        else
+            format->Grouping = (grouping[2] == '2' ? 32 : grouping[0] - '0');
+
+        if(!GetLocaleInfoW(lcid, LOCALE_ILZERO | LOCALE_RETURN_NUMBER, (WCHAR*)&format->LeadingZero, 2))
+            format->LeadingZero = 0;
+        if(!GetLocaleInfoW(lcid, LOCALE_INEGNUMBER | LOCALE_RETURN_NUMBER, (WCHAR*)&format->NegativeOrder, 2))
+            format->NegativeOrder = 1;
+        format->lpDecimalSep = decimal;
+        if(!GetLocaleInfoW(lcid, LOCALE_SDECIMAL, decimal, ARRAY_SIZE(decimal)))
+            wcscpy(decimal, L".");
+        format->lpThousandSep = thousands;
+        if(!GetLocaleInfoW(lcid, LOCALE_STHOUSAND, thousands, ARRAY_SIZE(thousands)))
+            wcscpy(thousands, L",");
+    }
+
+    if(!(convlen = GetNumberFormatW(lcid, 0, buf, format, NULL, 0)) ||
+       !(str = jsstr_alloc_buf(convlen - 1, &numstr)))
+        return E_OUTOFMEMORY;
+
+    if(!GetNumberFormatW(lcid, 0, buf, format, numstr, convlen)) {
+        jsstr_release(str);
+        return E_OUTOFMEMORY;
+    }
+
+    *ret = str;
+    return S_OK;
 }
 
-static HRESULT Number_toFixed(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Number_toLocaleString(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
+        jsval_t *r)
+{
+    jsstr_t *str;
+    HRESULT hres;
+    DOUBLE val;
+
+    TRACE("\n");
+
+    hres = numberval_this(vthis, &val);
+    if(FAILED(hres)) {
+        if(hres == JS_E_NUMBER_EXPECTED && ctx->version >= SCRIPTLANGUAGEVERSION_ES5)
+            return throw_error(ctx, JS_E_WRONG_THIS, L"Number");
+        return hres;
+    }
+
+    if(r) {
+        hres = localize_number(ctx, val, ctx->version >= SCRIPTLANGUAGEVERSION_ES5, &str);
+        if(FAILED(hres))
+            return hres;
+        *r = jsval_string(str);
+    }
+    return S_OK;
+}
+
+static HRESULT Number_toFixed(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    NumberInstance *number;
     DOUBLE val;
     INT prec = 0;
     jsstr_t *str;
@@ -359,8 +441,9 @@ static HRESULT Number_toFixed(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
 
     TRACE("\n");
 
-    if(!(number = number_this(jsthis)))
-        return JS_E_NUMBER_EXPECTED;
+    hres = numberval_this(vthis, &val);
+    if(FAILED(hres))
+        return hres;
 
     if(argc) {
         hres = to_int32(ctx, argv[0], &prec);
@@ -371,7 +454,6 @@ static HRESULT Number_toFixed(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
             return JS_E_FRACTION_DIGITS_OUT_OF_RANGE;
     }
 
-    val = number->value;
     if(!isfinite(val)) {
         hres = to_string(ctx, jsval_number(val), &str);
         if(FAILED(hres))
@@ -389,10 +471,9 @@ static HRESULT Number_toFixed(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
     return S_OK;
 }
 
-static HRESULT Number_toExponential(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Number_toExponential(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    NumberInstance *number;
     DOUBLE val;
     INT prec = 0;
     jsstr_t *str;
@@ -400,8 +481,9 @@ static HRESULT Number_toExponential(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
 
     TRACE("\n");
 
-    if(!(number = number_this(jsthis)))
-        return JS_E_NUMBER_EXPECTED;
+    hres = numberval_this(vthis, &val);
+    if(FAILED(hres))
+        return hres;
 
     if(argc) {
         hres = to_int32(ctx, argv[0], &prec);
@@ -412,7 +494,6 @@ static HRESULT Number_toExponential(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
             return JS_E_FRACTION_DIGITS_OUT_OF_RANGE;
     }
 
-    val = number->value;
     if(!isfinite(val)) {
         hres = to_string(ctx, jsval_number(val), &str);
         if(FAILED(hres))
@@ -432,17 +513,17 @@ static HRESULT Number_toExponential(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
     return S_OK;
 }
 
-static HRESULT Number_toPrecision(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Number_toPrecision(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    NumberInstance *number;
     INT prec = 0, size;
     jsstr_t *str;
     DOUBLE val;
     HRESULT hres;
 
-    if(!(number = number_this(jsthis)))
-        return JS_E_NUMBER_EXPECTED;
+    hres = numberval_this(vthis, &val);
+    if(FAILED(hres))
+        return hres;
 
     if(argc) {
         hres = to_int32(ctx, argv[0], &prec);
@@ -453,7 +534,6 @@ static HRESULT Number_toPrecision(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
             return JS_E_PRECISION_OUT_OF_RANGE;
     }
 
-    val = number->value;
     if(!isfinite(val) || !prec) {
         hres = to_string(ctx, jsval_number(val), &str);
         if(FAILED(hres))
@@ -479,18 +559,20 @@ static HRESULT Number_toPrecision(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
     return S_OK;
 }
 
-static HRESULT Number_valueOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Number_valueOf(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    NumberInstance *number;
+    HRESULT hres;
+    DOUBLE val;
 
     TRACE("\n");
 
-    if(!(number = number_this(jsthis)))
-        return JS_E_NUMBER_EXPECTED;
+    hres = numberval_this(vthis, &val);
+    if(FAILED(hres))
+        return hres;
 
     if(r)
-        *r = jsval_number(number->value);
+        *r = jsval_number(val);
     return S_OK;
 }
 
@@ -520,7 +602,7 @@ static const builtin_info_t NumberInst_info = {
     NULL
 };
 
-static HRESULT NumberConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT NumberConstr_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     double n;
@@ -555,11 +637,12 @@ static HRESULT NumberConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
             n = 0;
         }
 
-        hres = create_number(ctx, n, &obj);
-        if(FAILED(hres))
-            return hres;
-
-        *r = jsval_obj(obj);
+        if(r) {
+            hres = create_number(ctx, n, &obj);
+            if(FAILED(hres))
+                return hres;
+            *r = jsval_obj(obj);
+        }
         break;
     }
     default:
