@@ -24,11 +24,13 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 
-static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Object_toString(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     jsdisp_t *jsdisp;
     const WCHAR *str;
+    IDispatch *disp;
+    HRESULT hres;
 
     /* Keep in sync with jsclass_t enum */
     static const WCHAR *names[] = {
@@ -54,7 +56,19 @@ static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
 
     TRACE("\n");
 
-    jsdisp = get_jsdisp(jsthis);
+    if(is_undefined(vthis) || is_null(vthis)) {
+        if(ctx->version < SCRIPTLANGUAGEVERSION_ES5)
+            str = L"[object Object]";
+        else
+            str = is_null(vthis) ? L"[object Null]" : L"[object Undefined]";
+        goto set_output;
+    }
+
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
+    jsdisp = to_jsdisp(disp);
     if(!jsdisp) {
         str = L"[object Object]";
     }else if(names[jsdisp->builtin_info->class]) {
@@ -62,9 +76,13 @@ static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     }else {
         assert(jsdisp->builtin_info->class != JSCLASS_NONE);
         FIXME("jsdisp->builtin_info->class = %d\n", jsdisp->builtin_info->class);
-        return E_FAIL;
+        hres = E_FAIL;
     }
+    IDispatch_Release(disp);
+    if(FAILED(hres))
+        return hres;
 
+set_output:
     if(r) {
         jsstr_t *ret;
         ret = jsstr_alloc(str);
@@ -76,34 +94,61 @@ static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     return S_OK;
 }
 
-static HRESULT Object_toLocaleString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Object_toLocaleString(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
+    jsdisp_t *jsdisp;
+    IDispatch *disp;
+    HRESULT hres;
+
     TRACE("\n");
 
-    if(!is_jsdisp(jsthis)) {
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
+    if(!(jsdisp = to_jsdisp(disp))) {
         FIXME("Host object this\n");
-        return E_FAIL;
+        hres = E_FAIL;
+        goto done;
     }
 
-    return jsdisp_call_name(jsthis->u.jsdisp, L"toString", DISPATCH_METHOD, 0, NULL, r);
+    hres = jsdisp_call_name(jsdisp, L"toString", DISPATCH_METHOD, 0, NULL, r);
+done:
+    IDispatch_Release(disp);
+    return hres;
 }
 
-static HRESULT Object_valueOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Object_valueOf(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
+    IDispatch *disp;
+    HRESULT hres;
+
     TRACE("\n");
 
-    if(r) {
-        IDispatch_AddRef(jsthis->u.disp);
-        *r = jsval_disp(jsthis->u.disp);
+    if(is_null_disp(vthis)) {
+        if(r) *r = jsval_null_disp();
+        return S_OK;
     }
+
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
+    if(r)
+        *r = jsval_disp(disp);
+    else
+        IDispatch_Release(disp);
     return S_OK;
 }
 
-static HRESULT Object_hasOwnProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Object_hasOwnProperty(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
+    IDispatchEx *dispex;
+    jsdisp_t *jsdisp;
+    IDispatch *disp;
     jsstr_t *name;
     DISPID id;
     BSTR bstr;
@@ -111,33 +156,39 @@ static HRESULT Object_hasOwnProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
 
     TRACE("\n");
 
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
     if(!argc) {
         if(r)
             *r = jsval_bool(FALSE);
-        return S_OK;
+        goto done;
     }
 
     hres = to_string(ctx, argv[0], &name);
     if(FAILED(hres))
-        return hres;
+        goto done;
 
-    if(is_jsdisp(jsthis)) {
+    if((jsdisp = to_jsdisp(disp))) {
         property_desc_t prop_desc;
         const WCHAR *name_str;
 
         name_str = jsstr_flatten(name);
         if(!name_str) {
             jsstr_release(name);
-            return E_OUTOFMEMORY;
+            hres = E_OUTOFMEMORY;
+            goto done;
         }
 
-        hres = jsdisp_get_own_property(jsthis->u.jsdisp, name_str, TRUE, &prop_desc);
+        hres = jsdisp_get_own_property(jsdisp, name_str, TRUE, &prop_desc);
         jsstr_release(name);
         if(FAILED(hres) && hres != DISP_E_UNKNOWNNAME)
-            return hres;
+            goto done;
 
         if(r) *r = jsval_bool(hres == S_OK);
-        return S_OK;
+        hres = S_OK;
+        goto done;
     }
 
 
@@ -145,81 +196,263 @@ static HRESULT Object_hasOwnProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
     if(bstr)
         jsstr_flush(name, bstr);
     jsstr_release(name);
-    if(!bstr)
-        return E_OUTOFMEMORY;
-
-    if(is_dispex(jsthis))
-        hres = IDispatchEx_GetDispID(jsthis->u.dispex, bstr, make_grfdex(ctx, fdexNameCaseSensitive), &id);
-    else
-        hres = IDispatch_GetIDsOfNames(jsthis->u.disp, &IID_NULL, &bstr, 1, ctx->lcid, &id);
+    if(!bstr) {
+        hres = E_OUTOFMEMORY;
+        goto done;
+    }
 
+    hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
+    if(SUCCEEDED(hres)) {
+        hres = IDispatchEx_GetDispID(dispex, bstr, make_grfdex(ctx, fdexNameCaseSensitive), &id);
+        IDispatchEx_Release(dispex);
+    }else {
+        hres = IDispatch_GetIDsOfNames(disp, &IID_NULL, &bstr, 1, ctx->lcid, &id);
+    }
     SysFreeString(bstr);
     if(r)
         *r = jsval_bool(SUCCEEDED(hres));
-    return S_OK;
+    hres = S_OK;
+done:
+    IDispatch_Release(disp);
+    return hres;
 }
 
-static HRESULT Object_propertyIsEnumerable(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Object_propertyIsEnumerable(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     property_desc_t prop_desc;
     const WCHAR *name;
     jsstr_t *name_str;
+    jsdisp_t *jsdisp;
+    IDispatch *disp;
     HRESULT hres;
 
     TRACE("\n");
 
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
     if(argc != 1) {
         FIXME("argc %d not supported\n", argc);
-        return E_NOTIMPL;
+        hres = E_NOTIMPL;
+        goto done;
     }
 
-    if(!is_jsdisp(jsthis)) {
+    if(!(jsdisp = to_jsdisp(disp))) {
         FIXME("Host object this\n");
-        return E_FAIL;
+        hres = E_FAIL;
+        goto done;
     }
 
+    hres = to_flat_string(ctx, argv[0], &name_str, &name);
+    if(FAILED(hres))
+        goto done;
+
+    hres = jsdisp_get_own_property(jsdisp, name, TRUE, &prop_desc);
+    jsstr_release(name_str);
+    if(FAILED(hres) && hres != DISP_E_UNKNOWNNAME)
+        goto done;
+
+    if(r)
+        *r = jsval_bool(hres == S_OK && (prop_desc.flags & PROPF_ENUMERABLE) != 0);
+    hres = S_OK;
+done:
+    IDispatch_Release(disp);
+    return hres;
+}
+
+static HRESULT Object_isPrototypeOf(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
+        jsval_t *r)
+{
+    jsdisp_t *jsthis, *jsdisp;
+    IDispatch *disp;
+    BOOL ret = FALSE;
+    HRESULT hres;
+
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
+    if(!r)
+        goto done;
+
+    if(argc && (jsthis = to_jsdisp(disp)) && is_object_instance(argv[0]) &&
+       (jsdisp = to_jsdisp(get_object(argv[0])))) {
+        while(jsdisp->prototype) {
+            if(jsdisp->prototype == jsthis) {
+                ret = TRUE;
+                break;
+            }
+            jsdisp = jsdisp->prototype;
+        }
+    }
+
+    *r = jsval_bool(ret);
+done:
+    IDispatch_Release(disp);
+    return hres;
+}
+
+static HRESULT Object_defineGetter(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    property_desc_t desc;
+    const WCHAR *name;
+    jsstr_t *name_str;
+    jsdisp_t *jsthis;
+    HRESULT hres;
+
+    TRACE("\n");
+
+    if(!is_object_instance(vthis) || !(jsthis = to_jsdisp(get_object(vthis))))
+        goto done;
+
+    if(argc < 2 || !is_object_instance(argv[1]))
+        return JS_E_FUNCTION_EXPECTED;
+
+    desc.getter = to_jsdisp(get_object(argv[1]));
+    if(!desc.getter) {
+        FIXME("getter is not JS object\n");
+        return E_NOTIMPL;
+    }
+    /* FIXME: Check IsCallable */
+
     hres = to_flat_string(ctx, argv[0], &name_str, &name);
     if(FAILED(hres))
         return hres;
 
-    hres = jsdisp_get_own_property(jsthis->u.jsdisp, name, TRUE, &prop_desc);
-    jsstr_release(name_str);
-    if(FAILED(hres) && hres != DISP_E_UNKNOWNNAME)
-        return hres;
+    desc.flags = desc.mask = PROPF_CONFIGURABLE | PROPF_ENUMERABLE;
+    desc.explicit_getter = TRUE;
+    desc.explicit_setter = FALSE;
+    desc.explicit_value  = FALSE;
+    desc.setter = NULL;
+    hres = jsdisp_define_property(jsthis, name, &desc);
 
+    jsstr_release(name_str);
+    if(FAILED(hres))
+        return hres;
+done:
     if(r)
-        *r = jsval_bool(hres == S_OK && (prop_desc.flags & PROPF_ENUMERABLE) != 0);
+        *r = jsval_undefined();
     return S_OK;
 }
 
-static HRESULT Object_isPrototypeOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
-        jsval_t *r)
+static HRESULT Object_defineSetter(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
-}
-
-static HRESULT Object_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
-{
-    jsstr_t *ret;
+    property_desc_t desc;
+    const WCHAR *name;
+    jsstr_t *name_str;
+    jsdisp_t *jsthis;
+    HRESULT hres;
 
     TRACE("\n");
 
-    ret = jsstr_alloc(L"[object Object]");
-    if(!ret)
-        return E_OUTOFMEMORY;
+    if(!is_object_instance(vthis) || !(jsthis = to_jsdisp(get_object(vthis))))
+        goto done;
 
-    *r = jsval_string(ret);
+    if(argc < 2 || !is_object_instance(argv[1]))
+        return JS_E_FUNCTION_EXPECTED;
+
+    desc.setter = to_jsdisp(get_object(argv[1]));
+    if(!desc.setter) {
+        FIXME("setter is not JS object\n");
+        return E_NOTIMPL;
+    }
+    /* FIXME: Check IsCallable */
+
+    hres = to_flat_string(ctx, argv[0], &name_str, &name);
+    if(FAILED(hres))
+        return hres;
+
+    desc.flags = desc.mask = PROPF_CONFIGURABLE | PROPF_ENUMERABLE;
+    desc.explicit_getter = FALSE;
+    desc.explicit_setter = TRUE;
+    desc.explicit_value  = FALSE;
+    desc.getter = NULL;
+    hres = jsdisp_define_property(jsthis, name, &desc);
+
+    jsstr_release(name_str);
+    if(FAILED(hres))
+        return hres;
+done:
+    if(r)
+        *r = jsval_undefined();
     return S_OK;
 }
 
+HRESULT Object_get_proto_(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    jsdisp_t *jsthis;
+    IDispatch *disp;
+    HRESULT hres;
+
+    TRACE("%s\n", debugstr_jsval(vthis));
+
+    hres = to_object(ctx, vthis, &disp);
+    if(FAILED(hres))
+        return hres;
+
+    if(!r)
+        goto done;
+
+    if(!(jsthis = to_jsdisp(disp))) {
+        FIXME("Host object this\n");
+        hres = E_FAIL;
+        goto done;
+    }
+
+    *r = jsthis->prototype
+        ? jsval_obj(jsdisp_addref(jsthis->prototype))
+        : jsval_null();
+done:
+    IDispatch_Release(disp);
+    return hres;
+}
+
+HRESULT Object_set_proto_(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    jsdisp_t *jsthis, *proto;
+    HRESULT hres;
+
+    TRACE("%s\n", debugstr_jsval(vthis));
+
+    if(is_undefined(vthis) || is_null(vthis))
+        return JS_E_OBJECT_EXPECTED;
+    if(!argc) {
+        if(r)
+            *r = jsval_undefined();
+        return S_OK;
+    }
+    if(!is_object_instance(vthis) || !(jsthis = to_jsdisp(get_object(vthis))))
+        goto done;
+
+    if(is_null(argv[0])) {
+        proto = NULL;
+    }else if(is_object_instance(argv[0])) {
+        proto = to_jsdisp(get_object(argv[0]));
+        if(!proto) {
+            FIXME("Host object value\n");
+            return E_FAIL;
+        }
+    }else
+        goto done;
+
+    hres = jsdisp_change_prototype(jsthis, proto);
+    if(FAILED(hres))
+        return hres;
+
+done:
+    return r ? jsval_copy(argv[0], r) : S_OK;
+}
+
 static void Object_destructor(jsdisp_t *dispex)
 {
     heap_free(dispex);
 }
 
 static const builtin_prop_t Object_props[] = {
+    {L"__defineGetter__",      Object_defineGetter,          PROPF_METHOD|PROPF_ES6|2},
+    {L"__defineSetter__",      Object_defineSetter,          PROPF_METHOD|PROPF_ES6|2},
     {L"hasOwnProperty",        Object_hasOwnProperty,        PROPF_METHOD|1},
     {L"isPrototypeOf",         Object_isPrototypeOf,         PROPF_METHOD|1},
     {L"propertyIsEnumerable",  Object_propertyIsEnumerable,  PROPF_METHOD|1},
@@ -230,7 +463,7 @@ static const builtin_prop_t Object_props[] = {
 
 static const builtin_info_t Object_info = {
     JSCLASS_OBJECT,
-    {NULL, NULL,0, Object_get_value},
+    NULL,
     ARRAY_SIZE(Object_props),
     Object_props,
     Object_destructor,
@@ -239,7 +472,7 @@ static const builtin_info_t Object_info = {
 
 static const builtin_info_t ObjectInst_info = {
     JSCLASS_OBJECT,
-    {NULL, NULL,0, Object_get_value},
+    NULL,
     0, NULL,
     Object_destructor,
     NULL
@@ -414,7 +647,7 @@ static HRESULT jsdisp_define_properties(script_ctx_t *ctx, jsdisp_t *obj, jsval_
         if(FAILED(hres))
             break;
 
-        if(!is_object_instance(desc_val) || !get_object(desc_val) || !(desc_obj = to_jsdisp(get_object(desc_val)))) {
+        if(!is_object_instance(desc_val) || !(desc_obj = to_jsdisp(get_object(desc_val)))) {
             jsval_release(desc_val);
             break;
         }
@@ -436,7 +669,7 @@ static HRESULT jsdisp_define_properties(script_ctx_t *ctx, jsdisp_t *obj, jsval_
     return FAILED(hres) ? hres : S_OK;
 }
 
-static HRESULT Object_defineProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_defineProperty(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                                      unsigned argc, jsval_t *argv, jsval_t *r)
 {
     property_desc_t prop_desc;
@@ -485,13 +718,13 @@ static HRESULT Object_defineProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
     return hres;
 }
 
-static HRESULT Object_defineProperties(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_defineProperties(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                                      unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
     HRESULT hres;
 
-    if(argc < 1 || !is_object_instance(argv[0]) || !get_object(argv[0]) || !(obj = to_jsdisp(get_object(argv[0])))) {
+    if(argc < 1 || !is_object_instance(argv[0]) || !(obj = to_jsdisp(get_object(argv[0])))) {
         FIXME("not an object\n");
         return E_NOTIMPL;
     }
@@ -504,7 +737,7 @@ static HRESULT Object_defineProperties(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     return hres;
 }
 
-static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                                                unsigned argc, jsval_t *argv, jsval_t *r)
 {
     property_desc_t prop_desc;
@@ -567,7 +800,7 @@ static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthi
     return hres;
 }
 
-static HRESULT Object_create(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_create(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                              unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *proto = NULL, *obj;
@@ -606,26 +839,19 @@ static HRESULT Object_create(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     return hres;
 }
 
-static HRESULT Object_getPrototypeOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_getPrototypeOf(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                                      unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0])) {
-        FIXME("invalid arguments\n");
-        return E_NOTIMPL;
-    }
+    if(!argc || !is_object_instance(argv[0]))
+        return JS_E_OBJECT_EXPECTED;
 
     TRACE("(%s)\n", debugstr_jsval(argv[0]));
 
     obj = to_jsdisp(get_object(argv[0]));
-    if(!obj) {
-        FIXME("Non-JS object\n");
-        return E_NOTIMPL;
-    }
-
     if(r)
-        *r = obj->prototype
+        *r = obj && obj->prototype
             ? jsval_obj(jsdisp_addref(obj->prototype))
             : jsval_null();
     return S_OK;
@@ -639,10 +865,8 @@ static HRESULT object_keys(script_ctx_t *ctx, jsval_t arg, enum jsdisp_enum_type
     jsstr_t *key;
     HRESULT hres;
 
-    if(!is_object_instance(arg) || !get_object(arg)) {
-        FIXME("invalid arguments %s\n", debugstr_jsval(arg));
-        return E_NOTIMPL;
-    }
+    if(!is_object_instance(arg))
+        return JS_E_OBJECT_EXPECTED;
 
     obj = to_jsdisp(get_object(arg));
     if(!obj) {
@@ -674,7 +898,7 @@ static HRESULT object_keys(script_ctx_t *ctx, jsval_t arg, enum jsdisp_enum_type
     return hres;
 }
 
-static HRESULT Object_keys(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_keys(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                            unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsval_t arg = argc ? argv[0] : jsval_undefined();
@@ -684,7 +908,7 @@ static HRESULT Object_keys(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     return object_keys(ctx, arg, JSDISP_ENUM_OWN_ENUMERABLE, r);
 }
 
-static HRESULT Object_getOwnPropertyNames(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+static HRESULT Object_getOwnPropertyNames(script_ctx_t *ctx, jsval_t vthis, WORD flags,
                                           unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsval_t arg = argc ? argv[0] : jsval_undefined();
@@ -694,14 +918,12 @@ static HRESULT Object_getOwnPropertyNames(script_ctx_t *ctx, vdisp_t *jsthis, WO
     return object_keys(ctx, arg, JSDISP_ENUM_OWN, r);
 }
 
-static HRESULT Object_preventExtensions(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+static HRESULT Object_preventExtensions(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0]) || !get_object(argv[0])) {
-        FIXME("invalid arguments\n");
-        return E_NOTIMPL;
-    }
+    if(!argc || !is_object_instance(argv[0]))
+        return JS_E_OBJECT_EXPECTED;
 
     TRACE("(%s)\n", debugstr_jsval(argv[0]));
 
@@ -716,12 +938,12 @@ static HRESULT Object_preventExtensions(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     return S_OK;
 }
 
-static HRESULT Object_freeze(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc,
+static HRESULT Object_freeze(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc,
                              jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0]) || !get_object(argv[0])) {
+    if(!argc || !is_object_instance(argv[0])) {
         WARN("argument is not an object\n");
         return JS_E_OBJECT_EXPECTED;
     }
@@ -739,12 +961,12 @@ static HRESULT Object_freeze(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     return S_OK;
 }
 
-static HRESULT Object_seal(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc,
+static HRESULT Object_seal(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc,
                            jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0]) || !get_object(argv[0])) {
+    if(!argc || !is_object_instance(argv[0])) {
         WARN("argument is not an object\n");
         return JS_E_OBJECT_EXPECTED;
     }
@@ -762,11 +984,11 @@ static HRESULT Object_seal(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsig
     return S_OK;
 }
 
-static HRESULT Object_isExtensible(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+static HRESULT Object_isExtensible(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0]) || !get_object(argv[0])) {
+    if(!argc || !is_object_instance(argv[0])) {
         WARN("argument is not an object\n");
         return JS_E_OBJECT_EXPECTED;
     }
@@ -783,12 +1005,12 @@ static HRESULT Object_isExtensible(script_ctx_t *ctx, vdisp_t *jsthis, WORD flag
     return S_OK;
 }
 
-static HRESULT Object_isFrozen(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc,
+static HRESULT Object_isFrozen(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc,
                                jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0]) || !get_object(argv[0])) {
+    if(!argc || !is_object_instance(argv[0])) {
         WARN("argument is not an object\n");
         return JS_E_OBJECT_EXPECTED;
     }
@@ -805,12 +1027,12 @@ static HRESULT Object_isFrozen(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     return S_OK;
 }
 
-static HRESULT Object_isSealed(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc,
+static HRESULT Object_isSealed(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc,
                                jsval_t *argv, jsval_t *r)
 {
     jsdisp_t *obj;
 
-    if(!argc || !is_object_instance(argv[0]) || !get_object(argv[0])) {
+    if(!argc || !is_object_instance(argv[0])) {
         WARN("argument is not an object\n");
         return JS_E_OBJECT_EXPECTED;
     }
@@ -845,14 +1067,14 @@ static const builtin_prop_t ObjectConstr_props[] = {
 
 static const builtin_info_t ObjectConstr_info = {
     JSCLASS_FUNCTION,
-    DEFAULT_FUNCTION_VALUE,
+    Function_value,
     ARRAY_SIZE(ObjectConstr_props),
     ObjectConstr_props,
     NULL,
     NULL
 };
 
-static HRESULT ObjectConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT ObjectConstr_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     HRESULT hres;
@@ -865,7 +1087,7 @@ static HRESULT ObjectConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
         jsdisp_t *obj;
 
         if(argc) {
-            if(!is_undefined(argv[0]) && !is_null(argv[0]) && (!is_object_instance(argv[0]) || get_object(argv[0]))) {
+            if(!is_undefined(argv[0]) && !is_null(argv[0])) {
                 IDispatch *disp;
 
                 hres = to_object(ctx, argv[0], &disp);
