@@ -77,6 +77,40 @@ static const SecPkgInfoW infoW =
 static ULONG kerberos_package_id;
 static LSA_DISPATCH_TABLE lsa_dispatch;
 
+struct cred_handle
+{
+    UINT64 handle;
+};
+
+struct context_handle
+{
+    UINT64 handle;
+};
+
+static LSA_SEC_HANDLE create_context_handle( struct context_handle *ctx, UINT64 new_context )
+{
+    UINT64 context = ctx ? ctx->handle : 0;
+    if (new_context && new_context != context)
+    {
+        struct context_handle *new_ctx = malloc(sizeof(*new_ctx));
+        new_ctx->handle = new_context;
+        return (LSA_SEC_HANDLE)new_ctx;
+    }
+    else
+        return (LSA_SEC_HANDLE)ctx;
+}
+
+static int get_buffer_index( const SecBufferDesc *desc, DWORD type )
+{
+    UINT i;
+    if (!desc) return -1;
+    for (i = 0; i < desc->cBuffers; i++)
+    {
+        if (desc->pBuffers[i].BufferType == type) return i;
+    }
+    return -1;
+}
+
 static const char *debugstr_us( const UNICODE_STRING *us )
 {
     if (!us) return "<null>";
@@ -285,6 +319,7 @@ static NTSTATUS NTAPI kerberos_SpAcquireCredentialsHandle(
     char *principal = NULL, *username = NULL,  *password = NULL;
     SEC_WINNT_AUTH_IDENTITY_W *id = auth_data;
     NTSTATUS status = SEC_E_INSUFFICIENT_MEMORY;
+    struct cred_handle *cred_handle;
     ULONG exptime;
 
     TRACE( "%s, %#lx, %p, %p, %p, %p, %p, %p\n", debugstr_us(principal_us), credential_use,
@@ -303,10 +338,19 @@ static NTSTATUS NTAPI kerberos_SpAcquireCredentialsHandle(
         if (!(password = get_password_unixcp( id->Password, id->PasswordLength ))) goto done;
     }
 
+    if (!(cred_handle = calloc( 1, sizeof(*cred_handle) )))
+    {
+        status = SEC_E_INSUFFICIENT_MEMORY;
+        goto done;
+    }
+
     {
         struct acquire_credentials_handle_params params = { principal, credential_use, username, password,
-                                                            credential, &exptime };
-        status = KRB5_CALL( acquire_credentials_handle, &params );
+                                                            &cred_handle->handle, &exptime };
+        if (!(status = KRB5_CALL( acquire_credentials_handle, &params )))
+            *credential = (LSA_SEC_HANDLE)cred_handle;
+        else
+            free( cred_handle );
         expiry_to_timestamp( exptime, expiry );
     }
 
@@ -319,9 +363,18 @@ done:
 
 static NTSTATUS NTAPI kerberos_SpFreeCredentialsHandle( LSA_SEC_HANDLE credential )
 {
+    struct cred_handle *cred_handle = (void *)credential;
+    struct free_credentials_handle_params params;
+    NTSTATUS status;
+
     TRACE( "%Ix\n", credential );
-    if (!credential) return SEC_E_INVALID_HANDLE;
-    return KRB5_CALL( free_credentials_handle, (void *)credential );
+
+    if (!cred_handle) return SEC_E_INVALID_HANDLE;
+
+    params.credential = cred_handle->handle;
+    status = KRB5_CALL( free_credentials_handle, &params );
+    free(cred_handle);
+    return status;
 }
 
 static NTSTATUS NTAPI kerberos_SpInitLsaModeContext( LSA_SEC_HANDLE credential, LSA_SEC_HANDLE context,
@@ -345,9 +398,34 @@ static NTSTATUS NTAPI kerberos_SpInitLsaModeContext( LSA_SEC_HANDLE credential,
     if (target_name && !(target = get_str_unixcp( target_name ))) return SEC_E_INSUFFICIENT_MEMORY;
     else
     {
-        struct initialize_context_params params = { credential, context, target, context_req, input,
-                                                    new_context, output, context_attr, &exptime };
+        struct cred_handle *cred_handle = (struct cred_handle *)credential;
+        struct context_handle *context_handle = (struct context_handle *)context;
+        struct initialize_context_params params = { 0 };
+        UINT64 new_context_handle = 0;
+        int idx;
+
+        params.credential = cred_handle ? cred_handle->handle : 0;
+        params.context = context_handle ? context_handle->handle : 0;
+        params.target_name = target;
+        params.context_req = context_req;
+        params.new_context = &new_context_handle;
+        params.context_attr = context_attr;
+        params.expiry = &exptime;
+
+        idx = get_buffer_index( input, SECBUFFER_TOKEN );
+        if (idx != -1)
+        {
+            params.input_token = input->pBuffers[idx].pvBuffer;
+            params.input_token_length = input->pBuffers[idx].cbBuffer;
+        }
+
+        if ((idx = get_buffer_index( output, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+        params.output_token = output->pBuffers[idx].pvBuffer;
+        params.output_token_length = &output->pBuffers[idx].cbBuffer;
+
         status = KRB5_CALL( initialize_context, &params );
+        if (status == SEC_E_OK || status == SEC_I_CONTINUE_NEEDED)
+            *new_context = create_context_handle( context_handle, new_context_handle );
         if (!status)
         {
             *mapped_context = TRUE;
@@ -365,6 +443,7 @@ static NTSTATUS NTAPI kerberos_SpAcceptLsaModeContext( LSA_SEC_HANDLE credential
 {
     NTSTATUS status = SEC_E_INVALID_HANDLE;
     ULONG exptime;
+    int idx;
 
     TRACE( "%Ix, %Ix, %#lx, %lu, %p, %p, %p, %p, %p, %p, %p\n", credential, context, context_req, target_data_rep,
            input, new_context, output, context_attr, expiry, mapped_context, context_data );
@@ -372,8 +451,31 @@ static NTSTATUS NTAPI kerberos_SpAcceptLsaModeContext( LSA_SEC_HANDLE credential
 
     if (context || input || credential)
     {
-        struct accept_context_params params = { credential, context, input, new_context, output, context_attr, &exptime };
+        struct cred_handle *cred_handle = (struct cred_handle *)credential;
+        struct context_handle *context_handle = (struct context_handle *)context;
+        struct accept_context_params params = { 0 };
+        UINT64 new_context_handle = 0;
+
+        params.credential = cred_handle ? cred_handle->handle : 0;
+        params.context = context_handle ? context_handle->handle : 0;
+        params.new_context = &new_context_handle;
+        params.context_attr = context_attr;
+        params.expiry = &exptime;
+
+        if (input)
+        {
+            if ((idx = get_buffer_index( input, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+            params.input_token  = input->pBuffers[idx].pvBuffer;
+            params.input_token_length = input->pBuffers[idx].cbBuffer;
+        }
+        if ((idx = get_buffer_index( output, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+        params.output_token = output->pBuffers[idx].pvBuffer;
+        params.output_token_length = &output->pBuffers[idx].cbBuffer;
+
+        /* FIXME: check if larger output buffer exists */
         status = KRB5_CALL( accept_context, &params );
+        if (status == SEC_E_OK || status == SEC_I_CONTINUE_NEEDED)
+            *new_context = create_context_handle( context_handle, new_context_handle );
         if (!status)
         {
             *mapped_context = TRUE;
@@ -386,9 +488,18 @@ static NTSTATUS NTAPI kerberos_SpAcceptLsaModeContext( LSA_SEC_HANDLE credential
 
 static NTSTATUS NTAPI kerberos_SpDeleteContext( LSA_SEC_HANDLE context )
 {
+    struct context_handle *context_handle = (void *)context;
+    struct delete_context_params params;
+    NTSTATUS status;
+
     TRACE( "%Ix\n", context );
+
     if (!context) return SEC_E_INVALID_HANDLE;
-    return KRB5_CALL( delete_context, (void *)context );
+
+    params.context = context_handle->handle;
+    status = KRB5_CALL( delete_context, &params );
+    free( context_handle );
+    return status;
 }
 
 static SecPkgInfoW *build_package_info( const SecPkgInfoW *info )
@@ -411,6 +522,8 @@ static SecPkgInfoW *build_package_info( const SecPkgInfoW *info )
 
 static NTSTATUS NTAPI kerberos_SpQueryContextAttributes( LSA_SEC_HANDLE context, ULONG attribute, void *buffer )
 {
+    struct context_handle *context_handle = (void *)context;
+
     TRACE( "%Ix, %lu, %p\n", context, attribute, buffer );
 
     if (!context) return SEC_E_INVALID_HANDLE;
@@ -432,7 +545,7 @@ static NTSTATUS NTAPI kerberos_SpQueryContextAttributes( LSA_SEC_HANDLE context,
     X(SECPKG_ATTR_TARGET_INFORMATION);
     case SECPKG_ATTR_SIZES:
     {
-        struct query_context_attributes_params params = { context, attribute, buffer };
+        struct query_context_attributes_params params = { context_handle->handle, attribute, buffer };
         return KRB5_CALL( query_context_attributes, &params );
     }
     case SECPKG_ATTR_NEGOTIATION_INFO:
@@ -539,7 +652,20 @@ static NTSTATUS SEC_ENTRY kerberos_SpMakeSignature( LSA_SEC_HANDLE context, ULON
 
     if (context)
     {
-        struct make_signature_params params = { context, message };
+        struct context_handle *context_handle = (void *)context;
+        struct make_signature_params params;
+        int data_idx, token_idx;
+
+        /* FIXME: multiple data buffers, read-only buffers */
+        if ((data_idx = get_buffer_index( message, SECBUFFER_DATA )) == -1) return SEC_E_INVALID_TOKEN;
+        if ((token_idx = get_buffer_index( message, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+
+        params.context = context_handle->handle;
+        params.data_length = message->pBuffers[data_idx].cbBuffer;
+        params.data = message->pBuffers[data_idx].pvBuffer;
+        params.token_length = &message->pBuffers[token_idx].cbBuffer;
+        params.token = message->pBuffers[token_idx].pvBuffer;
+
         return KRB5_CALL( make_signature, &params );
     }
     else return SEC_E_INVALID_HANDLE;
@@ -553,7 +679,20 @@ static NTSTATUS NTAPI kerberos_SpVerifySignature( LSA_SEC_HANDLE context, SecBuf
 
     if (context)
     {
-        struct verify_signature_params params = { context, message, quality_of_protection };
+        struct context_handle *context_handle = (void *)context;
+        struct verify_signature_params params;
+        int data_idx, token_idx;
+
+        if ((data_idx = get_buffer_index( message, SECBUFFER_DATA )) == -1) return SEC_E_INVALID_TOKEN;
+        if ((token_idx = get_buffer_index( message, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+
+        params.context = context_handle->handle;
+        params.data_length = message->pBuffers[data_idx].cbBuffer;
+        params.data = message->pBuffers[data_idx].pvBuffer;
+        params.token_length = message->pBuffers[token_idx].cbBuffer;
+        params.token = message->pBuffers[token_idx].pvBuffer;
+        params.qop = quality_of_protection;
+
         return KRB5_CALL( verify_signature, &params );
     }
     else return SEC_E_INVALID_HANDLE;
@@ -567,7 +706,21 @@ static NTSTATUS NTAPI kerberos_SpSealMessage( LSA_SEC_HANDLE context, ULONG qual
 
     if (context)
     {
-        struct seal_message_params params = { context, message, quality_of_protection };
+        struct context_handle *context_handle = (void *)context;
+        struct seal_message_params params;
+        int data_idx, token_idx;
+
+        /* FIXME: multiple data buffers, read-only buffers */
+        if ((data_idx = get_buffer_index( message, SECBUFFER_DATA )) == -1) return SEC_E_INVALID_TOKEN;
+        if ((token_idx = get_buffer_index( message, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+
+        params.context = context_handle->handle;
+        params.data_length = message->pBuffers[data_idx].cbBuffer;
+        params.data = message->pBuffers[data_idx].pvBuffer;
+        params.token_length = &message->pBuffers[token_idx].cbBuffer;
+        params.token = message->pBuffers[token_idx].pvBuffer;
+        params.qop = quality_of_protection;
+
         return KRB5_CALL( seal_message, &params );
     }
     else return SEC_E_INVALID_HANDLE;
@@ -581,7 +734,20 @@ static NTSTATUS NTAPI kerberos_SpUnsealMessage( LSA_SEC_HANDLE context, SecBuffe
 
     if (context)
     {
-        struct unseal_message_params params = { context, message, quality_of_protection };
+        struct context_handle *context_handle = (void *)context;
+        struct unseal_message_params params;
+        int data_idx, token_idx;
+
+        if ((data_idx = get_buffer_index( message, SECBUFFER_DATA )) == -1) return SEC_E_INVALID_TOKEN;
+        if ((token_idx = get_buffer_index( message, SECBUFFER_TOKEN )) == -1) return SEC_E_INVALID_TOKEN;
+
+        params.context = context_handle->handle;
+        params.data_length = message->pBuffers[data_idx].cbBuffer;
+        params.data = message->pBuffers[data_idx].pvBuffer;
+        params.token_length = message->pBuffers[token_idx].cbBuffer;
+        params.token = message->pBuffers[token_idx].pvBuffer;
+        params.qop = quality_of_protection;
+
         return KRB5_CALL( unseal_message, &params );
     }
     else return SEC_E_INVALID_HANDLE;
