@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -279,7 +278,7 @@ static HRESULT WINAPI HTMLObjectElement_put_width(IHTMLObjectElement *iface, VAR
     nsres = nsIDOMHTMLObjectElement_SetWidth(This->nsobject, &width_str);
     nsAString_Finish(&width_str);
     if(NS_FAILED(nsres)) {
-        FIXME("SetWidth failed: %08x\n", nsres);
+        FIXME("SetWidth failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -306,7 +305,7 @@ static HRESULT WINAPI HTMLObjectElement_get_width(IHTMLObjectElement *iface, VAR
         V_BSTR(p) = SysAllocString(width);
         hres = V_BSTR(p) ? S_OK : E_OUTOFMEMORY;
     }else {
-        ERR("GetWidth failed: %08x\n", nsres);
+        ERR("GetWidth failed: %08lx\n", nsres);
         hres = E_FAIL;
     }
 
@@ -337,7 +336,7 @@ static HRESULT WINAPI HTMLObjectElement_put_height(IHTMLObjectElement *iface, VA
     nsres = nsIDOMHTMLObjectElement_SetHeight(This->nsobject, &height_str);
     nsAString_Finish(&height_str);
     if(NS_FAILED(nsres)) {
-        FIXME("SetHeight failed: %08x\n", nsres);
+        FIXME("SetHeight failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -364,7 +363,7 @@ static HRESULT WINAPI HTMLObjectElement_get_height(IHTMLObjectElement *iface, VA
         V_BSTR(p) = SysAllocString(height);
         hres = V_BSTR(p) ? S_OK : E_OUTOFMEMORY;
     }else {
-        ERR("GetHeight failed: %08x\n", nsres);
+        ERR("GetHeight failed: %08lx\n", nsres);
         hres = E_FAIL;
     }
 
@@ -424,7 +423,7 @@ static HRESULT WINAPI HTMLObjectElement_get_altHtml(IHTMLObjectElement *iface, B
 static HRESULT WINAPI HTMLObjectElement_put_vspace(IHTMLObjectElement *iface, LONG v)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -437,7 +436,7 @@ static HRESULT WINAPI HTMLObjectElement_get_vspace(IHTMLObjectElement *iface, LO
 
     nsres = nsIDOMHTMLObjectElement_GetVspace(This->nsobject, p);
     if(NS_FAILED(nsres)) {
-        ERR("GetVspace failed: %08x\n", nsres);
+        ERR("GetVspace failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -447,7 +446,7 @@ static HRESULT WINAPI HTMLObjectElement_get_vspace(IHTMLObjectElement *iface, LO
 static HRESULT WINAPI HTMLObjectElement_put_hspace(IHTMLObjectElement *iface, LONG v)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -705,17 +704,26 @@ static HRESULT HTMLObjectElement_get_dispid(HTMLDOMNode *iface, BSTR name,
 {
     HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
 
-    TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(name), grfdex, pid);
+    TRACE("(%p)->(%s %lx %p)\n", This, debugstr_w(name), grfdex, pid);
 
     return get_plugin_dispid(&This->plugin_container, name, pid);
 }
 
+static HRESULT HTMLObjectElement_dispex_get_name(HTMLDOMNode *iface, DISPID id, BSTR *name)
+{
+    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
+
+    FIXME("(%p)->(%lx %p)\n", This, id, name);
+
+    return E_NOTIMPL;
+}
+
 static HRESULT HTMLObjectElement_invoke(HTMLDOMNode *iface, DISPID id, LCID lcid,
         WORD flags, DISPPARAMS *params, VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
     HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
 
-    TRACE("(%p)->(%d)\n", This, id);
+    TRACE("(%p)->(%ld)\n", This, id);
 
     return invoke_plugin_prop(&This->plugin_container, id, lcid, flags, params, res, ei);
 }
@@ -754,6 +762,7 @@ static const NodeImplVtbl HTMLObjectElementImplVtbl = {
     NULL,
     HTMLObjectElement_get_readystate,
     HTMLObjectElement_get_dispid,
+    HTMLObjectElement_dispex_get_name,
     HTMLObjectElement_invoke,
     NULL,
     HTMLObjectElement_traverse,
@@ -767,6 +776,7 @@ static const tid_t HTMLObjectElement_iface_tids[] = {
     0
 };
 static dispex_static_data_t HTMLObjectElement_dispex = {
+    L"HTMLObjectElement",
     NULL,
     DispHTMLObjectElement_tid,
     HTMLObjectElement_iface_tids,
@@ -1029,6 +1039,7 @@ static const tid_t HTMLEmbedElement_iface_tids[] = {
     0
 };
 static dispex_static_data_t HTMLEmbedElement_dispex = {
+    L"HTMLEmbedElement",
     NULL,
     DispHTMLEmbed_tid,
     HTMLEmbedElement_iface_tids,
