@@ -419,19 +419,19 @@ static inline HTMLDocumentObj *impl_from_IOleDocumentView(IOleDocumentView *ifac
 static HRESULT WINAPI OleDocumentView_QueryInterface(IOleDocumentView *iface, REFIID riid, void **ppvObject)
 {
     HTMLDocumentObj *This = impl_from_IOleDocumentView(iface);
-    return htmldoc_query_interface(&This->basedoc, riid, ppvObject);
+    return IUnknown_QueryInterface(This->outer_unk, riid, ppvObject);
 }
 
 static ULONG WINAPI OleDocumentView_AddRef(IOleDocumentView *iface)
 {
     HTMLDocumentObj *This = impl_from_IOleDocumentView(iface);
-    return htmldoc_addref(&This->basedoc);
+    return IUnknown_AddRef(This->outer_unk);
 }
 
 static ULONG WINAPI OleDocumentView_Release(IOleDocumentView *iface)
 {
     HTMLDocumentObj *This = impl_from_IOleDocumentView(iface);
-    return htmldoc_release(&This->basedoc);
+    return IUnknown_Release(This->outer_unk);
 }
 
 static HRESULT WINAPI OleDocumentView_SetInPlaceSite(IOleDocumentView *iface, IOleInPlaceSite *pIPSite)
@@ -473,7 +473,7 @@ static HRESULT WINAPI OleDocumentView_GetDocument(IOleDocumentView *iface, IUnkn
     if(!ppunk)
         return E_INVALIDARG;
 
-    *ppunk = (IUnknown*)&This->basedoc.IHTMLDocument2_iface;
+    *ppunk = (IUnknown*)&This->IHTMLDocument2_iface;
     IUnknown_AddRef(*ppunk);
     return S_OK;
 }
@@ -541,7 +541,7 @@ static HRESULT WINAPI OleDocumentView_Show(IOleDocumentView *iface, BOOL fShow)
         ShowWindow(This->hwnd, SW_HIDE);
 
         if(This->in_place_active)
-            IOleInPlaceObjectWindowless_InPlaceDeactivate(&This->basedoc.IOleInPlaceObjectWindowless_iface);
+            IOleInPlaceObjectWindowless_InPlaceDeactivate(&This->IOleInPlaceObjectWindowless_iface);
 
         if(This->ip_window) {
             IOleInPlaceUIWindow_Release(This->ip_window);
@@ -612,8 +612,7 @@ static HRESULT WINAPI OleDocumentView_UIActivate(IOleDocumentView *iface, BOOL f
 
         hres = IOleInPlaceSite_OnUIActivate(This->ipsite);
         if(SUCCEEDED(hres)) {
-            call_set_active_object((IOleInPlaceUIWindow*)This->frame,
-                    &This->basedoc.IOleInPlaceActiveObject_iface);
+            call_set_active_object((IOleInPlaceUIWindow*)This->frame, &This->IOleInPlaceActiveObject_iface);
         }else {
             FIXME("OnUIActivate failed: %08lx\n", hres);
             IOleInPlaceFrame_Release(This->frame);
@@ -625,14 +624,14 @@ static HRESULT WINAPI OleDocumentView_UIActivate(IOleDocumentView *iface, BOOL f
         if(This->hostui) {
             hres = IDocHostUIHandler_ShowUI(This->hostui,
                     This->nscontainer->usermode == EDITMODE ? DOCHOSTUITYPE_AUTHOR : DOCHOSTUITYPE_BROWSE,
-                    &This->basedoc.IOleInPlaceActiveObject_iface, &This->basedoc.IOleCommandTarget_iface,
+                    &This->IOleInPlaceActiveObject_iface, &This->IOleCommandTarget_iface,
                     This->frame, This->ip_window);
             if(FAILED(hres))
                 IDocHostUIHandler_HideUI(This->hostui);
         }
 
         if(This->ip_window)
-            call_set_active_object(This->ip_window, &This->basedoc.IOleInPlaceActiveObject_iface);
+            call_set_active_object(This->ip_window, &This->IOleInPlaceActiveObject_iface);
 
         SetRectEmpty(&rcBorderWidths);
         IOleInPlaceFrame_SetBorderSpace(This->frame, &rcBorderWidths);
@@ -728,19 +727,19 @@ static inline HTMLDocumentObj *impl_from_IViewObjectEx(IViewObjectEx *iface)
 static HRESULT WINAPI ViewObject_QueryInterface(IViewObjectEx *iface, REFIID riid, void **ppv)
 {
     HTMLDocumentObj *This = impl_from_IViewObjectEx(iface);
-    return htmldoc_query_interface(&This->basedoc, riid, ppv);
+    return IUnknown_QueryInterface(This->outer_unk, riid, ppv);
 }
 
 static ULONG WINAPI ViewObject_AddRef(IViewObjectEx *iface)
 {
     HTMLDocumentObj *This = impl_from_IViewObjectEx(iface);
-    return htmldoc_addref(&This->basedoc);
+    return IUnknown_AddRef(This->outer_unk);
 }
 
 static ULONG WINAPI ViewObject_Release(IViewObjectEx *iface)
 {
     HTMLDocumentObj *This = impl_from_IViewObjectEx(iface);
-    return htmldoc_release(&This->basedoc);
+    return IUnknown_Release(This->outer_unk);
 }
 
 static HRESULT WINAPI ViewObject_Draw(IViewObjectEx *iface, DWORD dwDrawAspect, LONG lindex, void *pvAspect,
@@ -895,13 +894,13 @@ static HRESULT WINAPI WindowForBindingUI_QueryInterface(IWindowForBindingUI *ifa
 static ULONG WINAPI WindowForBindingUI_AddRef(IWindowForBindingUI *iface)
 {
     HTMLDocumentObj *This = impl_from_IWindowForBindingUI(iface);
-    return htmldoc_addref(&This->basedoc);
+    return IUnknown_AddRef(This->outer_unk);
 }
 
 static ULONG WINAPI WindowForBindingUI_Release(IWindowForBindingUI *iface)
 {
     HTMLDocumentObj *This = impl_from_IWindowForBindingUI(iface);
-    return htmldoc_release(&This->basedoc);
+    return IUnknown_Release(This->outer_unk);
 }
 
 static HRESULT WINAPI WindowForBindingUI_GetWindow(IWindowForBindingUI *iface, REFGUID rguidReason, HWND *phwnd)
