@@ -36,8 +36,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
-#define MSHTML_DISPID_HTMLXMLHTTPREQUEST_ONLOAD MSHTML_DISPID_CUSTOM_MIN
-
 static HRESULT bstr_to_nsacstr(BSTR bstr, nsACString *str)
 {
     char *cstr = heap_strdupWtoU(bstr);
@@ -105,6 +103,27 @@ static const eventid_t events[] = {
     EVENTID_TIMEOUT,
 };
 
+typedef enum {
+    response_type_empty,
+    response_type_text,
+    response_type_doc,
+    response_type_arraybuf,
+    response_type_blob,
+    response_type_stream
+} response_type_t;
+
+static const struct {
+    const WCHAR *str;
+    const WCHAR *nsxhr_str;
+} response_type_desc[] = {
+    [response_type_empty]       = { L"",            L"" },
+    [response_type_text]        = { L"text",        L"" },
+    [response_type_doc]         = { L"document",    L"" }, /* FIXME */
+    [response_type_arraybuf]    = { L"arraybuffer", L"arraybuffer" },
+    [response_type_blob]        = { L"blob",        L"arraybuffer" },
+    [response_type_stream]      = { L"ms-stream",   L"arraybuffer" } /* FIXME */
+};
+
 typedef struct {
     nsIDOMEventListener nsIDOMEventListener_iface;
     LONG ref;
@@ -119,6 +138,7 @@ struct HTMLXMLHttpRequest {
     IWineXMLHttpRequestPrivate IWineXMLHttpRequestPrivate_iface;
     IProvideClassInfo2 IProvideClassInfo2_iface;
     LONG ref;
+    response_type_t response_type;
     nsIXMLHttpRequest *nsxhr;
     XMLHttpReqEventListener *event_listener;
 };
@@ -374,6 +394,20 @@ static HRESULT WINAPI HTMLXMLHttpRequest_get_responseXML(IHTMLXMLHttpRequest *if
 
     TRACE("(%p)->(%p)\n", This, p);
 
+    if(dispex_compat_mode(&This->event_target.dispex) >= COMPAT_MODE_IE10) {
+        nsIDOMDocument *nsdoc;
+        nsresult nsres;
+
+        nsres = nsIXMLHttpRequest_GetResponseXML(This->nsxhr, &nsdoc);
+        if(NS_FAILED(nsres))
+            return map_nsresult(nsres);
+        if(!nsdoc) {
+            *p = NULL;
+            return S_OK;
+        }
+        nsIDOMDocument_Release(nsdoc);
+    }
+
     hres = CoCreateInstance(&CLSID_DOMDocument, NULL, CLSCTX_INPROC_SERVER, &IID_IXMLDOMDocument, (void**)&xmldoc);
     if(FAILED(hres)) {
         ERR("CoCreateInstance failed: %08lx\n", hres);
@@ -904,28 +938,93 @@ static HRESULT WINAPI HTMLXMLHttpRequest_private_Invoke(IWineXMLHttpRequestPriva
 static HRESULT WINAPI HTMLXMLHttpRequest_private_get_response(IWineXMLHttpRequestPrivate *iface, VARIANT *p)
 {
     HTMLXMLHttpRequest *This = impl_from_IWineXMLHttpRequestPrivate(iface);
+    HRESULT hres = S_OK;
+    nsresult nsres;
+    UINT16 state;
 
-    FIXME("(%p)->(%p)\n", This, p);
+    TRACE("(%p)->(%p)\n", This, p);
 
-    return E_NOTIMPL;
+    switch(This->response_type) {
+    case response_type_empty:
+    case response_type_text:
+        hres = IHTMLXMLHttpRequest_get_responseText(&This->IHTMLXMLHttpRequest_iface, &V_BSTR(p));
+        if(SUCCEEDED(hres))
+            V_VT(p) = VT_BSTR;
+        break;
+
+    case response_type_doc:
+        FIXME("response_type_doc\n");
+        return E_NOTIMPL;
+
+    case response_type_arraybuf:
+    case response_type_blob:
+        nsres = nsIXMLHttpRequest_GetReadyState(This->nsxhr, &state);
+        if(NS_FAILED(nsres) || state < 4) {
+            V_VT(p) = VT_EMPTY;
+            break;
+        }
+        if(This->response_type == response_type_arraybuf) {
+            FIXME("response_type_arraybuf\n");
+            return E_NOTIMPL;
+        }
+        FIXME("response_type_blob\n");
+        return E_NOTIMPL;
+
+    case response_type_stream:
+        FIXME("response_type_stream\n");
+        return E_NOTIMPL;
+
+    default:
+        assert(0);
+    }
+
+    return hres;
 }
 
 static HRESULT WINAPI HTMLXMLHttpRequest_private_put_responseType(IWineXMLHttpRequestPrivate *iface, BSTR v)
 {
     HTMLXMLHttpRequest *This = impl_from_IWineXMLHttpRequestPrivate(iface);
+    nsAString nsstr;
+    nsresult nsres;
+    HRESULT hres;
+    unsigned i;
+    LONG state;
 
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
 
-    return E_NOTIMPL;
+    hres = IHTMLXMLHttpRequest_get_readyState(&This->IHTMLXMLHttpRequest_iface, &state);
+    if(FAILED(hres))
+        return hres;
+
+    if(state < 1 || state > 2) {
+        /* FIXME: Return InvalidStateError */
+        return E_FAIL;
+    }
+
+    for(i = 0; i < ARRAY_SIZE(response_type_desc); i++)
+        if(!wcscmp(v, response_type_desc[i].str))
+            break;
+    if(i >= ARRAY_SIZE(response_type_desc))
+        return S_OK;
+
+    nsAString_InitDepend(&nsstr, response_type_desc[i].nsxhr_str);
+    nsres = nsIXMLHttpRequest_SetResponseType(This->nsxhr, &nsstr);
+    nsAString_Finish(&nsstr);
+    if(NS_FAILED(nsres))
+        return map_nsresult(nsres);
+
+    This->response_type = i;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLXMLHttpRequest_private_get_responseType(IWineXMLHttpRequestPrivate *iface, BSTR *p)
 {
     HTMLXMLHttpRequest *This = impl_from_IWineXMLHttpRequestPrivate(iface);
 
-    FIXME("(%p)->(%p)\n", This, p);
+    TRACE("(%p)->(%p)\n", This, p);
 
-    return E_NOTIMPL;
+    *p = SysAllocString(response_type_desc[This->response_type].str);
+    return *p ? S_OK : E_OUTOFMEMORY;
 }
 
 static HRESULT WINAPI HTMLXMLHttpRequest_private_get_upload(IWineXMLHttpRequestPrivate *iface, IDispatch **p)
@@ -964,10 +1063,29 @@ static HRESULT WINAPI HTMLXMLHttpRequest_private_get_withCredentials(IWineXMLHtt
 static HRESULT WINAPI HTMLXMLHttpRequest_private_overrideMimeType(IWineXMLHttpRequestPrivate *iface, BSTR mimeType)
 {
     HTMLXMLHttpRequest *This = impl_from_IWineXMLHttpRequestPrivate(iface);
+    static const WCHAR generic_type[] = L"application/octet-stream";
+    const WCHAR *type = NULL;
+    WCHAR *lowercase = NULL;
+    nsAString nsstr;
+    nsresult nsres;
 
-    FIXME("(%p)->(%s)\n", This, debugstr_w(mimeType));
+    TRACE("(%p)->(%s)\n", This, debugstr_w(mimeType));
 
-    return E_NOTIMPL;
+    if(mimeType) {
+        if(mimeType[0]) {
+            if(!(lowercase = heap_strdupW(mimeType)))
+                return E_OUTOFMEMORY;
+            _wcslwr(lowercase);
+            type = lowercase;
+        }else
+            type = generic_type;
+    }
+
+    nsAString_InitDepend(&nsstr, type);
+    nsres = nsIXMLHttpRequest_SlowOverrideMimeType(This->nsxhr, &nsstr);
+    nsAString_Finish(&nsstr);
+    heap_free(lowercase);
+    return map_nsresult(nsres);
 }
 
 static HRESULT WINAPI HTMLXMLHttpRequest_private_put_onerror(IWineXMLHttpRequestPrivate *iface, VARIANT v)
@@ -1060,6 +1178,24 @@ static HRESULT WINAPI HTMLXMLHttpRequest_private_get_onloadend(IWineXMLHttpReque
     return get_event_handler(&This->event_target, EVENTID_LOADEND, p);
 }
 
+static HRESULT WINAPI HTMLXMLHttpRequest_private_put_onload(IWineXMLHttpRequestPrivate *iface, VARIANT v)
+{
+    HTMLXMLHttpRequest *This = impl_from_IWineXMLHttpRequestPrivate(iface);
+
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
+
+    return set_event_handler(&This->event_target, EVENTID_LOAD, &v);
+}
+
+static HRESULT WINAPI HTMLXMLHttpRequest_private_get_onload(IWineXMLHttpRequestPrivate *iface, VARIANT *p)
+{
+    HTMLXMLHttpRequest *This = impl_from_IWineXMLHttpRequestPrivate(iface);
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_event_handler(&This->event_target, EVENTID_LOAD, p);
+}
+
 static const IWineXMLHttpRequestPrivateVtbl WineXMLHttpRequestPrivateVtbl = {
     HTMLXMLHttpRequest_private_QueryInterface,
     HTMLXMLHttpRequest_private_AddRef,
@@ -1084,7 +1220,9 @@ static const IWineXMLHttpRequestPrivateVtbl WineXMLHttpRequestPrivateVtbl = {
     HTMLXMLHttpRequest_private_put_onloadstart,
     HTMLXMLHttpRequest_private_get_onloadstart,
     HTMLXMLHttpRequest_private_put_onloadend,
-    HTMLXMLHttpRequest_private_get_onloadend
+    HTMLXMLHttpRequest_private_get_onloadend,
+    HTMLXMLHttpRequest_private_put_onload,
+    HTMLXMLHttpRequest_private_get_onload
 };
 
 static inline HTMLXMLHttpRequest *impl_from_IProvideClassInfo2(IProvideClassInfo2 *iface)
@@ -1137,47 +1275,6 @@ static inline HTMLXMLHttpRequest *impl_from_DispatchEx(DispatchEx *iface)
     return CONTAINING_RECORD(iface, HTMLXMLHttpRequest, event_target.dispex);
 }
 
-static HRESULT HTMLXMLHttpRequest_get_dispid(DispatchEx *dispex, BSTR name, DWORD flags, DISPID *dispid)
-{
-    /* onload event handler property is supported, but not exposed by any interface. We implement as a custom property. */
-    if(!wcscmp(L"onload", name)) {
-        *dispid = MSHTML_DISPID_HTMLXMLHTTPREQUEST_ONLOAD;
-        return S_OK;
-    }
-
-    return DISP_E_UNKNOWNNAME;
-}
-
-static HRESULT HTMLXMLHttpRequest_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
-        VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
-{
-    HTMLXMLHttpRequest *This = impl_from_DispatchEx(dispex);
-
-    if(id == MSHTML_DISPID_HTMLXMLHTTPREQUEST_ONLOAD) {
-        switch(flags) {
-        case DISPATCH_PROPERTYGET:
-            TRACE("(%p) get onload\n", This);
-            return get_event_handler(&This->event_target, EVENTID_LOAD, res);
-
-        case DISPATCH_PROPERTYPUT:
-            if(params->cArgs != 1 || (params->cNamedArgs == 1 && *params->rgdispidNamedArgs != DISPID_PROPERTYPUT)
-               || params->cNamedArgs > 1) {
-                FIXME("invalid args\n");
-                return E_INVALIDARG;
-            }
-
-            TRACE("(%p)->(%p) set onload\n", This, params->rgvarg);
-            return set_event_handler(&This->event_target, EVENTID_LOAD, params->rgvarg);
-
-        default:
-            FIXME("Unimplemented flags %x\n", flags);
-            return E_NOTIMPL;
-        }
-    }
-
-    return DISP_E_UNKNOWNNAME;
-}
-
 static nsISupports *HTMLXMLHttpRequest_get_gecko_target(DispatchEx *dispex)
 {
     HTMLXMLHttpRequest *This = impl_from_DispatchEx(dispex);
@@ -1233,22 +1330,33 @@ static void HTMLXMLHttpRequest_init_dispex_info(dispex_data_t *info, compat_mode
         {DISPID_IHTMLXMLHTTPREQUEST_OPEN, HTMLXMLHttpRequest_open_hook},
         {DISPID_UNKNOWN}
     };
-    static const dispex_hook_t private_ie10_hooks[] = {
+    static const dispex_hook_t private_hooks[] = {
+        {DISPID_IWINEXMLHTTPREQUESTPRIVATE_RESPONSE},
+        {DISPID_IWINEXMLHTTPREQUESTPRIVATE_RESPONSETYPE},
+        {DISPID_IWINEXMLHTTPREQUESTPRIVATE_UPLOAD},
+        {DISPID_IWINEXMLHTTPREQUESTPRIVATE_WITHCREDENTIALS},
+        {DISPID_EVPROP_ONERROR},
+        {DISPID_EVPROP_ONABORT},
+        {DISPID_EVPROP_PROGRESS},
+        {DISPID_EVPROP_LOADSTART},
+        {DISPID_EVPROP_LOADEND},
+
+        /* IE10 only */
         {DISPID_IWINEXMLHTTPREQUESTPRIVATE_OVERRIDEMIMETYPE},
         {DISPID_UNKNOWN}
     };
+    const dispex_hook_t *const private_ie10_hooks = private_hooks + ARRAY_SIZE(private_hooks) - 2;
 
     EventTarget_init_dispex_info(info, compat_mode);
     dispex_info_add_interface(info, IHTMLXMLHttpRequest_tid, compat_mode >= COMPAT_MODE_IE10 ? xhr_hooks : NULL);
-    if(compat_mode >= COMPAT_MODE_IE10)
-        dispex_info_add_interface(info, IWineXMLHttpRequestPrivate_tid, compat_mode == COMPAT_MODE_IE10 ? private_ie10_hooks : NULL);
+    dispex_info_add_interface(info, IWineXMLHttpRequestPrivate_tid,
+        compat_mode < COMPAT_MODE_IE10 ? private_hooks :
+        compat_mode < COMPAT_MODE_IE11 ? private_ie10_hooks : NULL);
 }
 
 static event_target_vtbl_t HTMLXMLHttpRequest_event_target_vtbl = {
     {
         NULL,
-        HTMLXMLHttpRequest_get_dispid,
-        HTMLXMLHttpRequest_invoke
     },
     HTMLXMLHttpRequest_get_gecko_target,
     HTMLXMLHttpRequest_bind_event
