@@ -48,6 +48,15 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
 
+BOOL msi_get_temp_file_name( MSIPACKAGE *package, const WCHAR *tmp_path, const WCHAR *prefix, WCHAR *tmp_filename )
+{
+    BOOL ret;
+    msi_disable_fs_redirection( package );
+    ret = GetTempFileNameW( tmp_path, prefix, 0, tmp_filename );
+    msi_revert_fs_redirection( package );
+    return ret;
+}
+
 HANDLE msi_create_file( MSIPACKAGE *package, const WCHAR *filename, DWORD access, DWORD sharing, DWORD creation,
                         DWORD flags )
 {
@@ -161,7 +170,8 @@ DWORD msi_get_file_version_info( MSIPACKAGE *package, const WCHAR *path, DWORD b
 VS_FIXEDFILEINFO *msi_get_disk_file_version( MSIPACKAGE *package, const WCHAR *filename )
 {
     VS_FIXEDFILEINFO *ptr, *ret;
-    DWORD version_size, size;
+    DWORD version_size;
+    UINT size;
     void *version;
 
     if (!(version_size = msi_get_file_version_info( package, filename, 0, NULL ))) return NULL;
@@ -354,7 +364,7 @@ static msi_file_state calculate_install_state( MSIPACKAGE *package, MSIFILE *fil
     }
     if ((size = msi_get_disk_file_size( package, file->TargetPath )) != file->FileSize)
     {
-        TRACE("overwriting %s (old size %u new size %u)\n", debugstr_w(file->File), size, file->FileSize);
+        TRACE("overwriting %s (old size %lu new size %d)\n", debugstr_w(file->File), size, file->FileSize);
         return msifs_overwrite;
     }
     if (file->hash.dwFileHashInfoSize)
@@ -721,7 +731,7 @@ static UINT patch_file( MSIPACKAGE *package, MSIFILEPATCH *patch )
     }
     else
     {
-        WARN("failed to patch %s: %08x\n", debugstr_w(patch->File->TargetPath), GetLastError());
+        WARN( "failed to patch %s: %#lx\n", debugstr_w(patch->File->TargetPath), GetLastError() );
         r = ERROR_INSTALL_FAILURE;
     }
     DeleteFileW( patch->path );
@@ -742,7 +752,7 @@ static UINT patch_assembly( MSIPACKAGE *package, MSIASSEMBLY *assembly, MSIFILEP
     while ((IAssemblyEnum_GetNextAssembly( iter, NULL, &name, 0 ) == S_OK))
     {
         WCHAR *displayname, *path;
-        UINT len = 0;
+        DWORD len = 0;
         HRESULT hr;
 
         hr = IAssemblyName_GetDisplayName( name, NULL, &len, 0 );
@@ -760,8 +770,8 @@ static UINT patch_assembly( MSIPACKAGE *package, MSIASSEMBLY *assembly, MSIFILEP
         {
             if (!msi_copy_file( package, path, patch->File->TargetPath, FALSE ))
             {
-                ERR("Failed to copy file %s -> %s (%u)\n", debugstr_w(path),
-                    debugstr_w(patch->File->TargetPath), GetLastError() );
+                ERR( "failed to copy file %s -> %s (%lu)\n", debugstr_w(path),
+                     debugstr_w(patch->File->TargetPath), GetLastError() );
                 msi_free( path );
                 msi_free( displayname );
                 IAssemblyName_Release( name );
@@ -897,7 +907,7 @@ static BOOL move_file( MSIPACKAGE *package, const WCHAR *source, const WCHAR *de
         ret = msi_move_file( package, source, dest, MOVEFILE_REPLACE_EXISTING );
         if (!ret)
         {
-            WARN("msi_move_file failed: %u\n", GetLastError());
+            WARN( "msi_move_file failed: %lu\n", GetLastError() );
             return FALSE;
         }
     }
@@ -907,7 +917,7 @@ static BOOL move_file( MSIPACKAGE *package, const WCHAR *source, const WCHAR *de
         ret = msi_copy_file( package, source, dest, FALSE );
         if (!ret)
         {
-            WARN("msi_copy_file failed: %u\n", GetLastError());
+            WARN( "msi_copy_file failed: %lu\n", GetLastError() );
             return FALSE;
         }
     }
@@ -1175,7 +1185,7 @@ static UINT ITERATE_MoveFiles( MSIRECORD *rec, LPVOID param )
     {
         if (!msi_create_full_path( package, destdir ))
         {
-            WARN("failed to create directory %u\n", GetLastError());
+            WARN( "failed to create directory %lu\n", GetLastError() );
             goto done;
         }
     }
@@ -1316,8 +1326,8 @@ static UINT ITERATE_DuplicateFiles(MSIRECORD *row, LPVOID param)
     TRACE("Duplicating file %s to %s\n", debugstr_w(file->TargetPath), debugstr_w(dest));
     if (!msi_copy_file( package, file->TargetPath, dest, TRUE ))
     {
-        WARN("Failed to copy file %s -> %s (%u)\n",
-             debugstr_w(file->TargetPath), debugstr_w(dest), GetLastError());
+        WARN( "failed to copy file %s -> %s (%lu)\n",
+              debugstr_w(file->TargetPath), debugstr_w(dest), GetLastError() );
     }
     FIXME("We should track these duplicate files as well\n");
 
@@ -1394,7 +1404,7 @@ static UINT ITERATE_RemoveDuplicateFiles( MSIRECORD *row, LPVOID param )
     TRACE("Removing duplicate %s of %s\n", debugstr_w(dest), debugstr_w(file->TargetPath));
     if (!msi_delete_file( package, dest ))
     {
-        WARN("Failed to delete duplicate file %s (%u)\n", debugstr_w(dest), GetLastError());
+        WARN( "failed to delete duplicate file %s (%lu)\n", debugstr_w(dest), GetLastError() );
     }
 
     uirow = MSI_CreateRecord( 9 );
@@ -1605,7 +1615,7 @@ UINT ACTION_RemoveFiles( MSIPACKAGE *package )
         msi_set_file_attributes( package, file->TargetPath, FILE_ATTRIBUTE_NORMAL );
         if (!msi_delete_file( package, file->TargetPath ))
         {
-            WARN("failed to delete %s (%u)\n",  debugstr_w(file->TargetPath), GetLastError());
+            WARN( "failed to delete %s (%lu)\n",  debugstr_w(file->TargetPath), GetLastError() );
         }
         file->state = msifs_missing;
 
