@@ -43,7 +43,6 @@
 #include "msidefs.h"
 #include "sddl.h"
 
-#include "wine/heap.h"
 #include "wine/debug.h"
 #include "wine/exception.h"
 
@@ -292,7 +291,7 @@ static void free_package_structures( MSIPACKAGE *package )
 
         list_remove( &binary->entry );
         if (!DeleteFileW( binary->tmpfile ))
-            ERR("failed to delete %s (%u)\n", debugstr_w(binary->tmpfile), GetLastError());
+            ERR( "failed to delete %s (%lu)\n", debugstr_w(binary->tmpfile), GetLastError() );
         msi_free( binary->source );
         msi_free( binary->tmpfile );
         msi_free( binary );
@@ -315,7 +314,7 @@ static void free_package_structures( MSIPACKAGE *package )
         list_remove( &patch->entry );
         if (patch->delete_on_close && !DeleteFileW( patch->localfile ))
         {
-            ERR("failed to delete %s (%u)\n", debugstr_w(patch->localfile), GetLastError());
+            ERR( "failed to delete %s (%lu)\n", debugstr_w(patch->localfile), GetLastError() );
         }
         msi_free_patchinfo( patch );
     }
@@ -527,14 +526,14 @@ static LPWSTR get_fusion_filename(MSIPACKAGE *package)
 
     if (!RegCreateKeyExW(netsetup, L"v4\\Client", 0, NULL, 0, KEY_QUERY_VALUE, NULL, &hkey, NULL))
     {
-        size = ARRAY_SIZE(path);
+        size = sizeof(path);
         if (!RegQueryValueExW(hkey, L"InstallPath", NULL, &type, (BYTE *)path, &size))
         {
             len = lstrlenW(path) + lstrlenW(L"fusion.dll") + 2;
             if (!(filename = msi_alloc(len * sizeof(WCHAR)))) return NULL;
 
             lstrcpyW(filename, path);
-            lstrcpyW(filename, L"\\");
+            lstrcatW(filename, L"\\");
             lstrcatW(filename, L"fusion.dll");
             if (GetFileAttributesW(filename) != INVALID_FILE_ATTRIBUTES)
             {
@@ -553,6 +552,7 @@ static LPWSTR get_fusion_filename(MSIPACKAGE *package)
         GetWindowsDirectoryW(windir, MAX_PATH);
         len = lstrlenW(windir) + lstrlenW(L"Microsoft.NET\\Framework\\") + lstrlenW(L"v2.0.50727") +
               lstrlenW(L"fusion.dll") + 3;
+        msi_free(filename);
         if (!(filename = msi_alloc(len * sizeof(WCHAR)))) return NULL;
 
         lstrcpyW(filename, windir);
@@ -782,12 +782,12 @@ static VOID set_installer_properties(MSIPACKAGE *package)
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"SystemFolder", pth, -1 );
 
-        len = MAX_PATH;
+        len = sizeof(pth);
         RegQueryValueExW(hkey, L"ProgramFilesDir", 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"ProgramFilesFolder", pth, -1 );
 
-        len = MAX_PATH;
+        len = sizeof(pth);
         RegQueryValueExW(hkey, L"CommonFilesDir", 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"CommonFilesFolder", pth, -1 );
@@ -806,22 +806,22 @@ static VOID set_installer_properties(MSIPACKAGE *package)
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"SystemFolder", pth, -1 );
 
-        len = MAX_PATH;
+        len = sizeof(pth);
         RegQueryValueExW(hkey, L"ProgramFilesDir", 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"ProgramFiles64Folder", pth, -1 );
 
-        len = MAX_PATH;
+        len = sizeof(pth);
         RegQueryValueExW(hkey, L"ProgramFilesDir (x86)", 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"ProgramFilesFolder", pth, -1 );
 
-        len = MAX_PATH;
+        len = sizeof(pth);
         RegQueryValueExW(hkey, L"CommonFilesDir", 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"CommonFiles64Folder", pth, -1 );
 
-        len = MAX_PATH;
+        len = sizeof(pth);
         RegQueryValueExW(hkey, L"CommonFilesDir (x86)", 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, L"CommonFilesFolder", pth, -1 );
@@ -1546,12 +1546,12 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, DWORD dwOptions, MSIPACKAGE **pPackage)
     return ERROR_SUCCESS;
 }
 
-UINT WINAPI MsiOpenPackageExW(LPCWSTR szPackage, DWORD dwOptions, MSIHANDLE *phPackage)
+UINT WINAPI MsiOpenPackageExW( const WCHAR *szPackage, DWORD dwOptions, MSIHANDLE *phPackage )
 {
     MSIPACKAGE *package = NULL;
     UINT ret;
 
-    TRACE("%s %08x %p\n", debugstr_w(szPackage), dwOptions, phPackage );
+    TRACE( "%s, %#lx, %p\n", debugstr_w(szPackage), dwOptions, phPackage );
 
     if( !szPackage || !phPackage )
         return ERROR_INVALID_PARAMETER;
@@ -1563,7 +1563,7 @@ UINT WINAPI MsiOpenPackageExW(LPCWSTR szPackage, DWORD dwOptions, MSIHANDLE *phP
     }
 
     if( dwOptions )
-        FIXME("dwOptions %08x not supported\n", dwOptions);
+        FIXME( "dwOptions %#lx not supported\n", dwOptions );
 
     ret = MSI_OpenPackageW( szPackage, 0, &package );
     if( ret == ERROR_SUCCESS )
@@ -1608,13 +1608,13 @@ UINT WINAPI MsiOpenPackageA(LPCSTR szPackage, MSIHANDLE *phPackage)
     return MsiOpenPackageExA( szPackage, 0, phPackage );
 }
 
-MSIHANDLE WINAPI MsiGetActiveDatabase(MSIHANDLE hInstall)
+MSIHANDLE WINAPI MsiGetActiveDatabase( MSIHANDLE hInstall )
 {
     MSIPACKAGE *package;
     MSIHANDLE handle = 0;
     MSIHANDLE remote;
 
-    TRACE("(%d)\n",hInstall);
+    TRACE( "%lu\n", hInstall );
 
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE);
     if( package)
@@ -1806,7 +1806,7 @@ INT MSI_ProcessMessageVerbatim(MSIPACKAGE *package, INSTALLMESSAGE eMessageType,
         MSI_FormatRecordW(package, record, message, &len);
     }
 
-    /* convert it to ASCII */
+    /* convert it to ANSI */
     len = WideCharToMultiByte( CP_ACP, 0, message, -1, NULL, 0, NULL, NULL );
     msg = msi_alloc( len );
     WideCharToMultiByte( CP_ACP, 0, message, -1, msg, len, NULL, NULL );
@@ -1814,21 +1814,21 @@ INT MSI_ProcessMessageVerbatim(MSIPACKAGE *package, INSTALLMESSAGE eMessageType,
     if (gUIHandlerRecord && (gUIFilterRecord & log_type))
     {
         MSIHANDLE rec = alloc_msihandle(&record->hdr);
-        TRACE("Calling UI handler %p(pvContext=%p, iMessageType=%08x, hRecord=%u)\n",
-              gUIHandlerRecord, gUIContextRecord, eMessageType, rec);
+        TRACE( "calling UI handler %p(pvContext = %p, iMessageType = %#x, hRecord = %lu)\n",
+               gUIHandlerRecord, gUIContextRecord, eMessageType, rec );
         rc = gUIHandlerRecord( gUIContextRecord, eMessageType, rec );
         MsiCloseHandle( rec );
     }
     if (!rc && gUIHandlerW && (gUIFilter & log_type))
     {
-        TRACE("Calling UI handler %p(pvContext=%p, iMessageType=%08x, szMessage=%s)\n",
-              gUIHandlerW, gUIContext, eMessageType, debugstr_w(message));
+        TRACE( "calling UI handler %p(pvContext = %p, iMessageType = %#x, szMessage = %s)\n",
+               gUIHandlerW, gUIContext, eMessageType, debugstr_w(message) );
         rc = gUIHandlerW( gUIContext, eMessageType, message );
     }
     else if (!rc && gUIHandlerA && (gUIFilter & log_type))
     {
-        TRACE("Calling UI handler %p(pvContext=%p, iMessageType=%08x, szMessage=%s)\n",
-              gUIHandlerA, gUIContext, eMessageType, debugstr_a(msg));
+        TRACE( "calling UI handler %p(pvContext = %p, iMessageType = %#x, szMessage = %s)\n",
+               gUIHandlerA, gUIContext, eMessageType, debugstr_a(msg) );
         rc = gUIHandlerA( gUIContext, eMessageType, msg );
     }
 
@@ -2207,8 +2207,7 @@ UINT msi_get_property( MSIDATABASE *db, LPCWSTR szName,
         TRACE("returning %s for property %s\n", debugstr_wn(szValueBuf, *pchValueBuf),
             debugstr_w(szName));
     else if (rc == ERROR_MORE_DATA)
-        TRACE("need %d sized buffer for %s\n", *pchValueBuf,
-            debugstr_w(szName));
+        TRACE( "need %lu sized buffer for %s\n", *pchValueBuf, debugstr_w(szName) );
     else
     {
         *pchValueBuf = 0;
@@ -2272,7 +2271,7 @@ UINT WINAPI MsiGetPropertyA(MSIHANDLE hinst, const char *name, char *buf, DWORD
 
         if (!(remote = msi_get_remote(hinst)))
         {
-            heap_free(nameW);
+            free(nameW);
             return ERROR_INVALID_HANDLE;
         }
 
@@ -2286,13 +2285,13 @@ UINT WINAPI MsiGetPropertyA(MSIHANDLE hinst, const char *name, char *buf, DWORD
         }
         __ENDTRY
 
-        heap_free(nameW);
+        free(nameW);
 
         if (!r)
         {
             /* String might contain embedded nulls.
              * Native returns the correct size but truncates the string. */
-            tmp = heap_alloc_zero((len + 1) * sizeof(WCHAR));
+            tmp = calloc(1, (len + 1) * sizeof(WCHAR));
             if (!tmp)
             {
                 midl_user_free(value);
@@ -2302,7 +2301,7 @@ UINT WINAPI MsiGetPropertyA(MSIHANDLE hinst, const char *name, char *buf, DWORD
 
             r = msi_strncpyWtoA(tmp, len, buf, sz, TRUE);
 
-            heap_free(tmp);
+            free(tmp);
         }
         midl_user_free(value);
         return r;
@@ -2314,7 +2313,7 @@ UINT WINAPI MsiGetPropertyA(MSIHANDLE hinst, const char *name, char *buf, DWORD
 
     r = msi_strncpyWtoA(value, len, buf, sz, FALSE);
 
-    heap_free(nameW);
+    free(nameW);
     if (row) msiobj_release(&row->hdr);
     msiobj_release(&package->hdr);
     return r;
@@ -2355,7 +2354,7 @@ UINT WINAPI MsiGetPropertyW(MSIHANDLE hinst, const WCHAR *name, WCHAR *buf, DWOR
         {
             /* String might contain embedded nulls.
              * Native returns the correct size but truncates the string. */
-            tmp = heap_alloc_zero((len + 1) * sizeof(WCHAR));
+            tmp = calloc(1, (len + 1) * sizeof(WCHAR));
             if (!tmp)
             {
                 midl_user_free(value);
@@ -2365,7 +2364,7 @@ UINT WINAPI MsiGetPropertyW(MSIHANDLE hinst, const WCHAR *name, WCHAR *buf, DWOR
 
             r = msi_strncpyW(tmp, len, buf, sz);
 
-            heap_free(tmp);
+            free(tmp);
         }
         midl_user_free(value);
         return r;
