@@ -255,22 +255,21 @@ static int compare_info(LCID lcid, DWORD flags, char* buff, const char* cmp, BOO
 }
 
 static BOOL CALLBACK
-find_best_locale_proc(HMODULE hModule, LPCSTR type, LPCSTR name, WORD LangID, LONG_PTR lParam)
+find_best_locale_proc( WCHAR *name, DWORD locale_flags, LPARAM lParam )
 {
   locale_search_t *res = (locale_search_t *)lParam;
-  const LCID lcid = MAKELCID(LangID, SORT_DEFAULT);
+  const LCID lcid = LocaleNameToLCID( name, 0 );
   char buff[MAX_ELEM_LEN];
   unsigned int flags = 0;
 
-  if(PRIMARYLANGID(LangID) == LANG_NEUTRAL)
-    return CONTINUE_LOOKING;
+  if (lcid == LOCALE_CUSTOM_UNSPECIFIED) return CONTINUE_LOOKING;
 
 #if _MSVCR_VER >= 110
   if (res->allow_sname && compare_info(lcid,LOCALE_SNAME,buff,res->search_language, TRUE))
   {
     TRACE(":Found locale: %s->%s\n", res->search_language, buff);
     res->match_flags = FOUND_SNAME;
-    res->found_lang_id = LangID;
+    res->found_lang_id = LANGIDFROMLCID(lcid);
     return STOP_LOOKING;
   }
 #endif
@@ -305,7 +304,7 @@ find_best_locale_proc(HMODULE hModule, LPCSTR type, LPCSTR name, WORD LangID, LO
   {
     /* Found a better match than previously */
     res->match_flags = flags;
-    res->found_lang_id = LangID;
+    res->found_lang_id = LANGIDFROMLCID(lcid);
   }
   if ((flags & (FOUND_LANGUAGE | FOUND_COUNTRY)) ==
         (FOUND_LANGUAGE | FOUND_COUNTRY))
@@ -363,12 +362,7 @@ LCID locale_to_LCID(const char *locale, unsigned short *codepage, BOOL *sname)
             search.allow_sname = TRUE;
         }
 
-        if(!_stricmp(search.search_country, "China"))
-            strcpy(search.search_country, "People's Republic of China");
-
-        EnumResourceLanguagesA(GetModuleHandleA("KERNEL32"), (LPSTR)RT_STRING,
-                (LPCSTR)LOCALE_ILANGUAGE,find_best_locale_proc,
-                (LONG_PTR)&search);
+        EnumSystemLocalesEx( find_best_locale_proc, 0, (LPARAM)&search, NULL);
 
         if (!search.match_flags)
             return -1;
@@ -525,8 +519,13 @@ static BOOL update_threadlocinfo_category(LCID lcid, unsigned short cp,
     if(!locinfo->lc_category[category].locale) {
         int len = 0;
 
-        len += GetLocaleInfoA(lcid, LOCALE_SENGLANGUAGE
-                |LOCALE_NOUSEROVERRIDE, buf, 256);
+        if (lcid == MAKELANGID( LANG_NORWEGIAN, SUBLANG_NORWEGIAN_NYNORSK ))
+        {
+            /* locale.nls contains "Norwegian Nynorsk" instead for LOCALE_SENGLANGUAGE */
+            strcpy( buf, "Norwegian-Nynorsk" );
+            len = strlen( buf ) + 1;
+        }
+        else len += GetLocaleInfoA(lcid, LOCALE_SENGLANGUAGE|LOCALE_NOUSEROVERRIDE, buf, 256);
         buf[len-1] = '_';
         len += GetLocaleInfoA(lcid, LOCALE_SENGCOUNTRY
                 |LOCALE_NOUSEROVERRIDE, &buf[len], 256-len);
@@ -1335,6 +1334,14 @@ static pthreadlocinfo create_locinfo(int category,
     }
 
     for(i=1; i<6; i++) {
+#if _MSVCR_VER < 140
+        if(i==LC_CTYPE && cp[i]==CP_UTF8) {
+            locale_name[i] = NULL;
+            locale_len[i] = 0;
+            lcid[i] = old_locinfo ? old_locinfo->lc_handle[i] : 0;
+            cp[i] = old_locinfo ? old_locinfo->lc_id[i].wCodePage : 0;
+        }
+#endif
         if(category!=LC_ALL && category!=i) {
             if(old_locinfo) {
                 lcid[i] = old_locinfo->lc_handle[i];
