@@ -32,11 +32,58 @@
 #include "ddk/wdm.h"
 #include "wine/exception.h"
 
+WINE_DEFAULT_DEBUG_CHANNEL(thread);
 WINE_DECLARE_DEBUG_CHANNEL(relay);
-WINE_DECLARE_DEBUG_CHANNEL(thread);
+WINE_DECLARE_DEBUG_CHANNEL(pid);
+WINE_DECLARE_DEBUG_CHANNEL(timestamp);
 
 struct _KUSER_SHARED_DATA *user_shared_data = (void *)0x7ffe0000;
 
+struct debug_info
+{
+    unsigned int str_pos;       /* current position in strings buffer */
+    unsigned int out_pos;       /* current position in output buffer */
+    char         strings[1020]; /* buffer for temporary strings */
+    char         output[1020];  /* current output line */
+};
+
+C_ASSERT( sizeof(struct debug_info) == 0x800 );
+
+static int nb_debug_options;
+static struct __wine_debug_channel *debug_options;
+
+static inline struct debug_info *get_info(void)
+{
+#ifdef _WIN64
+    return (struct debug_info *)((TEB32 *)((char *)NtCurrentTeb() + 0x2000) + 1);
+#else
+    return (struct debug_info *)(NtCurrentTeb() + 1);
+#endif
+}
+
+static void init_options(void)
+{
+    unsigned int offset = page_size * (sizeof(void *) / 4);
+
+    debug_options = (struct __wine_debug_channel *)((char *)NtCurrentTeb()->Peb + offset);
+    while (debug_options[nb_debug_options].name[0]) nb_debug_options++;
+}
+
+/* add a string to the output buffer */
+static int append_output( struct debug_info *info, const char *str, size_t len )
+{
+    if (len >= sizeof(info->output) - info->out_pos)
+    {
+        __wine_dbg_write( info->output, info->out_pos );
+        info->out_pos = 0;
+        ERR_(thread)( "debug buffer overflow:\n" );
+        __wine_dbg_write( str, len );
+        RtlRaiseStatus( STATUS_BUFFER_OVERFLOW );
+    }
+    memcpy( info->output + info->out_pos, str, len );
+    info->out_pos += len;
+    return len;
+}
 
 /***********************************************************************
  *		__wine_dbg_get_channel_flags  (NTDLL.@)
@@ -45,7 +92,25 @@ struct _KUSER_SHARED_DATA *user_shared_data = (void *)0x7ffe0000;
  */
 unsigned char __cdecl __wine_dbg_get_channel_flags( struct __wine_debug_channel *channel )
 {
-    return unix_funcs->dbg_get_channel_flags( channel );
+    int min, max, pos, res;
+    unsigned char default_flags;
+
+    if (!debug_options) init_options();
+
+    min = 0;
+    max = nb_debug_options - 1;
+    while (min <= max)
+    {
+        pos = (min + max) / 2;
+        res = strcmp( channel->name, debug_options[pos].name );
+        if (!res) return debug_options[pos].flags;
+        if (res < 0) max = pos - 1;
+        else min = pos + 1;
+    }
+    /* no option for this channel */
+    default_flags = debug_options[nb_debug_options].flags;
+    if (channel->flags & (1 << __WINE_DBCL_INIT)) channel->flags = default_flags;
+    return default_flags;
 }
 
 /***********************************************************************
@@ -53,7 +118,14 @@ unsigned char __cdecl __wine_dbg_get_channel_flags( struct __wine_debug_channel
  */
 const char * __cdecl __wine_dbg_strdup( const char *str )
 {
-    return unix_funcs->dbg_strdup( str );
+    struct debug_info *info = get_info();
+    unsigned int pos = info->str_pos;
+    size_t n = strlen( str ) + 1;
+
+    assert( n <= sizeof(info->strings) );
+    if (pos + n > sizeof(info->strings)) pos = 0;
+    info->str_pos = pos + n;
+    return memcpy( info->strings + pos, str, n );
 }
 
 /***********************************************************************
@@ -62,7 +134,27 @@ const char * __cdecl __wine_dbg_strdup( const char *str )
 int __cdecl __wine_dbg_header( enum __wine_debug_class cls, struct __wine_debug_channel *channel,
                                const char *function )
 {
-    return unix_funcs->dbg_header( cls, channel, function );
+    static const char * const classes[] = { "fixme", "err", "warn", "trace" };
+    struct debug_info *info = get_info();
+    char *pos = info->output;
+
+    if (!(__wine_dbg_get_channel_flags( channel ) & (1 << cls))) return -1;
+
+    /* only print header if we are at the beginning of the line */
+    if (info->out_pos) return 0;
+
+    if (TRACE_ON(timestamp))
+    {
+        ULONG ticks = NtGetTickCount();
+        pos += sprintf( pos, "%3u.%03u:", ticks / 1000, ticks % 1000 );
+    }
+    if (TRACE_ON(pid)) pos += sprintf( pos, "%04x:", GetCurrentProcessId() );
+    pos += sprintf( pos, "%04x:", GetCurrentThreadId() );
+    if (function && cls < ARRAY_SIZE( classes ))
+        pos += snprintf( pos, sizeof(info->output) - (pos - info->output), "%s:%s:%s ",
+                         classes[cls], channel->name, function );
+    info->out_pos = pos - info->output;
+    return info->out_pos;
 }
 
 /***********************************************************************
@@ -70,7 +162,19 @@ int __cdecl __wine_dbg_header( enum __wine_debug_class cls, struct __wine_debug_
  */
 int __cdecl __wine_dbg_output( const char *str )
 {
-    return unix_funcs->dbg_output( str );
+    struct debug_info *info = get_info();
+    const char *end = strrchr( str, '\n' );
+    int ret = 0;
+
+    if (end)
+    {
+        ret += append_output( info, str, end + 1 - str );
+        __wine_dbg_write( info->output, info->out_pos );
+        info->out_pos = 0;
+        str = end + 1;
+    }
+    if (*str) ret += append_output( info, str, strlen( str ));
+    return ret;
 }
 
 
@@ -150,7 +254,7 @@ void WINAPI RtlUserThreadStart( PRTL_THREAD_START_ROUTINE entry, void *arg )
  *              RtlCreateUserThread   (NTDLL.@)
  */
 NTSTATUS WINAPI RtlCreateUserThread( HANDLE process, SECURITY_DESCRIPTOR *descr,
-                                     BOOLEAN suspended, PVOID stack_addr,
+                                     BOOLEAN suspended, ULONG zero_bits,
                                      SIZE_T stack_reserve, SIZE_T stack_commit,
                                      PRTL_THREAD_START_ROUTINE start, void *param,
                                      HANDLE *handle_ptr, CLIENT_ID *id )
@@ -181,7 +285,7 @@ NTSTATUS WINAPI RtlCreateUserThread( HANDLE process, SECURITY_DESCRIPTOR *descr,
     if (actctx) flags |= THREAD_CREATE_FLAGS_CREATE_SUSPENDED;
 
     status = NtCreateThreadEx( &handle, THREAD_ALL_ACCESS, &attr, process, start, param,
-                               flags, 0, stack_commit, stack_reserve, attr_list );
+                               flags, zero_bits, stack_commit, stack_reserve, attr_list );
     if (!status)
     {
         if (actctx)
@@ -199,6 +303,75 @@ NTSTATUS WINAPI RtlCreateUserThread( HANDLE process, SECURITY_DESCRIPTOR *descr,
 }
 
 
+/**********************************************************************
+ *           RtlCreateUserStack (NTDLL.@)
+ */
+NTSTATUS WINAPI RtlCreateUserStack( SIZE_T commit, SIZE_T reserve, ULONG zero_bits,
+                                    SIZE_T commit_align, SIZE_T reserve_align, INITIAL_TEB *stack )
+{
+    PROCESS_STACK_ALLOCATION_INFORMATION alloc;
+    NTSTATUS status;
+
+    TRACE("commit %#lx, reserve %#lx, zero_bits %u, commit_align %#lx, reserve_align %#lx, stack %p\n",
+            commit, reserve, zero_bits, commit_align, reserve_align, stack);
+
+    if (!commit_align || !reserve_align)
+        return STATUS_INVALID_PARAMETER;
+
+    if (!commit || !reserve)
+    {
+        IMAGE_NT_HEADERS *nt = RtlImageNtHeader( NtCurrentTeb()->Peb->ImageBaseAddress );
+        if (!reserve) reserve = nt->OptionalHeader.SizeOfStackReserve;
+        if (!commit) commit = nt->OptionalHeader.SizeOfStackCommit;
+    }
+
+    reserve = (reserve + reserve_align - 1) & ~(reserve_align - 1);
+    commit = (commit + commit_align - 1) & ~(commit_align - 1);
+
+    if (reserve < commit) reserve = commit;
+    if (reserve < 0x100000) reserve = 0x100000;
+    reserve = (reserve + 0xffff) & ~0xffff;  /* round to 64K boundary */
+
+    alloc.ReserveSize = reserve;
+    alloc.ZeroBits = zero_bits;
+    status = NtSetInformationProcess( GetCurrentProcess(), ProcessThreadStackAllocation,
+                                      &alloc, sizeof(alloc) );
+    if (!status)
+    {
+        void *addr = alloc.StackBase;
+        SIZE_T size = page_size;
+
+        NtAllocateVirtualMemory( GetCurrentProcess(), &addr, 0, &size, MEM_COMMIT, PAGE_NOACCESS );
+        addr = (char *)alloc.StackBase + page_size;
+        NtAllocateVirtualMemory( GetCurrentProcess(), &addr, 0, &size, MEM_COMMIT, PAGE_READWRITE | PAGE_GUARD );
+        addr = (char *)alloc.StackBase + 2 * page_size;
+        size = reserve - 2 * page_size;
+        NtAllocateVirtualMemory( GetCurrentProcess(), &addr, 0, &size, MEM_COMMIT, PAGE_READWRITE );
+
+        /* note: limit is lower than base since the stack grows down */
+        stack->OldStackBase = 0;
+        stack->OldStackLimit = 0;
+        stack->DeallocationStack = alloc.StackBase;
+        stack->StackBase = (char *)alloc.StackBase + reserve;
+        stack->StackLimit = (char *)alloc.StackBase + 2 * page_size;
+    }
+    return status;
+}
+
+
+/**********************************************************************
+ *           RtlFreeUserStack (NTDLL.@)
+ */
+void WINAPI RtlFreeUserStack( void *stack )
+{
+    SIZE_T size = 0;
+
+    TRACE("stack %p\n", stack);
+
+    NtFreeVirtualMemory( NtCurrentProcess(), &stack, &size, MEM_RELEASE );
+}
+
+
 /******************************************************************************
  *              RtlGetNtGlobalFlags   (NTDLL.@)
  */
