@@ -920,7 +920,7 @@ static WCHAR *get_initial_environment( SIZE_T *pos, SIZE_T *size )
     *size = 1;
     for (e = main_envp; *e; e++) *size += strlen(*e) + 1;
 
-    if (!(env = malloc( *size * sizeof(WCHAR) ))) return NULL;
+    env = malloc( *size * sizeof(WCHAR) );
     ptr = env;
     end = env + *size - 1;
     for (e = main_envp; *e && ptr < end; e++)
@@ -2390,3 +2390,19 @@ ULONG WINAPI RtlNtStatusToDosError( NTSTATUS status )
 
     return map_status( status );
 }
+
+/**********************************************************************
+ *      RtlSetLastWin32Error  (ntdll.so)
+ */
+void WINAPI RtlSetLastWin32Error( DWORD err )
+{
+    TEB *teb = NtCurrentTeb();
+#ifdef _WIN64
+    if (teb->WowTebOffset)
+    {
+        TEB32 *teb32 = (TEB32 *)((char *)teb + teb->WowTebOffset);
+        teb32->LastErrorValue = err;
+    }
+#endif
+    teb->LastErrorValue = err;
+}
