@@ -213,7 +213,7 @@ PDH_STATUS WINAPI PdhAddCounterA( PDH_HQUERY query, LPCSTR path,
     PDH_STATUS ret;
     WCHAR *pathW;
 
-    TRACE("%p %s %lx %p\n", query, debugstr_a(path), userdata, counter);
+    TRACE("%p %s %Ix %p\n", query, debugstr_a(path), userdata, counter);
 
     if (!path) return PDH_INVALID_ARGUMENT;
 
@@ -236,7 +236,7 @@ PDH_STATUS WINAPI PdhAddCounterW( PDH_HQUERY hquery, LPCWSTR path,
     struct counter *counter;
     unsigned int i;
 
-    TRACE("%p %s %lx %p\n", hquery, debugstr_w(path), userdata, hcounter);
+    TRACE("%p %s %Ix %p\n", hquery, debugstr_w(path), userdata, hcounter);
 
     if (!path  || !hcounter) return PDH_INVALID_ARGUMENT;
 
@@ -282,9 +282,10 @@ PDH_STATUS WINAPI PdhAddCounterW( PDH_HQUERY hquery, LPCWSTR path,
 PDH_STATUS WINAPI PdhAddEnglishCounterA( PDH_HQUERY query, LPCSTR path,
                                          DWORD_PTR userdata, PDH_HCOUNTER *counter )
 {
-    TRACE("%p %s %lx %p\n", query, debugstr_a(path), userdata, counter);
+    TRACE("%p %s %Ix %p\n", query, debugstr_a(path), userdata, counter);
 
-    if (!query) return PDH_INVALID_ARGUMENT;
+    if (!counter) return PDH_INVALID_ARGUMENT;
+    if (!query) return PDH_INVALID_HANDLE;
     return PdhAddCounterA( query, path, userdata, counter );
 }
 
@@ -294,9 +295,10 @@ PDH_STATUS WINAPI PdhAddEnglishCounterA( PDH_HQUERY query, LPCSTR path,
 PDH_STATUS WINAPI PdhAddEnglishCounterW( PDH_HQUERY query, LPCWSTR path,
                                          DWORD_PTR userdata, PDH_HCOUNTER *counter )
 {
-    TRACE("%p %s %lx %p\n", query, debugstr_w(path), userdata, counter);
+    TRACE("%p %s %Ix %p\n", query, debugstr_w(path), userdata, counter);
 
-    if (!query) return PDH_INVALID_ARGUMENT;
+    if (!counter) return PDH_INVALID_ARGUMENT;
+    if (!query) return PDH_INVALID_HANDLE;
     return PdhAddCounterW( query, path, userdata, counter );
 }
 
@@ -324,7 +326,7 @@ static PDH_STATUS format_value( struct counter *counter, DWORD format, union val
     }
     else
     {
-        WARN("unknown format %x\n", format);
+        WARN("unknown format %lx\n", format);
         return PDH_INVALID_ARGUMENT;
     }
     return ERROR_SUCCESS;
@@ -340,7 +342,7 @@ PDH_STATUS WINAPI PdhCalculateCounterFromRawValue( PDH_HCOUNTER handle, DWORD fo
     PDH_STATUS ret;
     struct counter *counter = handle;
 
-    TRACE("%p 0x%08x %p %p %p\n", handle, format, raw1, raw2, value);
+    TRACE("%p 0x%08lx %p %p %p\n", handle, format, raw1, raw2, value);
 
     if (!value) return PDH_INVALID_ARGUMENT;
 
@@ -490,7 +492,7 @@ PDH_STATUS WINAPI PdhCollectQueryDataEx( PDH_HQUERY handle, DWORD interval, HAND
     PDH_STATUS ret;
     struct query *query = handle;
 
-    TRACE("%p %d %p\n", handle, interval, event);
+    TRACE("%p %ld %p\n", handle, interval, event);
 
     EnterCriticalSection( &pdh_handle_cs );
     if (!query || query->magic != PDH_MAGIC_QUERY)
@@ -582,7 +584,7 @@ PDH_STATUS WINAPI PdhCollectQueryDataWithTime( PDH_HQUERY handle, LONGLONG *time
  */
 PDH_STATUS WINAPI PdhExpandWildCardPathA( LPCSTR szDataSource, LPCSTR szWildCardPath, LPSTR mszExpandedPathList, LPDWORD pcchPathListLength, DWORD dwFlags )
 {
-    FIXME("%s, %s, %p, %p, 0x%x: stub\n", debugstr_a(szDataSource), debugstr_a(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
+    FIXME("%s, %s, %p, %p, 0x%lx: stub\n", debugstr_a(szDataSource), debugstr_a(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
     return PDH_NOT_IMPLEMENTED;
 }
 
@@ -591,7 +593,7 @@ PDH_STATUS WINAPI PdhExpandWildCardPathA( LPCSTR szDataSource, LPCSTR szWildCard
  */
 PDH_STATUS WINAPI PdhExpandWildCardPathW( LPCWSTR szDataSource, LPCWSTR szWildCardPath, LPWSTR mszExpandedPathList, LPDWORD pcchPathListLength, DWORD dwFlags )
 {
-    FIXME("%s, %s, %p, %p, 0x%x: stub\n", debugstr_w(szDataSource), debugstr_w(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
+    FIXME("%s, %s, %p, %p, 0x%lx: stub\n", debugstr_w(szDataSource), debugstr_w(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
     return PDH_NOT_IMPLEMENTED;
 }
 
@@ -743,7 +745,7 @@ PDH_STATUS WINAPI PdhGetFormattedCounterValue( PDH_HCOUNTER handle, DWORD format
     PDH_STATUS ret;
     struct counter *counter = handle;
 
-    TRACE("%p %x %p %p\n", handle, format, type, value);
+    TRACE("%p %lx %p %p\n", handle, format, type, value);
 
     if (!value) return PDH_INVALID_ARGUMENT;
 
@@ -862,7 +864,7 @@ PDH_STATUS WINAPI PdhLookupPerfNameByIndexA( LPCSTR machine, DWORD index, LPSTR
     WCHAR bufferW[PDH_MAX_COUNTER_NAME];
     DWORD sizeW = ARRAY_SIZE(bufferW);
 
-    TRACE("%s %d %p %p\n", debugstr_a(machine), index, buffer, size);
+    TRACE("%s %ld %p %p\n", debugstr_a(machine), index, buffer, size);
 
     if (!buffer || !size) return PDH_INVALID_ARGUMENT;
 
@@ -888,7 +890,7 @@ PDH_STATUS WINAPI PdhLookupPerfNameByIndexW( LPCWSTR machine, DWORD index, LPWST
     PDH_STATUS ret;
     unsigned int i;
 
-    TRACE("%s %d %p %p\n", debugstr_w(machine), index, buffer, size);
+    TRACE("%s %ld %p %p\n", debugstr_w(machine), index, buffer, size);
 
     if (machine)
     {
@@ -927,7 +929,7 @@ PDH_STATUS WINAPI PdhOpenQueryA( LPCSTR source, DWORD_PTR userdata, PDH_HQUERY *
     PDH_STATUS ret;
     WCHAR *sourceW = NULL;
 
-    TRACE("%s %lx %p\n", debugstr_a(source), userdata, query);
+    TRACE("%s %Ix %p\n", debugstr_a(source), userdata, query);
 
     if (source && !(sourceW = pdh_strdup_aw( source ))) return PDH_MEMORY_ALLOCATION_FAILURE;
 
@@ -944,7 +946,7 @@ PDH_STATUS WINAPI PdhOpenQueryW( LPCWSTR source, DWORD_PTR userdata, PDH_HQUERY
 {
     struct query *query;
 
-    TRACE("%s %lx %p\n", debugstr_w(source), userdata, handle);
+    TRACE("%s %Ix %p\n", debugstr_w(source), userdata, handle);
 
     if (!handle) return PDH_INVALID_ARGUMENT;
 
@@ -1110,7 +1112,7 @@ PDH_STATUS WINAPI PdhMakeCounterPathA( PDH_COUNTER_PATH_ELEMENTS_A *e, LPSTR buf
     WCHAR *bufferW;
     DWORD buflenW;
 
-    TRACE("%p %p %p 0x%08x\n", e, buffer, buflen, flags);
+    TRACE("%p %p %p 0x%08lx\n", e, buffer, buflen, flags);
 
     if (!e || !buflen) return PDH_INVALID_ARGUMENT;
 
@@ -1160,9 +1162,9 @@ PDH_STATUS WINAPI PdhMakeCounterPathW( PDH_COUNTER_PATH_ELEMENTS_W *e, LPWSTR bu
     PDH_STATUS ret = ERROR_SUCCESS;
     DWORD len;
 
-    TRACE("%p %p %p 0x%08x\n", e, buffer, buflen, flags);
+    TRACE("%p %p %p 0x%08lx\n", e, buffer, buflen, flags);
 
-    if (flags) FIXME("unimplemented flags 0x%08x\n", flags);
+    if (flags) FIXME("unimplemented flags 0x%08lx\n", flags);
 
     if (!e || !e->szCounterName || !e->szObjectName || !buflen)
         return PDH_INVALID_ARGUMENT;
@@ -1205,7 +1207,7 @@ PDH_STATUS WINAPI PdhEnumObjectItemsA(LPCSTR szDataSource, LPCSTR szMachineName,
                                       LPSTR mszCounterList, LPDWORD pcchCounterListLength, LPSTR mszInstanceList,
                                       LPDWORD pcchInstanceListLength, DWORD dwDetailLevel, DWORD dwFlags)
 {
-    FIXME("%s, %s, %s, %p, %p, %p, %p, %d, 0x%x: stub\n", debugstr_a(szDataSource), debugstr_a(szMachineName),
+    FIXME("%s, %s, %s, %p, %p, %p, %p, %ld, 0x%lx: stub\n", debugstr_a(szDataSource), debugstr_a(szMachineName),
          debugstr_a(szObjectName), mszCounterList, pcchCounterListLength, mszInstanceList,
          pcchInstanceListLength, dwDetailLevel, dwFlags);
 
@@ -1219,7 +1221,7 @@ PDH_STATUS WINAPI PdhEnumObjectItemsW(LPCWSTR szDataSource, LPCWSTR szMachineNam
                                       LPWSTR mszCounterList, LPDWORD pcchCounterListLength, LPWSTR mszInstanceList,
                                       LPDWORD pcchInstanceListLength, DWORD dwDetailLevel, DWORD dwFlags)
 {
-    FIXME("%s, %s, %s, %p, %p, %p, %p, %d, 0x%x: stub\n", debugstr_w(szDataSource), debugstr_w(szMachineName),
+    FIXME("%s, %s, %s, %p, %p, %p, %p, %ld, 0x%lx: stub\n", debugstr_w(szDataSource), debugstr_w(szMachineName),
          debugstr_w(szObjectName), mszCounterList, pcchCounterListLength, mszInstanceList,
          pcchInstanceListLength, dwDetailLevel, dwFlags);
 
@@ -1231,7 +1233,7 @@ PDH_STATUS WINAPI PdhEnumObjectItemsW(LPCWSTR szDataSource, LPCWSTR szMachineNam
  */
 PDH_STATUS WINAPI PdhSetDefaultRealTimeDataSource( DWORD source )
 {
-    FIXME("%u\n", source);
+    FIXME("%lu\n", source);
     return ERROR_SUCCESS;
 }
 
