@@ -44,11 +44,13 @@ struct host
     unsigned int client_edge : 1;
     unsigned int use_set_rect : 1;
     unsigned int use_back_colour : 1;
+    unsigned int defer_release : 1;
     PARAFORMAT2 para_fmt;
     DWORD props, scrollbars, event_mask;
     RECT client_rect, set_rect;
     COLORREF back_colour;
     WCHAR password_char;
+    unsigned int notify_level;
 };
 
 static const ITextHost2Vtbl textHostVtbl;
@@ -118,6 +120,8 @@ struct host *host_create( HWND hwnd, CREATESTRUCTW *cs, BOOL emulate_10 )
     GetClientRect( hwnd, &texthost->client_rect );
     texthost->use_back_colour = 0;
     texthost->password_char = (texthost->props & TXTBIT_USEPASSWORD) ? '*' : 0;
+    texthost->defer_release = 0;
+    texthost->notify_level = 0;
 
     return texthost;
 }
@@ -1080,7 +1084,7 @@ static LRESULT RichEditWndProc_common( HWND hwnd, UINT msg, WPARAM wparam,
     HRESULT hr = S_OK;
     LRESULT res = 0;
 
-    TRACE( "enter hwnd %p msg %04x (%s) %lx %lx, unicode %d\n",
+    TRACE( "enter hwnd %p msg %04x (%s) %Ix %Ix, unicode %d\n",
            hwnd, msg, get_msg_name(msg), wparam, lparam, unicode );
 
     host = (struct host *)GetWindowLongPtrW( hwnd, 0 );
@@ -1090,7 +1094,7 @@ static LRESULT RichEditWndProc_common( HWND hwnd, UINT msg, WPARAM wparam,
         {
             CREATESTRUCTW *pcs = (CREATESTRUCTW *)lparam;
 
-            TRACE( "WM_NCCREATE: hwnd %p style 0x%08x\n", hwnd, pcs->style );
+            TRACE( "WM_NCCREATE: hwnd %p style 0x%08lx\n", hwnd, pcs->style );
             return create_windowed_editor( hwnd, pcs, FALSE );
         }
         else return DefWindowProcW( hwnd, msg, wparam, lparam );
@@ -1098,12 +1102,24 @@ static LRESULT RichEditWndProc_common( HWND hwnd, UINT msg, WPARAM wparam,
 
     if ((((host->event_mask & ENM_KEYEVENTS) && msg >= WM_KEYFIRST && msg <= WM_KEYLAST) ||
          ((host->event_mask & ENM_MOUSEEVENTS) && msg >= WM_MOUSEFIRST && msg <= WM_MOUSELAST) ||
-         ((host->event_mask & ENM_SCROLLEVENTS) && msg >= WM_HSCROLL && msg <= WM_VSCROLL)) &&
-        send_msg_filter( host, msg, &wparam, &lparam ))
+         ((host->event_mask & ENM_SCROLLEVENTS) && msg >= WM_HSCROLL && msg <= WM_VSCROLL)))
     {
-        TRACE( "exit (filtered) hwnd %p msg %04x (%s) %lx %lx -> %lu\n",
-               hwnd, msg, get_msg_name(msg), wparam, lparam, res );
-        return res;
+        host->notify_level++;
+        res = send_msg_filter( host, msg, &wparam, &lparam );
+        if (!--host->notify_level && host->defer_release)
+        {
+            TRACE( "exit (filtered deferred release) hwnd %p msg %04x (%s) %Ix %Ix -> 0\n",
+                   hwnd, msg, get_msg_name(msg), wparam, lparam );
+            ITextHost2_Release( &host->ITextHost_iface );
+            return 0;
+        }
+
+        if (res)
+        {
+            TRACE( "exit (filtered %Iu) hwnd %p msg %04x (%s) %Ix %Ix -> 0\n",
+                   res, hwnd, msg, get_msg_name(msg), wparam, lparam );
+            return 0;
+        }
     }
 
     switch (msg)
@@ -1139,7 +1155,8 @@ static LRESULT RichEditWndProc_common( HWND hwnd, UINT msg, WPARAM wparam,
         break;
     }
     case WM_DESTROY:
-        ITextHost2_Release( &host->ITextHost_iface );
+        if (!host->notify_level) ITextHost2_Release( &host->ITextHost_iface );
+        else host->defer_release = 1;
         return 0;
 
     case WM_ERASEBKGND:
@@ -1480,7 +1497,7 @@ static LRESULT RichEditWndProc_common( HWND hwnd, UINT msg, WPARAM wparam,
     if (hr == S_FALSE)
         res = DefWindowProcW( hwnd, msg, wparam, lparam );
 
-    TRACE( "exit hwnd %p msg %04x (%s) %lx %lx, unicode %d -> %lu\n",
+    TRACE( "exit hwnd %p msg %04x (%s) %Ix %Ix, unicode %d -> %Iu\n",
            hwnd, msg, get_msg_name(msg), wparam, lparam, unicode, res );
 
     return res;
@@ -1519,7 +1536,7 @@ LRESULT WINAPI RichEdit10ANSIWndProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM
     {
         CREATESTRUCTW *pcs = (CREATESTRUCTW *)lparam;
 
-        TRACE( "WM_NCCREATE: hwnd %p style 0x%08x\n", hwnd, pcs->style );
+        TRACE( "WM_NCCREATE: hwnd %p style 0x%08lx\n", hwnd, pcs->style );
         return create_windowed_editor( hwnd, pcs, TRUE );
     }
     return RichEditANSIWndProc( hwnd, msg, wparam, lparam );
@@ -1528,7 +1545,7 @@ LRESULT WINAPI RichEdit10ANSIWndProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM
 static LRESULT WINAPI REComboWndProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
 {
     /* FIXME: Not implemented */
-    TRACE( "hwnd %p msg %04x (%s) %08lx %08lx\n",
+    TRACE( "hwnd %p msg %04x (%s) %08Ix %08Ix\n",
            hwnd, msg, get_msg_name( msg ), wparam, lparam );
     return DefWindowProcW( hwnd, msg, wparam, lparam );
 }
@@ -1536,7 +1553,7 @@ static LRESULT WINAPI REComboWndProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM
 static LRESULT WINAPI REListWndProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
 {
     /* FIXME: Not implemented */
-    TRACE( "hwnd %p msg %04x (%s) %08lx %08lx\n",
+    TRACE( "hwnd %p msg %04x (%s) %08Ix %08Ix\n",
            hwnd, msg, get_msg_name( msg ), wparam, lparam );
     return DefWindowProcW( hwnd, msg, wparam, lparam );
 }
