@@ -40,10 +40,11 @@
 
 #define INT_CODE 4198
 
-static const char *progname;
+static const char *progname, *client_test_name;
 static BOOL old_windows_version;
 
 static HANDLE stop_event, stop_wait_event;
+static PROCESS_INFORMATION client_info;
 
 static void (WINAPI *pNDRSContextMarshall2)(RPC_BINDING_HANDLE, NDR_SCONTEXT, void*, NDR_RUNDOWN, void*, ULONG);
 static NDR_SCONTEXT (WINAPI *pNDRSContextUnmarshall2)(RPC_BINDING_HANDLE, void*, ULONG, void*, ULONG);
@@ -147,6 +148,7 @@ static int (__cdecl *sum_array_ptr)(int (*a)[2]);
 static ctx_handle_t (__cdecl *get_handle)(void);
 static void (__cdecl *get_handle_by_ptr)(ctx_handle_t *r);
 static void (__cdecl *test_handle)(ctx_handle_t ctx_handle);
+static void (__cdecl *test_I_RpcBindingInqLocalClientPID)(unsigned int protseq, RPC_BINDING_HANDLE binding);
 
 #define SERVER_FUNCTIONS \
     X(int_return) \
@@ -240,7 +242,8 @@ static void (__cdecl *test_handle)(ctx_handle_t ctx_handle);
     X(sum_array_ptr) \
     X(get_handle) \
     X(get_handle_by_ptr) \
-    X(test_handle)
+    X(test_handle) \
+    X(test_I_RpcBindingInqLocalClientPID)
 
 /* type check statements generated in header file */
 fnprintf *p_printf = printf;
@@ -916,7 +919,7 @@ void __cdecl s_context_handle_test(void)
     /* marshal a context handle with NULL userContext */
     memset(buf, 0xcc, sizeof(buf));
     pNDRSContextMarshall2(binding, h, buf, NULL, NULL, 0);
-    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%x\n", *(ULONG *)buf);
+    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%lx\n", *(ULONG *)buf);
     ok(UuidIsNil((UUID *)&buf[4], &status), "uuid should have been nil\n");
 
     h = pNDRSContextUnmarshall2(binding, NULL, NDR_LOCAL_DATA_REPRESENTATION, NULL, 0);
@@ -926,7 +929,7 @@ void __cdecl s_context_handle_test(void)
     memset(buf, 0xcc, sizeof(buf));
     h->userContext = (void *)0xdeadbeef;
     pNDRSContextMarshall2(binding, h, buf, NULL, NULL, 0);
-    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%x\n", *(ULONG *)buf);
+    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%lx\n", *(ULONG *)buf);
     ok(!UuidIsNil((UUID *)&buf[4], &status), "uuid should not have been nil\n");
 
     /* raises ERROR_INVALID_HANDLE exception on Vista upwards */
@@ -943,7 +946,7 @@ void __cdecl s_context_handle_test(void)
     memset(buf, 0xcc, sizeof(buf));
     h->userContext = (void *)0xcafebabe;
     pNDRSContextMarshall2(binding, h, buf, NULL, &server_if.InterfaceId, 0);
-    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%x\n", *(ULONG *)buf);
+    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%lx\n", *(ULONG *)buf);
     ok(!UuidIsNil((UUID *)&buf[4], &status), "uuid should not have been nil\n");
 
     h = pNDRSContextUnmarshall2(binding, buf, NDR_LOCAL_DATA_REPRESENTATION, &server_if.InterfaceId, 0);
@@ -984,7 +987,7 @@ void __cdecl s_context_handle_test(void)
     binding = NULL;
     status = RpcBindingServerFromClient(NULL, &binding);
 
-    ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+    ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
     ok(binding != NULL, "binding is NULL\n");
 
     if (status == RPC_S_OK && binding != NULL)
@@ -997,11 +1000,11 @@ void __cdecl s_context_handle_test(void)
         unsigned char* network_options = NULL;
 
         status = RpcBindingToStringBindingA(binding, &string_binding);
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
         ok(string_binding != NULL, "string_binding is NULL\n");
 
         status = RpcStringBindingParseA(string_binding, &object_uuid, &protseq, &network_address, &endpoint, &network_options);
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
         ok(protseq != NULL && *protseq != '\0', "protseq is %s\n", protseq);
         ok(network_address != NULL && *network_address != '\0', "network_address is %s\n", network_address);
 
@@ -1085,7 +1088,7 @@ void __cdecl s_stop_autolisten(void)
     RPC_STATUS status;
     status = RpcServerUnregisterIf(NULL, NULL, FALSE);
     todo_wine
-    ok(status == RPC_S_UNKNOWN_MGR_TYPE, "got %u\n", status);
+    ok(status == RPC_S_UNKNOWN_MGR_TYPE, "got %lu\n", status);
 }
 
 void __cdecl s_ip_test(ipu_t *a)
@@ -1094,7 +1097,7 @@ void __cdecl s_ip_test(ipu_t *a)
     HRESULT hr;
 
     hr = IStream_Stat(a->tagged_union.stream, &st, STATFLAG_NONAME);
-    ok(hr == S_OK, "got %#x\n", hr);
+    ok(hr == S_OK, "got %#lx\n", hr);
 }
 
 int __cdecl s_sum_ptr_array(int *a[2])
@@ -1122,6 +1125,81 @@ void __cdecl s_test_handle(ctx_handle_t ctx_handle)
     ok(ctx_handle == (ctx_handle_t)0xdeadbeef, "Unexpected ctx_handle %p\n", ctx_handle);
 }
 
+struct test_thread_params
+{
+    unsigned int protseq;
+    RPC_BINDING_HANDLE binding;
+};
+
+static DWORD CALLBACK test_I_RpcBindingInqLocalClientPID_thread_func(void *args)
+{
+    struct test_thread_params *params = (struct test_thread_params *)args;
+    RPC_STATUS status;
+    ULONG pid;
+
+    winetest_push_context("%s", client_test_name);
+
+    status = I_RpcBindingInqLocalClientPID(NULL, &pid);
+    ok(status == RPC_S_NO_CALL_ACTIVE, "Got unexpected %ld.\n", status);
+
+    /* Other protocol sequences throw exceptions */
+    if (params->protseq == RPC_PROTSEQ_LRPC)
+    {
+        status = I_RpcBindingInqLocalClientPID(params->binding, &pid);
+        ok(status == RPC_S_OK, "Got unexpected %ld.\n", status);
+        ok(pid == client_info.dwProcessId, "Got unexpected pid.\n");
+    }
+
+    winetest_pop_context();
+    return 0;
+}
+
+void __cdecl s_test_I_RpcBindingInqLocalClientPID(unsigned int protseq, RPC_BINDING_HANDLE binding)
+{
+    struct test_thread_params params;
+    RPC_STATUS status;
+    HANDLE thread;
+    ULONG pid;
+
+    winetest_push_context("%s", client_test_name);
+
+    /* Crash on Windows */
+    if (0)
+    {
+    status = I_RpcBindingInqLocalClientPID(NULL, NULL);
+    ok(status == RPC_S_INVALID_ARG, "Got unexpected %ld.\n", status);
+
+    status = I_RpcBindingInqLocalClientPID(binding, NULL);
+    ok(status == RPC_S_INVALID_ARG, "Got unexpected %ld.\n", status);
+    }
+
+    status = I_RpcBindingInqLocalClientPID(NULL, &pid);
+    if (protseq == RPC_PROTSEQ_LRPC)
+    {
+        ok(status == RPC_S_OK, "Got unexpected %ld.\n", status);
+        ok(pid == client_info.dwProcessId, "Got unexpected pid.\n");
+    }
+    else
+    {
+        ok(status == RPC_S_INVALID_BINDING, "Got unexpected %ld.\n", status);
+    }
+
+    if (protseq == RPC_PROTSEQ_LRPC) /* Other protocol sequences throw exceptions */
+    {
+        status = I_RpcBindingInqLocalClientPID(binding, &pid);
+        ok(status == RPC_S_OK, "Got unexpected %ld.\n", status);
+        ok(pid == client_info.dwProcessId, "Got unexpected pid.\n");
+    }
+
+    params.protseq = protseq;
+    params.binding = binding;
+    thread = CreateThread(NULL, 0, test_I_RpcBindingInqLocalClientPID_thread_func, &params, 0, NULL);
+    WaitForSingleObject(thread, INFINITE);
+    CloseHandle(thread);
+
+    winetest_pop_context();
+}
+
 void __RPC_USER ctx_handle_t_rundown(ctx_handle_t ctx_handle)
 {
     ok(ctx_handle == (ctx_handle_t)0xdeadbeef, "Unexpected ctx_handle %p\n", ctx_handle);
@@ -1137,17 +1215,17 @@ static void
 run_client(const char *test)
 {
   char cmdline[MAX_PATH];
-  PROCESS_INFORMATION info;
   STARTUPINFOA startup;
 
   memset(&startup, 0, sizeof startup);
   startup.cb = sizeof startup;
 
+  client_test_name = test;
   make_cmdline(cmdline, test);
-  ok(CreateProcessA(NULL, cmdline, NULL, NULL, FALSE, 0L, NULL, NULL, &startup, &info), "CreateProcess\n");
-  wait_child_process( info.hProcess );
-  ok(CloseHandle(info.hProcess), "CloseHandle\n");
-  ok(CloseHandle(info.hThread), "CloseHandle\n");
+  ok(CreateProcessA(NULL, cmdline, NULL, NULL, FALSE, 0L, NULL, NULL, &startup, &client_info), "CreateProcess\n");
+  wait_child_process(client_info.hProcess);
+  ok(CloseHandle(client_info.hProcess), "CloseHandle\n");
+  ok(CloseHandle(client_info.hThread), "CloseHandle\n");
 }
 
 static void
@@ -1377,7 +1455,7 @@ union_tests(void)
   CreateStreamOnHGlobal(NULL, TRUE, &ipu.tagged_union.stream);
   ip_test(&ipu);
   ref = IStream_Release(ipu.tagged_union.stream);
-  ok(!ref, "got %u refs\n", ref);
+  ok(!ref, "got %lu refs\n", ref);
 
   CoUninitialize();
 }
@@ -1837,7 +1915,7 @@ void __cdecl s_authinfo_test(unsigned int protseq, int secure)
             win_skip("RpcBindingInqAuthClientA not supported\n");
             return;
         }
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
         ok(privs != (RPC_AUTHZ_HANDLE)0xdeadbeef, "privs unchanged\n");
         ok(principal != (unsigned char *)0xdeadbeef, "principal unchanged\n");
         if (protseq != RPC_PROTSEQ_LRPC)
@@ -1862,27 +1940,27 @@ void __cdecl s_authinfo_test(unsigned int protseq, int secure)
         RpcStringFreeA(&principal);
 
         status = RpcBindingInqAuthClientA(NULL, &privs, &principal, &level, &authnsvc, NULL);
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
         RpcStringFreeA(&principal);
 
         status = RpcBindingInqAuthClientExA(NULL, &privs, &principal, &level, &authnsvc, NULL, 0);
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
         RpcStringFreeA(&principal);
 
         status = RpcImpersonateClient(NULL);
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
         status = RpcRevertToSelf();
-        ok(status == RPC_S_OK, "expected RPC_S_OK got %u\n", status);
+        ok(status == RPC_S_OK, "expected RPC_S_OK got %lu\n", status);
 
     }
     else
     {
         status = RpcBindingInqAuthClientA(binding, &privs, &principal, &level, &authnsvc, NULL);
-        ok(status == RPC_S_BINDING_HAS_NO_AUTH, "expected RPC_S_BINDING_HAS_NO_AUTH got %u\n", status);
+        ok(status == RPC_S_BINDING_HAS_NO_AUTH, "expected RPC_S_BINDING_HAS_NO_AUTH got %lu\n", status);
         ok(privs == (RPC_AUTHZ_HANDLE)0xdeadbeef, "got %p\n", privs);
         ok(principal == (unsigned char *)0xdeadbeef, "got %s\n", principal);
-        ok(level == 0xdeadbeef, "got %u\n", level);
-        ok(authnsvc == 0xdeadbeef, "got %u\n", authnsvc);
+        ok(level == 0xdeadbeef, "got %lu\n", level);
+        ok(authnsvc == 0xdeadbeef, "got %lu\n", authnsvc);
     }
 }
 
@@ -1920,7 +1998,7 @@ set_auth_info(RPC_BINDING_HANDLE handle)
 
     status = pRpcBindingSetAuthInfoExA(handle, (RPC_CSTR)domain_and_user, RPC_C_AUTHN_LEVEL_PKT_PRIVACY,
                                        RPC_C_AUTHN_WINNT, NULL, 0, &qos);
-    ok(status == RPC_S_OK, "RpcBindingSetAuthInfoExA failed %d\n", status);
+    ok(status == RPC_S_OK, "RpcBindingSetAuthInfoExA failed %ld\n", status);
 }
 
 #define test_is_server_listening(a,b) _test_is_server_listening(__LINE__,a,b)
@@ -1928,7 +2006,7 @@ static void _test_is_server_listening(unsigned line, RPC_BINDING_HANDLE binding,
 {
     RPC_STATUS status;
     status = RpcMgmtIsServerListening(binding);
-    ok_(__FILE__,line)(status == expected_status, "RpcMgmtIsServerListening returned %u, expected %u\n",
+    ok_(__FILE__,line)(status == expected_status, "RpcMgmtIsServerListening returned %lu, expected %lu\n",
                        status, expected_status);
 }
 
@@ -1939,7 +2017,7 @@ static void _test_is_server_listening2(unsigned line, RPC_BINDING_HANDLE binding
     RPC_STATUS status;
     status = RpcMgmtIsServerListening(binding);
     ok_(__FILE__,line)(status == expected_status || status == expected_status2,
-                       "RpcMgmtIsServerListening returned %u, expected %u or %u\n",
+                       "RpcMgmtIsServerListening returned %lu, expected %lu or %lu\n",
                        status, expected_status, expected_status2);
 }
 
@@ -1964,6 +2042,7 @@ client(const char *test)
 
     run_tests();
     authinfo_test(RPC_PROTSEQ_TCP, 0);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_TCP, IMixedServer_IfHandle);
     test_is_server_listening2(IMixedServer_IfHandle, RPC_S_OK, RPC_S_ACCESS_DENIED);
 
     ok(RPC_S_OK == RpcStringFreeA(&binding), "RpcStringFree\n");
@@ -1976,6 +2055,7 @@ client(const char *test)
 
     set_auth_info(IMixedServer_IfHandle);
     authinfo_test(RPC_PROTSEQ_TCP, 1);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_TCP, IMixedServer_IfHandle);
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_ACCESS_DENIED);
 
     ok(RPC_S_OK == RpcStringFreeA(&binding), "RpcStringFree\n");
@@ -1988,6 +2068,7 @@ client(const char *test)
 
     run_tests(); /* can cause RPC_X_BAD_STUB_DATA exception */
     authinfo_test(RPC_PROTSEQ_LRPC, 0);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_LRPC, IMixedServer_IfHandle);
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_OK);
 
     ok(RPC_S_OK == RpcStringFreeA(&binding), "RpcStringFree\n");
@@ -2000,6 +2081,7 @@ client(const char *test)
 
     run_tests();
     authinfo_test(RPC_PROTSEQ_LRPC, 0);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_LRPC, IMixedServer_IfHandle);
     todo_wine
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_NOT_LISTENING);
 
@@ -2016,6 +2098,7 @@ client(const char *test)
 
     set_auth_info(IMixedServer_IfHandle);
     authinfo_test(RPC_PROTSEQ_LRPC, 1);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_LRPC, IMixedServer_IfHandle);
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_OK);
 
     ok(RPC_S_OK == RpcStringFreeA(&binding), "RpcStringFree\n");
@@ -2029,6 +2112,7 @@ client(const char *test)
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_OK);
     run_tests();
     authinfo_test(RPC_PROTSEQ_NMP, 0);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_NMP, IMixedServer_IfHandle);
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_OK);
     stop();
     test_is_server_listening(IMixedServer_IfHandle, RPC_S_NOT_LISTENING);
@@ -2046,6 +2130,7 @@ client(const char *test)
     test_is_server_listening(IInterpServer_IfHandle, RPC_S_OK);
     run_tests();
     authinfo_test(RPC_PROTSEQ_NMP, 0);
+    test_I_RpcBindingInqLocalClientPID(RPC_PROTSEQ_NMP, IInterpServer_IfHandle);
     test_is_server_listening(IInterpServer_IfHandle, RPC_S_OK);
 
     ok(RPC_S_OK == RpcStringFreeA(&binding), "RpcStringFree\n");
@@ -2069,16 +2154,16 @@ server(void)
   CoInitializeEx(NULL, COINIT_MULTITHREADED);
 
   iptcp_status = RpcServerUseProtseqEpA(iptcp, 20, port, NULL);
-  ok(iptcp_status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_ip_tcp) failed with status %d\n", iptcp_status);
+  ok(iptcp_status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_ip_tcp) failed with status %ld\n", iptcp_status);
 
   ncalrpc_status = RpcServerUseProtseqEpA(ncalrpc, 0, guid, NULL);
-  ok(ncalrpc_status == RPC_S_OK, "RpcServerUseProtseqEp(ncalrpc) failed with status %d\n", ncalrpc_status);
+  ok(ncalrpc_status == RPC_S_OK, "RpcServerUseProtseqEp(ncalrpc) failed with status %ld\n", ncalrpc_status);
 
   np_status = RpcServerUseProtseqEpA(np, 0, pipe, NULL);
   if (np_status == RPC_S_PROTSEQ_NOT_SUPPORTED)
     skip("Protocol sequence ncacn_np is not supported\n");
   else
-    ok(np_status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_np) failed with status %d\n", np_status);
+    ok(np_status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_np) failed with status %ld\n", np_status);
 
   if (pRpcServerRegisterIfEx)
   {
@@ -2086,25 +2171,25 @@ server(void)
     status = pRpcServerRegisterIfEx(s_IMixedServer_v0_0_s_ifspec, NULL, NULL,
                                     RPC_IF_ALLOW_CALLBACKS_WITH_NO_AUTH,
                                     RPC_C_LISTEN_MAX_CALLS_DEFAULT, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIfEx failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIfEx failed with status %ld\n", status);
     status = pRpcServerRegisterIfEx(s_IInterpServer_v0_0_s_ifspec, NULL, NULL,
                                     RPC_IF_ALLOW_CALLBACKS_WITH_NO_AUTH,
                                     RPC_C_LISTEN_MAX_CALLS_DEFAULT, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIfEx failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIfEx failed with status %ld\n", status);
   }
   else
   {
     status = RpcServerRegisterIf(s_IMixedServer_v0_0_s_ifspec, NULL, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %ld\n", status);
     status = RpcServerRegisterIf(s_IInterpServer_v0_0_s_ifspec, NULL, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %ld\n", status);
   }
   test_is_server_listening(NULL, RPC_S_NOT_LISTENING);
   status = RpcServerListen(1, RPC_C_LISTEN_MAX_CALLS_DEFAULT, TRUE);
-  ok(status == RPC_S_OK, "RpcServerListen failed with status %d\n", status);
+  ok(status == RPC_S_OK, "RpcServerListen failed with status %ld\n", status);
   test_is_server_listening(NULL, RPC_S_OK);
   stop_event = CreateEventW(NULL, FALSE, FALSE, NULL);
-  ok(stop_event != NULL, "CreateEvent failed with error %d\n", GetLastError());
+  ok(stop_event != NULL, "CreateEvent failed with error %ld\n", GetLastError());
 
   if (iptcp_status == RPC_S_OK)
     run_client("tcp_basic");
@@ -2141,7 +2226,7 @@ server(void)
   if (ret == WAIT_OBJECT_0)
   {
     status = RpcMgmtWaitServerListen();
-    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %ld\n", status);
   }
 
   CloseHandle(stop_event);
@@ -2152,12 +2237,12 @@ server(void)
     status = pRpcServerRegisterIfEx(s_IMixedServer_v0_0_s_ifspec, NULL, NULL,
         RPC_IF_ALLOW_CALLBACKS_WITH_NO_AUTH | RPC_IF_AUTOLISTEN,
         RPC_C_LISTEN_MAX_CALLS_DEFAULT, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIf() failed: %u\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIf() failed: %lu\n", status);
 
     run_client("ncalrpc_autolisten");
 
     status = RpcServerUnregisterIf(s_IMixedServer_v0_0_s_ifspec, NULL, TRUE);
-    ok(status == RPC_S_OK, "RpcServerUnregisterIf() failed: %u\n", status);
+    ok(status == RPC_S_OK, "RpcServerUnregisterIf() failed: %lu\n", status);
   }
 
   CoUninitialize();
@@ -2185,7 +2270,7 @@ static DWORD WINAPI wait_listen_proc(void *arg)
 
     trace("waiting\n");
     status = RpcMgmtWaitServerListen();
-    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %ld\n", status);
     trace("done\n");
 
     return 0;
@@ -2198,13 +2283,13 @@ static void test_stop_wait_for_call(unsigned char *binding)
     DWORD ret;
 
     status = RpcServerListen(1, RPC_C_LISTEN_MAX_CALLS_DEFAULT, TRUE);
-    ok(status == RPC_S_OK, "RpcServerListen failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerListen failed with status %ld\n", status);
     test_is_server_listening(NULL, RPC_S_OK);
 
     stop_wait_event = CreateEventW(NULL, FALSE, FALSE, NULL);
-    ok(stop_wait_event != NULL, "CreateEvent failed with error %d\n", GetLastError());
+    ok(stop_wait_event != NULL, "CreateEvent failed with error %ld\n", GetLastError());
     stop_event = CreateEventW(NULL, FALSE, FALSE, NULL);
-    ok(stop_event != NULL, "CreateEvent failed with error %d\n", GetLastError());
+    ok(stop_event != NULL, "CreateEvent failed with error %ld\n", GetLastError());
 
     wait_listen_thread = CreateThread(NULL, 0, wait_listen_proc, 0, 0, NULL);
     ok(wait_listen_thread != NULL, "CreateThread failed\n");
@@ -2226,7 +2311,7 @@ static void test_stop_wait_for_call(unsigned char *binding)
     SetEvent(stop_wait_event);
 
     ret = WaitForSingleObject(wait_listen_thread, 10000);
-    ok(WAIT_OBJECT_0 == ret, "WaitForSingleObject returned %u\n", ret);
+    ok(WAIT_OBJECT_0 == ret, "WaitForSingleObject returned %lu\n", ret);
 
     CloseHandle(wait_listen_thread);
 
@@ -2247,28 +2332,28 @@ static void test_server_listening(void)
     RPC_STATUS status;
 
     status = RpcServerUseProtseqEpA(np, 0, pipe, NULL);
-    ok(status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_np) failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_np) failed with status %ld\n", status);
 
     status = RpcServerRegisterIf(s_IMixedServer_v0_0_s_ifspec, NULL, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %ld\n", status);
 
     test_is_server_listening(NULL, RPC_S_NOT_LISTENING);
     status = RpcServerListen(1, RPC_C_LISTEN_MAX_CALLS_DEFAULT, TRUE);
-    ok(status == RPC_S_OK, "RpcServerListen failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerListen failed with status %ld\n", status);
     test_is_server_listening(NULL, RPC_S_OK);
 
     status = RpcServerListen(1, RPC_C_LISTEN_MAX_CALLS_DEFAULT, TRUE);
-    ok(status == RPC_S_ALREADY_LISTENING, "RpcServerListen failed with status %d\n", status);
+    ok(status == RPC_S_ALREADY_LISTENING, "RpcServerListen failed with status %ld\n", status);
 
     status = RpcMgmtStopServerListening(NULL);
     ok(status == RPC_S_OK, "RpcMgmtStopServerListening\n");
     test_is_server_listening(NULL, RPC_S_NOT_LISTENING);
 
     status = RpcMgmtWaitServerListen();
-    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %ld\n", status);
 
     status = RpcMgmtWaitServerListen();
-    ok(status == RPC_S_NOT_LISTENING, "RpcMgmtWaitServerListening failed with status %d\n", status);
+    ok(status == RPC_S_NOT_LISTENING, "RpcMgmtWaitServerListening failed with status %ld\n", status);
 
     /* test that server stop waits for a call in progress */
     status = RpcStringBindingComposeA(NULL, np, address_np, pipe, NULL, &binding);
@@ -2281,7 +2366,7 @@ static void test_server_listening(void)
 
     /* repeat the test using ncalrpc */
     status = RpcServerUseProtseqEpA(ncalrpc, 0, guid, NULL);
-    ok(status == RPC_S_OK, "RpcServerUseProtseqEp(ncalrpc) failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerUseProtseqEp(ncalrpc) failed with status %ld\n", status);
 
     status = RpcStringBindingComposeA(NULL, ncalrpc, NULL, guid, NULL, &binding);
     ok(status == RPC_S_OK, "RpcStringBindingCompose\n");
@@ -2305,9 +2390,9 @@ static HANDLE create_server_process(void)
     startup.cb = sizeof startup;
 
     ready_event = CreateEventW(&sec_attr, TRUE, FALSE, NULL);
-    ok(ready_event != NULL, "CreateEvent failed: %u\n", GetLastError());
+    ok(ready_event != NULL, "CreateEvent failed: %lu\n", GetLastError());
 
-    sprintf(cmdline, "%s server run %lx", progname, (UINT_PTR)ready_event);
+    sprintf(cmdline, "%s server run %Ix", progname, (UINT_PTR)ready_event);
     trace("running server process...\n");
     ok(CreateProcessA(NULL, cmdline, NULL, NULL, TRUE, 0L, NULL, NULL, &startup, &info), "CreateProcess\n");
     ret = WaitForSingleObject(ready_event, 10000);
@@ -2326,26 +2411,26 @@ static void run_server(HANDLE ready_event)
     BOOL ret;
 
     status = RpcServerUseProtseqEpA(np, 0, pipe, NULL);
-    ok(status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_np) failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_np) failed with status %ld\n", status);
 
     status = RpcServerRegisterIf(s_IMixedServer_v0_0_s_ifspec, NULL, NULL);
-    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerRegisterIf failed with status %ld\n", status);
 
     test_is_server_listening(NULL, RPC_S_NOT_LISTENING);
     status = RpcServerListen(1, RPC_C_LISTEN_MAX_CALLS_DEFAULT, TRUE);
-    ok(status == RPC_S_OK, "RpcServerListen failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcServerListen failed with status %ld\n", status);
 
     stop_event = CreateEventW(NULL, FALSE, FALSE, NULL);
-    ok(stop_event != NULL, "CreateEvent failed with error %d\n", GetLastError());
+    ok(stop_event != NULL, "CreateEvent failed with error %ld\n", GetLastError());
 
     ret = SetEvent(ready_event);
-    ok(ret, "SetEvent failed: %u\n", GetLastError());
+    ok(ret, "SetEvent failed: %lu\n", GetLastError());
 
     ret = WaitForSingleObject(stop_event, 5000);
     ok(WAIT_OBJECT_0 == ret, "WaitForSingleObject\n");
 
     status = RpcMgmtWaitServerListen();
-    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %d\n", status);
+    ok(status == RPC_S_OK, "RpcMgmtWaitServerListening failed with status %ld\n", status);
 
     CloseHandle(stop_event);
     stop_event = NULL;
@@ -2376,7 +2461,7 @@ static void test_reconnect(void)
     for (i = 0; i < ARRAY_SIZE(threads); i++)
     {
         threads[i] = CreateThread(NULL, 0, basic_tests_thread, 0, 0, NULL);
-        ok(threads[i] != NULL, "CreateThread failed: %u\n", GetLastError());
+        ok(threads[i] != NULL, "CreateThread failed: %lu\n", GetLastError());
     }
 
     for (i = 0; i < ARRAY_SIZE(threads); i++)
@@ -2432,18 +2517,18 @@ static BOOL is_firewall_enabled(void)
 
     hr = CoCreateInstance( &CLSID_NetFwMgr, NULL, CLSCTX_INPROC_SERVER, &IID_INetFwMgr,
                            (void **)&mgr );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     hr = INetFwMgr_get_LocalPolicy( mgr, &policy );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     hr = INetFwPolicy_get_CurrentProfile( policy, &profile );
     if (hr != S_OK) goto done;
 
     hr = INetFwProfile_get_FirewallEnabled( profile, &enabled );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
 
 done:
     if (policy) INetFwPolicy_Release( policy );
@@ -2478,23 +2563,23 @@ static HRESULT set_firewall( enum firewall_op op )
 
     hr = CoCreateInstance( &CLSID_NetFwMgr, NULL, CLSCTX_INPROC_SERVER, &IID_INetFwMgr,
                            (void **)&mgr );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     hr = INetFwMgr_get_LocalPolicy( mgr, &policy );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     hr = INetFwPolicy_get_CurrentProfile( policy, &profile );
     if (hr != S_OK) goto done;
 
     hr = INetFwProfile_get_AuthorizedApplications( profile, &apps );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     hr = CoCreateInstance( &CLSID_NetFwAuthorizedApplication, NULL, CLSCTX_INPROC_SERVER,
                            &IID_INetFwAuthorizedApplication, (void **)&app );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     hr = INetFwAuthorizedApplication_put_ProcessImageFileName( app, image );
@@ -2503,7 +2588,7 @@ static HRESULT set_firewall( enum firewall_op op )
     name = SysAllocString( L"rpcrt4_test" );
     hr = INetFwAuthorizedApplication_put_Name( app, name );
     SysFreeString( name );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %08lx\n", hr );
     if (hr != S_OK) goto done;
 
     if (op == APP_ADD)
@@ -2562,7 +2647,7 @@ START_TEST(server)
     else if(!strcmp(argv[2], "run"))
     {
       ULONG event;
-      sscanf(argv[3], "%x", &event);
+      sscanf(argv[3], "%lx", &event);
       run_server(ULongToHandle(event));
     }
   }
@@ -2580,7 +2665,7 @@ START_TEST(server)
         }
         else
         {
-          skip("can't authorize app in firewall %08x\n", hr);
+          skip("can't authorize app in firewall %08lx\n", hr);
         }
       }
       else
