@@ -1288,6 +1288,8 @@ static DWORD WINAPI notify_thread(void *user)
     SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2 *cparams;
     BOOL dummy;
 
+    SetThreadDescription(GetCurrentThread(), L"wine_sechost_notify_service_status");
+
     __TRY
     {
         /* GetNotifyResults blocks until there is an event */
@@ -1585,6 +1587,7 @@ static DWORD WINAPI service_thread( void *arg )
     DWORD argc = 0, len = 0;
 
     TRACE("%p\n", arg);
+    SetThreadDescription(GetCurrentThread(), L"wine_sechost_service");
 
     while (str[len])
     {
@@ -2021,6 +2024,8 @@ static DWORD WINAPI device_notify_proc( void *arg )
     unsigned int i, size;
     BYTE *buf;
 
+    SetThreadDescription( GetCurrentThread(), L"wine_sechost_device_notify" );
+
     if ((err = RpcStringBindingComposeW( NULL, protseq, NULL, endpoint, NULL, &binding_str )))
     {
         ERR("RpcStringBindingCompose() failed, error %#lx\n", err);
