@@ -136,6 +136,11 @@ MAKE_FUNCPTR(gnutls_x509_privkey_deinit);
 #define GNUTLS_KX_ECDHE_PSK     14
 #endif
 
+#if GNUTLS_VERSION_MAJOR < 3 || (GNUTLS_VERSION_MAJOR == 3 && GNUTLS_VERSION_MINOR < 4)
+#define GNUTLS_CIPHER_AES_128_CCM 19
+#define GNUTLS_CIPHER_AES_256_CCM 20
+#endif
+
 #if GNUTLS_VERSION_MAJOR < 3 || (GNUTLS_VERSION_MAJOR == 3 && GNUTLS_VERSION_MINOR < 5)
 #define GNUTLS_ALPN_SERVER_PRECEDENCE (1<<1)
 #endif
