@@ -723,7 +723,7 @@ BOOL WINAPI SetupQueueRenameA( HSPFILEQ handle, PCSTR SourcePath, PCSTR SourceFi
     if (!(op = heap_alloc_zero( sizeof(*op) ))) return FALSE;
     op->src_path = strdupAtoW( SourcePath );
     op->src_file = strdupAtoW( SourceFilename );
-    op->dst_path = strdupAtoW( TargetPath );
+    op->dst_path = strdupAtoW( TargetPath ? TargetPath : SourcePath );
     op->dst_file = strdupAtoW( TargetFilename );
     queue_file_op( &queue->rename_queue, op );
     return TRUE;
@@ -742,7 +742,7 @@ BOOL WINAPI SetupQueueRenameW( HSPFILEQ handle, PCWSTR SourcePath, PCWSTR Source
     if (!(op = heap_alloc_zero( sizeof(*op) ))) return FALSE;
     op->src_path = strdupW( SourcePath );
     op->src_file = strdupW( SourceFilename );
-    op->dst_path = strdupW( TargetPath );
+    op->dst_path = strdupW( TargetPath ? TargetPath : SourcePath );
     op->dst_file = strdupW( TargetFilename );
     queue_file_op( &queue->rename_queue, op );
     return TRUE;
@@ -792,7 +792,7 @@ BOOL WINAPI SetupQueueCopySectionW( HSPFILEQ queue, PCWSTR src_root, HINF hinf,
     BOOL ret = FALSE;
     DWORD len;
 
-    TRACE("queue %p, src_root %s, hinf %p, hlist %p, section %s, style %#x.\n",
+    TRACE("queue %p, src_root %s, hinf %p, hlist %p, section %s, style %#lx.\n",
             queue, debugstr_w(src_root), hinf, hlist, debugstr_w(section), style);
 
     if (!src_root)
@@ -1068,7 +1068,7 @@ static BOOL do_file_copyW( LPCWSTR source, LPCWSTR target, DWORD style,
     BOOL rc = FALSE;
     BOOL docopy = TRUE;
 
-    TRACE("copy %s to %s style 0x%x\n",debugstr_w(source),debugstr_w(target),style);
+    TRACE("copy %s to %s style 0x%lx\n",debugstr_w(source),debugstr_w(target),style);
 
     /* before copy processing */
     if (style & SP_COPY_REPLACEONLY)
@@ -1126,7 +1126,7 @@ static BOOL do_file_copyW( LPCWSTR source, LPCWSTR target, DWORD style,
                 {
                     FILEPATHS_W filepaths;
 
-                    TRACE("Versions: Source %i.%i target %i.%i\n",
+                    TRACE("Versions: Source %li.%li target %li.%li\n",
                       SourceInfo->dwFileVersionMS, SourceInfo->dwFileVersionLS,
                       TargetInfo->dwFileVersionMS, TargetInfo->dwFileVersionLS);
 
@@ -1179,7 +1179,7 @@ static BOOL do_file_copyW( LPCWSTR source, LPCWSTR target, DWORD style,
     if (style & (SP_COPY_NODECOMP | SP_COPY_LANGUAGEAWARE | SP_COPY_FORCE_IN_USE |
                  SP_COPY_NOSKIP | SP_COPY_WARNIFSKIP))
     {
-        ERR("Unsupported style(s) 0x%x\n",style);
+        ERR("Unsupported style(s) 0x%lx\n",style);
     }
 
     if (docopy)
@@ -1201,7 +1201,7 @@ static BOOL do_file_copyW( LPCWSTR source, LPCWSTR target, DWORD style,
                     DeleteFileW(temp_file);
             }
         }
-        if (!rc) WARN( "failed to copy, err %u\n", GetLastError() );
+        if (!rc) WARN( "failed to copy, err %lu\n", GetLastError() );
     }
     else
         SetLastError(ERROR_SUCCESS);
@@ -1226,7 +1226,7 @@ BOOL WINAPI SetupInstallFileExA( HINF hinf, PINFCONTEXT inf_context, PCSTR sourc
     struct callback_WtoA_context ctx;
     UNICODE_STRING sourceW, rootW, destW;
 
-    TRACE("%p %p %s %s %s %x %p %p %p\n", hinf, inf_context, debugstr_a(source), debugstr_a(root),
+    TRACE("%p %p %s %s %s %lx %p %p %p\n", hinf, inf_context, debugstr_a(source), debugstr_a(root),
           debugstr_a(dest), style, handler, context, in_use);
 
     sourceW.Buffer = rootW.Buffer = destW.Buffer = NULL;
@@ -1275,9 +1275,9 @@ BOOL WINAPI SetupInstallFileExW( HINF hinf, PINFCONTEXT inf_context, PCWSTR sour
 {
     BOOL ret, absolute = (root && *root && !(style & SP_COPY_SOURCE_ABSOLUTE));
     WCHAR *buffer, *p, *inf_source = NULL, dest_path[MAX_PATH];
-    unsigned int len;
+    DWORD len;
 
-    TRACE("%p %p %s %s %s %x %p %p %p\n", hinf, inf_context, debugstr_w(source), debugstr_w(root),
+    TRACE("%p %p %s %s %s %lx %p %p %p\n", hinf, inf_context, debugstr_w(source), debugstr_w(root),
           debugstr_w(dest), style, handler, context, in_use);
 
     if (in_use) FIXME("no file in use support\n");
@@ -1598,7 +1598,7 @@ BOOL WINAPI SetupScanFileQueueA( HSPFILEQ handle, DWORD flags, HWND window,
 {
     struct callback_WtoA_context ctx;
 
-    TRACE("%p %x %p %p %p %p\n", handle, flags, window, handler, context, result);
+    TRACE("%p %lx %p %p %p %p\n", handle, flags, window, handler, context, result);
 
     ctx.orig_context = context;
     ctx.orig_handler = handler;
@@ -1619,7 +1619,7 @@ BOOL WINAPI SetupScanFileQueueW( HSPFILEQ handle, DWORD flags, HWND window,
     UINT notification = 0;
     BOOL ret = FALSE;
 
-    TRACE("%p %x %p %p %p %p\n", handle, flags, window, handler, context, result);
+    TRACE("%p %lx %p %p %p %p\n", handle, flags, window, handler, context, result);
 
     if (!queue->copy_queue.count) return TRUE;
 
@@ -1628,7 +1628,7 @@ BOOL WINAPI SetupScanFileQueueW( HSPFILEQ handle, DWORD flags, HWND window,
 
     if (flags & ~(SPQ_SCAN_USE_CALLBACK | SPQ_SCAN_USE_CALLBACKEX))
     {
-        FIXME("flags %x not fully implemented\n", flags);
+        FIXME("flags %lx not fully implemented\n", flags);
     }
 
     paths.Source = paths.Target = NULL;
@@ -1780,10 +1780,10 @@ UINT WINAPI SetupDefaultQueueCallbackA( PVOID context, UINT notification,
         TRACE( "end queue\n" );
         return 0;
     case SPFILENOTIFY_STARTSUBQUEUE:
-        TRACE( "start subqueue %ld count %ld\n", param1, param2 );
+        TRACE( "start subqueue %Id count %Id\n", param1, param2 );
         return TRUE;
     case SPFILENOTIFY_ENDSUBQUEUE:
-        TRACE( "end subqueue %ld\n", param1 );
+        TRACE( "end subqueue %Id\n", param1 );
         return 0;
     case SPFILENOTIFY_STARTDELETE:
         TRACE( "start delete %s\n", debugstr_a(paths->Target) );
@@ -1823,7 +1823,7 @@ UINT WINAPI SetupDefaultQueueCallbackA( PVOID context, UINT notification,
         return FILEOP_DOIT;
     }
     default:
-        FIXME( "notification %d params %lx,%lx\n", notification, param1, param2 );
+        FIXME( "notification %d params %Ix,%Ix\n", notification, param1, param2 );
         break;
     }
     return 0;
@@ -1848,10 +1848,10 @@ UINT WINAPI SetupDefaultQueueCallbackW( PVOID context, UINT notification,
         TRACE( "end queue\n" );
         return 0;
     case SPFILENOTIFY_STARTSUBQUEUE:
-        TRACE( "start subqueue %ld count %ld\n", param1, param2 );
+        TRACE( "start subqueue %Id count %Id\n", param1, param2 );
         return TRUE;
     case SPFILENOTIFY_ENDSUBQUEUE:
-        TRACE( "end subqueue %ld\n", param1 );
+        TRACE( "end subqueue %Id\n", param1 );
         return 0;
     case SPFILENOTIFY_STARTDELETE:
         TRACE( "start delete %s\n", debugstr_w(paths->Target) );
@@ -1892,7 +1892,7 @@ UINT WINAPI SetupDefaultQueueCallbackW( PVOID context, UINT notification,
         return FILEOP_DOIT;
     }
     default:
-        FIXME( "notification %d params %lx,%lx\n", notification, param1, param2 );
+        FIXME( "notification %d params %Ix,%Ix\n", notification, param1, param2 );
         break;
     }
     return 0;
