@@ -18,21 +18,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include <stdarg.h>
-#include <stdio.h>
-#include <string.h>
-#include "windef.h"
-#include "winbase.h"
-#include "wingdi.h"
-#include "winuser.h"
+#include "user_private.h"
+#include "controls.h"
 #include "imm.h"
 #include "ddk/imm.h"
-
-#include "controls.h"
-#include "user_private.h"
-#include "win.h"
 #include "wine/debug.h"
-#include "wine/exception.h"
+
 
 WINE_DEFAULT_DEBUG_CHANNEL(graphics);
 WINE_DECLARE_DEBUG_CHANNEL(message);
@@ -41,16 +32,6 @@ HMODULE user32_module = 0;
 
 extern void WDML_NotifyThreadDetach(void);
 
-/***********************************************************************
- *           USER_CheckNotLock
- *
- * Make sure that we don't hold the user lock.
- */
-void USER_CheckNotLock(void)
-{
-    NtUserCallOneParam( 2, NtUserLock );
-}
-
 
 /***********************************************************************
  *             UserRealizePalette (USER32.@)
@@ -135,8 +116,17 @@ static NTSTATUS WINAPI User32DrawScrollBar( const struct draw_scroll_bar_params
 
 static NTSTATUS WINAPI User32DrawText( const struct draw_text_params *params, ULONG size )
 {
+    RECT rect = params->rect;
+    int ret;
+
     size -= FIELD_OFFSET( struct draw_text_params, str );
-    return DrawTextW( params->hdc, params->str, size / sizeof(WCHAR), params->rect, params->flags );
+    ret = DrawTextW( params->hdc, params->str, size / sizeof(WCHAR), &rect, params->flags );
+    if (params->ret_rect)
+    {
+        *params->ret_rect = rect;
+        return ret;
+    }
+    return NtCallbackReturn( &rect, sizeof(rect), ret );
 }
 
 static NTSTATUS WINAPI User32ImmProcessKey( const struct imm_process_key_params *params, ULONG size )
@@ -190,12 +180,16 @@ static BOOL WINAPI User32LoadDriver( const WCHAR *path, ULONG size )
 
 static NTSTATUS WINAPI User32UnpackDDEMessage( const struct unpack_dde_message_params *params, ULONG size )
 {
-    struct unpack_dde_message_result *result = params->result;
-    result->wparam = params->wparam;
-    result->lparam = params->lparam;
+    struct unpack_dde_message_result result = { .wparam = params->wparam, .lparam = params->lparam };
+
     size -= FIELD_OFFSET( struct unpack_dde_message_params, data );
-    return unpack_dde_message( params->hwnd, params->message, &result->wparam, &result->lparam,
-                               params->data, size );
+    if (!unpack_dde_message( params->hwnd, params->message, &result.wparam, &result.lparam,
+                             params->data, size ))
+        return FALSE;
+
+    if (params->result) *params->result = result;
+    else NtCallbackReturn( &result, sizeof(result), TRUE );
+    return TRUE;
 }
 
 static const void *kernel_callback_table[NtUserCallCount] =
@@ -211,11 +205,11 @@ static const void *kernel_callback_table[NtUserCallCount] =
     User32FreeCachedClipboardData,
     User32ImmProcessKey,
     User32ImmTranslateMessage,
+    User32InitBuiltinClasses,
     User32LoadDriver,
     User32LoadImage,
     User32LoadSysMenu,
     User32PostDDEMessage,
-    User32RegisterBuiltinClasses,
     User32RenderSsynthesizedFormat,
     User32UnpackDDEMessage,
 };
@@ -228,11 +222,8 @@ static BOOL process_attach(void)
 {
     NtCurrentTeb()->Peb->KernelCallbackTable = kernel_callback_table;
 
-    dpiaware_init();
     winproc_init();
-    register_desktop_class();
-
-    /* Initialize system colors and metrics */
+    dpiaware_init();
     SYSPARAMS_Init();
 
     return TRUE;
@@ -244,14 +235,14 @@ static BOOL process_attach(void)
  */
 static void thread_detach(void)
 {
-    struct user_thread_info *thread_info = get_user_thread_info();
+    struct ntuser_thread_info *thread_info = NtUserGetThreadInfo();
 
     NtUserCallNoParam( NtUserExitingThread );
 
     WDML_NotifyThreadDetach();
 
     NtUserCallNoParam( NtUserThreadDetach );
-    HeapFree( GetProcessHeap(), 0, thread_info->wmchar_data );
+    HeapFree( GetProcessHeap(), 0, (void *)(UINT_PTR)thread_info->wmchar_data );
 }
 
 
@@ -363,7 +354,9 @@ BOOL WINAPI ShutdownBlockReasonDestroy(HWND hwnd)
 
 const char *SPY_GetMsgName( UINT msg, HWND hwnd )
 {
-    return (const char *)NtUserCallHwndParam( hwnd, msg, NtUserSpyGetMsgName );
+    char buf[128];
+    NtUserMessageCall( hwnd, msg, ARRAYSIZE(buf), 0, buf, NtUserSpyGetMsgName, FALSE );
+    return wine_dbg_sprintf( "%s", buf );
 }
 
 void SPY_EnterMessage( INT flag, HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
