@@ -92,6 +92,7 @@ static void dump_instr_arg(instr_arg_type_t type, instr_arg_t *arg)
     case ARG_ADDR:
         TRACE_(vbscript_disas)("\t%u", arg->uint);
         break;
+    case ARG_DATE:
     case ARG_DOUBLE:
         TRACE_(vbscript_disas)("\t%lf", *arg->dbl);
         break;
@@ -238,6 +239,24 @@ static HRESULT push_instr_double(compile_ctx_t *ctx, vbsop_t op, double arg)
     return S_OK;
 }
 
+static HRESULT push_instr_date(compile_ctx_t *ctx, vbsop_t op, DATE arg)
+{
+    unsigned instr;
+    DATE *d;
+
+    d = compiler_alloc(ctx->code, sizeof(DATE));
+    if(!d)
+        return E_OUTOFMEMORY;
+
+    instr = push_instr(ctx, op);
+    if(!instr)
+        return E_OUTOFMEMORY;
+
+    *d = arg;
+    instr_ptr(ctx, instr)->arg1.date = d;
+    return S_OK;
+}
+
 static BSTR alloc_bstr_arg(compile_ctx_t *ctx, const WCHAR *str)
 {
     if(!ctx->code->bstr_pool_size) {
@@ -380,9 +399,9 @@ static HRESULT compile_error(script_ctx_t *ctx, compile_ctx_t *compiler, HRESULT
         return error;
 
     clear_ei(&ctx->ei);
-    ctx->ei.scode = error = map_hres(error);
+    ctx->ei.scode = error;
     ctx->ei.bstrSource = get_vbscript_string(VBS_COMPILE_ERROR);
-    ctx->ei.bstrDescription = get_vbscript_error_string(error);
+    map_vbs_exception(&ctx->ei);
     return report_script_error(ctx, compiler->code, compiler->loc);
 }
 
@@ -427,7 +446,8 @@ static HRESULT compile_args(compile_ctx_t *ctx, expression_t *args, unsigned *re
     return S_OK;
 }
 
-static HRESULT compile_member_expression(compile_ctx_t *ctx, member_expression_t *expr, unsigned arg_cnt, BOOL ret_val)
+static HRESULT compile_member_call_expression(compile_ctx_t *ctx, member_expression_t *expr,
+                                              unsigned arg_cnt, BOOL ret_val)
 {
     HRESULT hres;
 
@@ -452,6 +472,20 @@ static HRESULT compile_member_expression(compile_ctx_t *ctx, member_expression_t
     return hres;
 }
 
+static HRESULT compile_member_expression(compile_ctx_t *ctx, member_expression_t *expr)
+{
+    expression_t *const_expr;
+
+    if (expr->obj_expr) /* FIXME: we should probably have a dedicated opcode as well */
+        return compile_member_call_expression(ctx, expr, 0, TRUE);
+
+    const_expr = lookup_const_decls(ctx, expr->identifier, TRUE);
+    if(const_expr)
+        return compile_expression(ctx, const_expr);
+
+    return push_instr_bstr(ctx, OP_ident, expr->identifier);
+}
+
 static HRESULT compile_call_expression(compile_ctx_t *ctx, call_expression_t *expr, BOOL ret_val)
 {
     unsigned arg_cnt = 0;
@@ -465,7 +499,7 @@ static HRESULT compile_call_expression(compile_ctx_t *ctx, call_expression_t *ex
     for(call = expr->call_expr; call->type == EXPR_BRACKETS; call = ((unary_expression_t*)call)->subexpr);
 
     if(call->type == EXPR_MEMBER)
-        return compile_member_expression(ctx, (member_expression_t*)call, arg_cnt, ret_val);
+        return compile_member_call_expression(ctx, (member_expression_t*)call, arg_cnt, ret_val);
 
     hres = compile_expression(ctx, call);
     if(FAILED(hres))
@@ -530,6 +564,8 @@ static HRESULT compile_expression(compile_ctx_t *ctx, expression_t *expr)
         return compile_call_expression(ctx, (call_expression_t*)expr, TRUE);
     case EXPR_CONCAT:
         return compile_binary_expression(ctx, (binary_expression_t*)expr, OP_concat);
+    case EXPR_DATE:
+        return push_instr_date(ctx, OP_date, ((date_expression_t*)expr)->value);
     case EXPR_DIV:
         return compile_binary_expression(ctx, (binary_expression_t*)expr, OP_div);
     case EXPR_DOT:
@@ -561,7 +597,7 @@ static HRESULT compile_expression(compile_ctx_t *ctx, expression_t *expr)
     case EXPR_ME:
         return push_instr(ctx, OP_me) ? S_OK : E_OUTOFMEMORY;
     case EXPR_MEMBER:
-        return compile_member_expression(ctx, (member_expression_t*)expr, 0, TRUE);
+        return compile_member_expression(ctx, (member_expression_t*)expr);
     case EXPR_MOD:
         return compile_binary_expression(ctx, (binary_expression_t*)expr, OP_mod);
     case EXPR_MUL:
@@ -793,6 +829,10 @@ static HRESULT compile_foreach_statement(compile_ctx_t *ctx, foreach_statement_t
         return hres;
 
     label_set_addr(ctx, loop_ctx.for_end_label);
+
+    if(!emit_catch(ctx, 0))
+        return E_OUTOFMEMORY;
+
     return S_OK;
 }
 
@@ -1962,6 +2002,7 @@ HRESULT compile_script(script_ctx_t *script, const WCHAR *src, const WCHAR *item
         item->ref++;
     }
 
+    ctx.parser.lcid = script->lcid;
     hres = parse_script(&ctx.parser, code->source, delimiter, flags);
     if(FAILED(hres)) {
         if(ctx.parser.error_loc != -1)
