@@ -848,9 +848,7 @@ static HRESULT Global_Hex(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
         n = (WORD)V_I2(arg);
         break;
     case VT_NULL:
-        if(res)
-            V_VT(res) = VT_NULL;
-        return S_OK;
+        return return_null(res);
     default:
         hres = to_int(arg, &ret);
         if(FAILED(hres))
@@ -889,9 +887,7 @@ static HRESULT Global_Oct(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
         n = (WORD)V_I2(arg);
         break;
     case VT_NULL:
-        if(res)
-            V_VT(res) = VT_NULL;
-        return S_OK;
+        return return_null(res);
     default:
         hres = to_int(arg, &ret);
         if(FAILED(hres))
@@ -1430,9 +1426,7 @@ static HRESULT Global_LCase(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
     TRACE("%s\n", debugstr_variant(arg));
 
     if(V_VT(arg) == VT_NULL) {
-        if(res)
-            V_VT(res) = VT_NULL;
-        return S_OK;
+        return return_null(res);
     }
 
     hres = to_string(arg, &str);
@@ -1461,9 +1455,7 @@ static HRESULT Global_UCase(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
     TRACE("%s\n", debugstr_variant(arg));
 
     if(V_VT(arg) == VT_NULL) {
-        if(res)
-            V_VT(res) = VT_NULL;
-        return S_OK;
+        return return_null(res);
     }
 
     hres = to_string(arg, &str);
@@ -3296,7 +3288,7 @@ static HRESULT Err_Raise(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VA
 {
     BSTR source = NULL, description = NULL, helpfile = NULL;
     int code,  helpcontext = 0;
-    HRESULT hres, error;
+    HRESULT hres;
 
     TRACE("%s %u...\n", debugstr_variant(args), args_cnt);
 
@@ -3318,20 +3310,14 @@ static HRESULT Err_Raise(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VA
     if(SUCCEEDED(hres)) {
         script_ctx_t *ctx = This->ctx;
 
-        error = (code & ~0xffff) ? map_hres(code) : MAKE_VBSERROR(code);
-
         if(source) {
             SysFreeString(ctx->ei.bstrSource);
             ctx->ei.bstrSource = source;
         }
-        if(!ctx->ei.bstrSource)
-            ctx->ei.bstrSource = get_vbscript_string(VBS_RUNTIME_ERROR);
         if(description) {
             SysFreeString(ctx->ei.bstrDescription);
             ctx->ei.bstrDescription = description;
         }
-        if(!ctx->ei.bstrDescription)
-            ctx->ei.bstrDescription = get_vbscript_error_string(error);
         if(helpfile) {
             SysFreeString(ctx->ei.bstrHelpFile);
             ctx->ei.bstrHelpFile = helpfile;
@@ -3339,7 +3325,9 @@ static HRESULT Err_Raise(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VA
         if(args_cnt >= 5)
             ctx->ei.dwHelpContext = helpcontext;
 
-        ctx->ei.scode = error;
+        ctx->ei.scode = (code & ~0xffff) ? code : MAKE_VBSERROR(code);
+        map_vbs_exception(&ctx->ei);
+
         hres = SCRIPT_E_RECORDED;
     }else {
         SysFreeString(source);
