@@ -566,8 +566,8 @@ static void init_gdi_shared(void)
 {
     SIZE_T size = sizeof(*gdi_shared);
 
-    if (NtAllocateVirtualMemory( GetCurrentProcess(), (void **)&gdi_shared, 0, &size,
-                                 MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE ))
+    if (NtAllocateVirtualMemory( GetCurrentProcess(), (void **)&gdi_shared, zero_bits(),
+                                 &size, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE ))
         return;
     next_unused = gdi_shared->Handles + FIRST_GDI_HANDLE;
 
@@ -943,7 +943,7 @@ INT WINAPI NtGdiExtGetObjectW( HGDIOBJ handle, INT count, void *buffer )
     if (funcs && funcs->pGetObjectW)
     {
         if (buffer && ((ULONG_PTR)buffer >> 16) == 0) /* catch apps getting argument order wrong */
-            SetLastError( ERROR_NOACCESS );
+            RtlSetLastWin32Error( ERROR_NOACCESS );
         else
             result = funcs->pGetObjectW( handle, count, buffer );
     }
@@ -1153,7 +1153,7 @@ static struct unix_funcs unix_funcs =
     __wine_send_input,
 };
 
-NTSTATUS gdi_init(void)
+void gdi_init(void)
 {
     pthread_mutexattr_t attr;
     unsigned int dpi;
@@ -1165,11 +1165,10 @@ NTSTATUS gdi_init(void)
 
     NtQuerySystemInformation( SystemBasicInformation, &system_info, sizeof(system_info), NULL );
     init_gdi_shared();
-    if (!gdi_shared) return STATUS_NO_MEMORY;
+    if (!gdi_shared) return;
 
     dpi = font_init();
     init_stock_objects( dpi );
-    return 0;
 }
 
 NTSTATUS callbacks_init( void *args )
