@@ -78,7 +78,7 @@ HHOOK WINAPI NtUserSetWindowsHookEx( HINSTANCE inst, UNICODE_STRING *module, DWO
 
     if (!proc)
     {
-        SetLastError( ERROR_INVALID_FILTER_PROC );
+        RtlSetLastWin32Error( ERROR_INVALID_FILTER_PROC );
         return 0;
     }
 
@@ -91,7 +91,7 @@ HHOOK WINAPI NtUserSetWindowsHookEx( HINSTANCE inst, UNICODE_STRING *module, DWO
             id == WH_SYSMSGFILTER)
         {
             /* these can only be global */
-            SetLastError( ERROR_INVALID_PARAMETER );
+            RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
             return 0;
         }
     }
@@ -100,7 +100,7 @@ HHOOK WINAPI NtUserSetWindowsHookEx( HINSTANCE inst, UNICODE_STRING *module, DWO
         if (id == WH_KEYBOARD_LL || id == WH_MOUSE_LL) inst = 0;
         else if (!inst)
         {
-            SetLastError( ERROR_HOOK_NEEDS_HMOD );
+            RtlSetLastWin32Error( ERROR_HOOK_NEEDS_HMOD );
             return 0;
         }
     }
@@ -148,7 +148,7 @@ BOOL WINAPI NtUserUnhookWindowsHookEx( HHOOK handle )
         if (!status) get_user_thread_info()->active_hooks = reply->active_hooks;
     }
     SERVER_END_REQ;
-    if (status == STATUS_INVALID_HANDLE) SetLastError( ERROR_INVALID_HOOK_HANDLE );
+    if (status == STATUS_INVALID_HANDLE) RtlSetLastWin32Error( ERROR_INVALID_HOOK_HANDLE );
     return !status;
 }
 
@@ -168,7 +168,7 @@ BOOL unhook_windows_hook( INT id, HOOKPROC proc )
         if (!status) get_user_thread_info()->active_hooks = reply->active_hooks;
     }
     SERVER_END_REQ;
-    if (status == STATUS_INVALID_HANDLE) SetLastError( ERROR_INVALID_HOOK_HANDLE );
+    if (status == STATUS_INVALID_HANDLE) RtlSetLastWin32Error( ERROR_INVALID_HOOK_HANDLE );
     return !status;
 }
 
@@ -179,8 +179,8 @@ BOOL WINAPI NtUserCallMsgFilter( MSG *msg, INT code )
 {
     /* FIXME: We should use NtCallbackReturn instead of passing (potentially kernel) pointer
      * like that, but we need to consequently use syscall thunks first for that to work. */
-    if (call_hooks( WH_SYSMSGFILTER, code, 0, (LPARAM)msg, TRUE )) return TRUE;
-    return call_hooks( WH_MSGFILTER, code, 0, (LPARAM)msg, TRUE );
+    if (call_hooks( WH_SYSMSGFILTER, code, 0, (LPARAM)msg, sizeof(*msg) )) return TRUE;
+    return call_hooks( WH_MSGFILTER, code, 0, (LPARAM)msg, sizeof(*msg) );
 }
 
 static UINT get_ll_hook_timeout(void)
@@ -195,7 +195,7 @@ static UINT get_ll_hook_timeout(void)
  * Call hook either in current thread or send message to the destination
  * thread.
  */
-static LRESULT call_hook( struct win_hook_params *info )
+static LRESULT call_hook( struct win_hook_params *info, const WCHAR *module )
 {
     DWORD_PTR ret = 0;
 
@@ -231,10 +231,70 @@ static LRESULT call_hook( struct win_hook_params *info )
         struct user_thread_info *thread_info = get_user_thread_info();
         HHOOK prev = thread_info->hook;
         BOOL prev_unicode = thread_info->hook_unicode;
-        size_t len = lstrlenW( info->module );
+        struct win_hook_params *params = info;
+        ULONG size = sizeof(*params);
+        ULONG lparam_ret_size = params->lparam_size;
+        CREATESTRUCTW *cs = NULL;
         void *ret_ptr;
         ULONG ret_len;
 
+        if (params->lparam_size)
+        {
+            lparam_ret_size = params->lparam_size;
+
+            if (params->id == WH_CBT && params->code == HCBT_CREATEWND)
+            {
+                cs = ((CBT_CREATEWNDW *)params->lparam)->lpcs;
+                params->lparam = 0;
+                lparam_ret_size = 0;
+                params->lparam_size = sizeof(*cs);
+                if (!IS_INTRESOURCE( cs->lpszName ))
+                    params->lparam_size += (wcslen( cs->lpszName ) + 1) * sizeof(WCHAR);
+                if (!IS_INTRESOURCE( cs->lpszClass ))
+                    params->lparam_size += (wcslen( cs->lpszClass ) + 1) * sizeof(WCHAR);
+            }
+
+            size += params->lparam_size;
+        }
+
+        if (module && module[0]) size += (lstrlenW( module ) + 1) * sizeof(WCHAR);
+        if (size != sizeof(*params))
+        {
+            if (!(params = malloc( size ))) return 0;
+            *params = *info;
+        }
+
+        if (params->lparam_size)
+        {
+            if (cs)
+            {
+                CREATESTRUCTW *params_cs = (CREATESTRUCTW *)(params + 1);
+                WCHAR *ptr = (WCHAR *)(params_cs + 1);
+                const void *inline_ptr = (void *)0xffffffff;
+
+                *params_cs = *cs;
+                if (!IS_INTRESOURCE( cs->lpszName ))
+                {
+                    UINT len = wcslen( cs->lpszName ) + 1;
+                    memcpy( ptr, cs->lpszName, len * sizeof(WCHAR) );
+                    ptr += len;
+                    params_cs->lpszName = inline_ptr;
+                }
+                if (!IS_INTRESOURCE( cs->lpszClass ))
+                {
+                    wcscpy( ptr, cs->lpszClass );
+                    params_cs->lpszClass = inline_ptr;
+                }
+            }
+            else
+            {
+                memcpy( params + 1, (const void *)params->lparam, params->lparam_size );
+            }
+        }
+
+        if (module && module[0])
+            wcscpy( (WCHAR *)((char *)(params + 1) + params->lparam_size), module );
+
         /*
          * Windows protects from stack overflow in recursive hook calls. Different Windows
          * allow different depths.
@@ -242,22 +302,25 @@ static LRESULT call_hook( struct win_hook_params *info )
         if (thread_info->hook_call_depth >= 25)
         {
             WARN("Too many hooks called recursively, skipping call.\n");
+            if (params != info) free( params );
             return 0;
         }
 
         TRACE( "calling hook %p %s code %x wp %lx lp %lx module %s\n",
-               info->proc, hook_names[info->id-WH_MINHOOK], info->code, info->wparam,
-               info->lparam, debugstr_w(info->module) );
+               params->proc, hook_names[params->id-WH_MINHOOK], params->code, params->wparam,
+               params->lparam, debugstr_w(module) );
 
-        thread_info->hook = info->handle;
-        thread_info->hook_unicode = info->next_unicode;
+        thread_info->hook = params->handle;
+        thread_info->hook_unicode = params->next_unicode;
         thread_info->hook_call_depth++;
-        ret = KeUserModeCallback( NtUserCallWindowsHook, info,
-                                  FIELD_OFFSET( struct win_hook_params, module[len + 1] ),
-                                  &ret_ptr, &ret_len );
+        ret = KeUserModeCallback( NtUserCallWindowsHook, params, size, &ret_ptr, &ret_len );
+        if (ret_len && ret_len == lparam_ret_size)
+            memcpy( (void *)params->lparam, ret_ptr, lparam_ret_size );
         thread_info->hook = prev;
         thread_info->hook_unicode = prev_unicode;
         thread_info->hook_call_depth--;
+
+        if (params != info) free( params );
     }
 
     if (info->id == WH_KEYBOARD_LL || info->id == WH_MOUSE_LL)
@@ -272,18 +335,19 @@ LRESULT WINAPI NtUserCallNextHookEx( HHOOK hhook, INT code, WPARAM wparam, LPARA
 {
     struct user_thread_info *thread_info = get_user_thread_info();
     struct win_hook_params info;
+    WCHAR module[MAX_PATH];
 
-    memset( &info, 0, sizeof(info) - sizeof(info.module) );
+    memset( &info, 0, sizeof(info) );
 
     SERVER_START_REQ( get_hook_info )
     {
         req->handle = wine_server_user_handle( thread_info->hook );
         req->get_next = 1;
         req->event = EVENT_MIN;
-        wine_server_set_reply( req, info.module, sizeof(info.module)-sizeof(WCHAR) );
+        wine_server_set_reply( req, module, sizeof(module) - sizeof(WCHAR) );
         if (!wine_server_call_err( req ))
         {
-            info.module[wine_server_reply_size(req) / sizeof(WCHAR)] = 0;
+            module[wine_server_reply_size(req) / sizeof(WCHAR)] = 0;
             info.handle       = wine_server_ptr_handle( reply->handle );
             info.id           = reply->id;
             info.pid          = reply->pid;
@@ -298,24 +362,25 @@ LRESULT WINAPI NtUserCallNextHookEx( HHOOK hhook, INT code, WPARAM wparam, LPARA
     info.wparam = wparam;
     info.lparam = lparam;
     info.prev_unicode = thread_info->hook_unicode;
-    return call_hook( &info );
+    return call_hook( &info, module );
 }
 
 LRESULT call_current_hook( HHOOK hhook, INT code, WPARAM wparam, LPARAM lparam )
 {
     struct win_hook_params info;
+    WCHAR module[MAX_PATH];
 
-    memset( &info, 0, sizeof(info) - sizeof(info.module) );
+    memset( &info, 0, sizeof(info) );
 
     SERVER_START_REQ( get_hook_info )
     {
         req->handle = wine_server_user_handle( hhook );
         req->get_next = 0;
         req->event = EVENT_MIN;
-        wine_server_set_reply( req, info.module, sizeof(info.module)-sizeof(WCHAR) );
+        wine_server_set_reply( req, module, sizeof(module) );
         if (!wine_server_call_err( req ))
         {
-            info.module[wine_server_reply_size(req) / sizeof(WCHAR)] = 0;
+            module[wine_server_reply_size(req) / sizeof(WCHAR)] = 0;
             info.handle       = wine_server_ptr_handle( reply->handle );
             info.id           = reply->id;
             info.pid          = reply->pid;
@@ -330,13 +395,14 @@ LRESULT call_current_hook( HHOOK hhook, INT code, WPARAM wparam, LPARAM lparam )
     info.wparam = wparam;
     info.lparam = lparam;
     info.prev_unicode = TRUE;  /* assume Unicode for this function */
-    return call_hook( &info );
+    return call_hook( &info, module );
 }
 
-LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, BOOL unicode )
+LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, size_t lparam_size )
 {
     struct user_thread_info *thread_info = get_user_thread_info();
     struct win_hook_params info;
+    WCHAR module[MAX_PATH];
     DWORD_PTR ret;
 
     user_check_not_lock();
@@ -347,18 +413,18 @@ LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, BOOL unicode
         return 0;
     }
 
-    memset( &info, 0, sizeof(info) - sizeof(info.module) );
-    info.prev_unicode = unicode;
+    memset( &info, 0, sizeof(info) );
+    info.prev_unicode = TRUE;
     info.id = id;
 
     SERVER_START_REQ( start_hook_chain )
     {
         req->id = info.id;
         req->event = EVENT_MIN;
-        wine_server_set_reply( req, info.module, sizeof(info.module)-sizeof(WCHAR) );
+        wine_server_set_reply( req, module, sizeof(module)-sizeof(WCHAR) );
         if (!wine_server_call( req ))
         {
-            info.module[wine_server_reply_size(req) / sizeof(WCHAR)] = 0;
+            module[wine_server_reply_size(req) / sizeof(WCHAR)] = 0;
             info.handle       = wine_server_ptr_handle( reply->handle );
             info.pid          = reply->pid;
             info.tid          = reply->tid;
@@ -373,7 +439,8 @@ LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, BOOL unicode
     info.code   = code;
     info.wparam = wparam;
     info.lparam = lparam;
-    ret = call_hook( &info );
+    info.lparam_size = lparam_size;
+    ret = call_hook( &info, module );
 
     SERVER_START_REQ( finish_hook_chain )
     {
@@ -395,13 +462,13 @@ HWINEVENTHOOK WINAPI NtUserSetWinEventHook( DWORD event_min, DWORD event_max, HM
 
     if ((flags & WINEVENT_INCONTEXT) && !inst)
     {
-        SetLastError(ERROR_HOOK_NEEDS_HMOD);
+        RtlSetLastWin32Error(ERROR_HOOK_NEEDS_HMOD);
         return 0;
     }
 
     if (event_min > event_max)
     {
-        SetLastError(ERROR_INVALID_HOOK_FILTER);
+        RtlSetLastWin32Error(ERROR_INVALID_HOOK_FILTER);
         return 0;
     }
 
@@ -471,7 +538,7 @@ void WINAPI NtUserNotifyWinEvent( DWORD event, HWND hwnd, LONG object_id, LONG c
 
     if (!hwnd)
     {
-        SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+        RtlSetLastWin32Error( ERROR_INVALID_WINDOW_HANDLE );
         return;
     }
 
@@ -514,7 +581,7 @@ void WINAPI NtUserNotifyWinEvent( DWORD event, HWND hwnd, LONG object_id, LONG c
 
         info.time = NtGetTickCount();
         KeUserModeCallback( NtUserCallWinEventHook, &info,
-                            FIELD_OFFSET( struct win_hook_params, module[lstrlenW(info.module) + 1] ),
+                            FIELD_OFFSET( struct win_event_hook_params, module[lstrlenW(info.module) + 1] ),
                             &ret_ptr, &ret_len );
 
         SERVER_START_REQ( get_hook_info )
