@@ -134,19 +134,19 @@ UINT WINAPI NtUserSendInput( UINT count, INPUT *inputs, int size )
 
     if (size != sizeof(INPUT))
     {
-        SetLastError( ERROR_INVALID_PARAMETER );
+        RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
         return 0;
     }
 
     if (!count)
     {
-        SetLastError( ERROR_INVALID_PARAMETER );
+        RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
         return 0;
     }
 
     if (!inputs)
     {
-        SetLastError( ERROR_NOACCESS );
+        RtlSetLastWin32Error( ERROR_NOACCESS );
         return 0;
     }
 
@@ -163,13 +163,13 @@ UINT WINAPI NtUserSendInput( UINT count, INPUT *inputs, int size )
             status = send_hardware_message( 0, &input, NULL, SEND_HWMSG_INJECTED );
             break;
         case INPUT_HARDWARE:
-            SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+            RtlSetLastWin32Error( ERROR_CALL_NOT_IMPLEMENTED );
             return 0;
         }
 
         if (status)
         {
-            SetLastError( RtlNtStatusToDosError(status) );
+            RtlSetLastWin32Error( RtlNtStatusToDosError(status) );
             break;
         }
     }
@@ -340,7 +340,7 @@ DWORD WINAPI NtUserGetQueueStatus( UINT flags )
 
     if (flags & ~(QS_ALLINPUT | QS_ALLPOSTMESSAGE | QS_SMRESULT))
     {
-        SetLastError( ERROR_INVALID_FLAGS );
+        RtlSetLastWin32Error( ERROR_INVALID_FLAGS );
         return 0;
     }
 
@@ -908,7 +908,7 @@ HKL WINAPI NtUserActivateKeyboardLayout( HKL layout, UINT flags )
 
     if (layout == (HKL)HKL_NEXT || layout == (HKL)HKL_PREV)
     {
-        SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+        RtlSetLastWin32Error( ERROR_CALL_NOT_IMPLEMENTED );
         FIXME_(keyboard)( "HKL_NEXT and HKL_PREV not supported\n" );
         return 0;
     }
@@ -1002,7 +1002,7 @@ BOOL WINAPI NtUserGetKeyboardLayoutName( WCHAR *name )
 
     if (!name)
     {
-        SetLastError( ERROR_NOACCESS );
+        RtlSetLastWin32Error( ERROR_NOACCESS );
         return FALSE;
     }
 
@@ -1127,20 +1127,20 @@ int WINAPI NtUserGetMouseMovePointsEx( UINT size, MOUSEMOVEPOINT *ptin, MOUSEMOV
 
     if ((size != sizeof(MOUSEMOVEPOINT)) || (count < 0) || (count > ARRAY_SIZE( positions )))
     {
-        SetLastError( ERROR_INVALID_PARAMETER );
+        RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
         return -1;
     }
 
     if (!ptin || (!ptout && count))
     {
-        SetLastError( ERROR_NOACCESS );
+        RtlSetLastWin32Error( ERROR_NOACCESS );
         return -1;
     }
 
     if (resolution != GMMP_USE_DISPLAY_POINTS)
     {
         FIXME( "only GMMP_USE_DISPLAY_POINTS is supported for now\n" );
-        SetLastError( ERROR_POINT_NOT_FOUND );
+        RtlSetLastWin32Error( ERROR_POINT_NOT_FOUND );
         return -1;
     }
 
@@ -1160,7 +1160,7 @@ int WINAPI NtUserGetMouseMovePointsEx( UINT size, MOUSEMOVEPOINT *ptin, MOUSEMOV
 
     if (i == ARRAY_SIZE( positions ))
     {
-        SetLastError( ERROR_POINT_NOT_FOUND );
+        RtlSetLastWin32Error( ERROR_POINT_NOT_FOUND );
         return -1;
     }
 
@@ -1320,7 +1320,7 @@ BOOL WINAPI NtUserTrackMouseEvent( TRACKMOUSEEVENT *info )
     if (info->cbSize != sizeof(TRACKMOUSEEVENT))
     {
         WARN( "wrong size %u\n", info->cbSize );
-        SetLastError( ERROR_INVALID_PARAMETER );
+        RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
         return FALSE;
     }
 
@@ -1333,7 +1333,7 @@ BOOL WINAPI NtUserTrackMouseEvent( TRACKMOUSEEVENT *info )
 
     if (!is_window( info->hwndTrack ))
     {
-        SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+        RtlSetLastWin32Error( ERROR_INVALID_WINDOW_HANDLE );
         return FALSE;
     }
 
@@ -1394,51 +1394,6 @@ BOOL WINAPI NtUserTrackMouseEvent( TRACKMOUSEEVENT *info )
     return TRUE;
 }
 
-/*******************************************************************
- *           NtUserDragDetect (win32u.@)
- */
-BOOL WINAPI NtUserDragDetect( HWND hwnd, int x, int y )
-{
-    WORD width, height;
-    RECT rect;
-    MSG msg;
-
-    TRACE( "%p (%d,%d)\n", hwnd, x, y );
-
-    if (!(NtUserGetKeyState( VK_LBUTTON ) & 0x8000)) return FALSE;
-
-    width  = get_system_metrics( SM_CXDRAG );
-    height = get_system_metrics( SM_CYDRAG );
-    SetRect( &rect, x - width, y - height, x + width, y + height );
-
-    NtUserSetCapture( hwnd );
-
-    for (;;)
-    {
-        while (NtUserPeekMessage( &msg, 0, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE ))
-        {
-            if (msg.message == WM_LBUTTONUP)
-            {
-                release_capture();
-                return FALSE;
-            }
-            if (msg.message == WM_MOUSEMOVE)
-            {
-                POINT tmp;
-                tmp.x = (short)LOWORD( msg.lParam );
-                tmp.y = (short)HIWORD( msg.lParam );
-                if (!PtInRect( &rect, tmp ))
-                {
-                    release_capture();
-                    return TRUE;
-                }
-            }
-        }
-        NtUserMsgWaitForMultipleObjectsEx( 0, NULL, INFINITE, QS_ALLINPUT, 0 );
-    }
-    return FALSE;
-}
-
 /**********************************************************************
  *		set_capture_window
  */
@@ -1551,7 +1506,7 @@ HWND get_focus(void)
  */
 static HWND set_focus_window( HWND hwnd )
 {
-    HWND previous = 0;
+    HWND previous = 0, ime_hwnd;
     BOOL ret;
 
     SERVER_START_REQ( set_focus_window )
@@ -1568,7 +1523,10 @@ static HWND set_focus_window( HWND hwnd )
     {
         send_message( previous, WM_KILLFOCUS, (WPARAM)hwnd, 0 );
 
-        if (user_callbacks) user_callbacks->notify_ime( previous, IME_INTERNAL_DEACTIVATE );
+        ime_hwnd = get_default_ime_window( previous );
+        if (ime_hwnd)
+            send_message( ime_hwnd, WM_IME_INTERNAL, IME_INTERNAL_DEACTIVATE,
+                          HandleToUlong(previous) );
 
         if (hwnd != get_focus()) return previous;  /* changed by the message */
     }
@@ -1576,7 +1534,10 @@ static HWND set_focus_window( HWND hwnd )
     {
         user_driver->pSetFocus(hwnd);
 
-        if (user_callbacks) user_callbacks->notify_ime( hwnd, IME_INTERNAL_ACTIVATE );
+        ime_hwnd = get_default_ime_window( hwnd );
+        if (ime_hwnd)
+            send_message( ime_hwnd, WM_IME_INTERNAL, IME_INTERNAL_ACTIVATE,
+                          HandleToUlong(hwnd) );
 
         if (previous)
             NtUserNotifyWinEvent( EVENT_OBJECT_FOCUS, hwnd, OBJID_CLIENT, 0 );
@@ -1605,7 +1566,7 @@ static BOOL set_active_window( HWND hwnd, HWND *prev, BOOL mouse, BOOL focus )
     /* call CBT hook chain */
     cbt.fMouse     = mouse;
     cbt.hWndActive = previous;
-    if (call_hooks( WH_CBT, HCBT_ACTIVATE, (WPARAM)hwnd, (LPARAM)&cbt, TRUE )) return FALSE;
+    if (call_hooks( WH_CBT, HCBT_ACTIVATE, (WPARAM)hwnd, (LPARAM)&cbt, sizeof(cbt) )) return FALSE;
 
     if (is_window( previous ))
     {
@@ -1630,7 +1591,7 @@ static BOOL set_active_window( HWND hwnd, HWND *prev, BOOL mouse, BOOL focus )
         /* send palette messages */
         if (send_message( hwnd, WM_QUERYNEWPALETTE, 0, 0 ))
             send_message_timeout( HWND_BROADCAST, WM_PALETTEISCHANGING, (WPARAM)hwnd, 0,
-                                  SMTO_ABORTIFHUNG, 2000, NULL, FALSE );
+                                  SMTO_ABORTIFHUNG, 2000, FALSE );
         if (!is_window(hwnd)) return FALSE;
     }
 
@@ -1707,7 +1668,7 @@ HWND WINAPI NtUserSetActiveWindow( HWND hwnd )
         hwnd = get_full_window_handle( hwnd );
         if (!is_window( hwnd ))
         {
-            SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+            RtlSetLastWin32Error( ERROR_INVALID_WINDOW_HANDLE );
             return 0;
         }
 
@@ -1736,7 +1697,7 @@ HWND WINAPI NtUserSetFocus( HWND hwnd )
         hwnd = get_full_window_handle( hwnd );
         if (!is_window( hwnd ))
         {
-            SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+            RtlSetLastWin32Error( ERROR_INVALID_WINDOW_HANDLE );
             return 0;
         }
         if (hwnd == previous) return previous;  /* nothing to do */
@@ -1757,7 +1718,7 @@ HWND WINAPI NtUserSetFocus( HWND hwnd )
         }
 
         /* call hooks */
-        if (call_hooks( WH_CBT, HCBT_SETFOCUS, (WPARAM)hwnd, (LPARAM)previous, TRUE )) return 0;
+        if (call_hooks( WH_CBT, HCBT_SETFOCUS, (WPARAM)hwnd, (LPARAM)previous, 0 )) return 0;
 
         /* activate hwndTop if needed. */
         if (hwndTop != get_active_window())
@@ -1772,7 +1733,7 @@ HWND WINAPI NtUserSetFocus( HWND hwnd )
     else /* NULL hwnd passed in */
     {
         if (!previous) return 0;  /* nothing to do */
-        if (call_hooks( WH_CBT, HCBT_SETFOCUS, 0, (LPARAM)previous, TRUE )) return 0;
+        if (call_hooks( WH_CBT, HCBT_SETFOCUS, 0, (LPARAM)previous, 0 )) return 0;
     }
 
     /* change focus and send messages */
