@@ -116,7 +116,7 @@ static struct gdi_path *alloc_gdi_path( int count )
 
     if (!path)
     {
-        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+        RtlSetLastWin32Error( ERROR_NOT_ENOUGH_MEMORY );
         return NULL;
     }
     count = max( NUM_ENTRIES_INITIAL, count );
@@ -126,7 +126,7 @@ static struct gdi_path *alloc_gdi_path( int count )
         if (!path->points)
         {
             free( path );
-            SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+            RtlSetLastWin32Error( ERROR_NOT_ENOUGH_MEMORY );
             return NULL;
         }
         path->flags = (BYTE *)(path->points + count);
@@ -542,7 +542,7 @@ struct gdi_path *get_gdi_flat_path( DC *dc, HRGN *rgn )
         dc->path = NULL;
         if (ret && rgn) *rgn = path_to_region( ret, dc->attr->poly_fill_mode );
     }
-    else SetLastError( ERROR_CAN_NOT_COMPLETE );
+    else RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
 
     return ret;
 }
@@ -638,7 +638,7 @@ INT WINAPI NtGdiGetPath( HDC hdc, POINT *points, BYTE *types, INT size )
 
     if (!dc->path)
     {
-        SetLastError( ERROR_CAN_NOT_COMPLETE );
+        RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
     }
     else if (size == 0)
     {
@@ -646,7 +646,7 @@ INT WINAPI NtGdiGetPath( HDC hdc, POINT *points, BYTE *types, INT size )
     }
     else if (size < dc->path->count)
     {
-        SetLastError( ERROR_INVALID_PARAMETER );
+        RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
     }
     else
     {
@@ -658,7 +658,7 @@ INT WINAPI NtGdiGetPath( HDC hdc, POINT *points, BYTE *types, INT size )
             ret = dc->path->count;
         else
             /* FIXME: Is this the correct value? */
-            SetLastError( ERROR_CAN_NOT_COMPLETE );
+            RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
     }
 
     release_dc_ptr( dc );
@@ -688,7 +688,7 @@ HRGN WINAPI NtGdiPathToRegion( HDC hdc )
             free_gdi_path( path );
         }
     }
-    else SetLastError( ERROR_CAN_NOT_COMPLETE );
+    else RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
 
     release_dc_ptr( dc );
     return ret;
@@ -1598,7 +1598,7 @@ BOOL WINAPI NtGdiFlattenPath( HDC hdc )
 
     if (!(dc = get_dc_ptr( hdc ))) return FALSE;
 
-    if (!dc->path) SetLastError( ERROR_CAN_NOT_COMPLETE );
+    if (!dc->path) RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
     else if ((path = PATH_FlattenPath( dc->path )))
     {
         free_gdi_path( dc->path );
@@ -1616,14 +1616,14 @@ BOOL WINAPI NtGdiFlattenPath( HDC hdc )
 static struct gdi_path *PATH_WidenPath(DC *dc)
 {
     INT i, j, numStrokes, penWidth, penWidthIn, penWidthOut, size, penStyle;
-    struct gdi_path *flat_path, *pNewPath, **pStrokes = NULL, *pUpPath, *pDownPath;
+    struct gdi_path *flat_path, *pNewPath, **pStrokes = NULL, **new_strokes, *pUpPath, *pDownPath;
     EXTLOGPEN *elp;
     BYTE *type;
     DWORD obj_type, joint, endcap, penType;
 
     size = NtGdiExtGetObjectW( dc->hPen, 0, NULL );
     if (!size) {
-        SetLastError(ERROR_CAN_NOT_COMPLETE);
+        RtlSetLastWin32Error(ERROR_CAN_NOT_COMPLETE);
         return NULL;
     }
 
@@ -1640,7 +1640,7 @@ static struct gdi_path *PATH_WidenPath(DC *dc)
         penStyle = elp->elpPenStyle;
         break;
     default:
-        SetLastError(ERROR_CAN_NOT_COMPLETE);
+        RtlSetLastWin32Error(ERROR_CAN_NOT_COMPLETE);
         free( elp );
         return NULL;
     }
@@ -1654,7 +1654,7 @@ static struct gdi_path *PATH_WidenPath(DC *dc)
 
     /* The function cannot apply to cosmetic pens */
     if(obj_type == OBJ_EXTPEN && penType == PS_COSMETIC) {
-        SetLastError(ERROR_CAN_NOT_COMPLETE);
+        RtlSetLastWin32Error(ERROR_CAN_NOT_COMPLETE);
         return NULL;
     }
 
@@ -1681,12 +1681,14 @@ static struct gdi_path *PATH_WidenPath(DC *dc)
             case PT_MOVETO:
                 numStrokes++;
                 j = 0;
-                pStrokes = realloc( pStrokes, numStrokes * sizeof(*pStrokes) );
-                if(!pStrokes)
+                new_strokes = realloc( pStrokes, numStrokes * sizeof(*pStrokes) );
+                if (!new_strokes)
                 {
                     free_gdi_path(flat_path);
+                    free(pStrokes);
                     return NULL;
                 }
+                pStrokes = new_strokes;
                 pStrokes[numStrokes - 1] = alloc_gdi_path(0);
                 /* fall through */
             case PT_LINETO:
@@ -1946,7 +1948,7 @@ BOOL WINAPI NtGdiWidenPath( HDC hdc )
 
     if (!(dc = get_dc_ptr( hdc ))) return FALSE;
 
-    if (!dc->path) SetLastError( ERROR_CAN_NOT_COMPLETE );
+    if (!dc->path) RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
     else if ((path = PATH_WidenPath( dc )))
     {
         free_gdi_path( dc->path );
@@ -1986,7 +1988,7 @@ BOOL CDECL nulldrv_BeginPath( PHYSDEV dev )
 
 BOOL CDECL nulldrv_EndPath( PHYSDEV dev )
 {
-    SetLastError( ERROR_CAN_NOT_COMPLETE );
+    RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
     return FALSE;
 }
 
@@ -2001,7 +2003,7 @@ BOOL CDECL nulldrv_AbortPath( PHYSDEV dev )
 
 BOOL CDECL nulldrv_CloseFigure( PHYSDEV dev )
 {
-    SetLastError( ERROR_CAN_NOT_COMPLETE );
+    RtlSetLastWin32Error( ERROR_CAN_NOT_COMPLETE );
     return FALSE;
 }
 
@@ -2118,8 +2120,9 @@ const struct gdi_dc_funcs path_driver =
     NULL,                               /* pStrokePath */
     NULL,                               /* pUnrealizePalette */
     NULL,                               /* pD3DKMTCheckVidPnExclusiveOwnership */
+    NULL,                               /* pD3DKMTCloseAdapter */
+    NULL,                               /* pD3DKMTOpenAdapterFromLuid */
+    NULL,                               /* pD3DKMTQueryVideoMemoryInfo */
     NULL,                               /* pD3DKMTSetVidPnSourceOwner */
-    NULL,                               /* wine_get_wgl_driver */
-    NULL,                               /* wine_get_vulkan_driver */
     GDI_PRIORITY_PATH_DRV               /* priority */
 };
