@@ -20,6 +20,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#if 0
+#pragma makedep unix
+#endif
+
 #define NONAMELESSUNION
 #define NONAMELESSSTRUCT
 
@@ -32,9 +36,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
 
 #define OEMRESOURCE
 #include "windef.h"
@@ -62,14 +64,7 @@ struct android_win_data
 
 #define SWP_AGG_NOPOSCHANGE (SWP_NOSIZE | SWP_NOMOVE | SWP_NOCLIENTSIZE | SWP_NOCLIENTMOVE | SWP_NOZORDER)
 
-static CRITICAL_SECTION win_data_section;
-static CRITICAL_SECTION_DEBUG critsect_debug =
-{
-    0, 0, &win_data_section,
-    { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": win_data_section") }
-};
-static CRITICAL_SECTION win_data_section = { &critsect_debug, -1, 0, 0, 0, 0 };
+pthread_mutex_t win_data_mutex;
 
 static struct android_win_data *win_data_context[32768];
 
@@ -101,19 +96,22 @@ static inline int get_dib_image_size( const BITMAPINFO *info )
         * abs( info->bmiHeader.biHeight );
 }
 
+static BOOL intersect_rect( RECT *dst, const RECT *src1, const RECT *src2 )
+{
+    dst->left   = max(src1->left, src2->left);
+    dst->top    = max(src1->top, src2->top);
+    dst->right  = min(src1->right, src2->right);
+    dst->bottom = min(src1->bottom, src2->bottom);
+    return !IsRectEmpty( dst );
+}
+
 
 /**********************************************************************
  *	     get_win_monitor_dpi
  */
 static UINT get_win_monitor_dpi( HWND hwnd )
 {
-    DPI_AWARENESS_CONTEXT context;
-    UINT ret;
-
-    context = SetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE );
-    ret = GetDpiForSystem();  /* FIXME: get monitor dpi */
-    SetThreadDpiAwarenessContext( context );
-    return ret;
+    return NtUserGetSystemDpiForProcess( NULL );  /* FIXME: get monitor dpi */
 }
 
 
@@ -124,12 +122,12 @@ static struct android_win_data *alloc_win_data( HWND hwnd )
 {
     struct android_win_data *data;
 
-    if ((data = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*data))))
+    if ((data = calloc( 1, sizeof(*data) )))
     {
         data->hwnd = hwnd;
         data->window = create_ioctl_window( hwnd, FALSE,
-                                            (float)get_win_monitor_dpi( hwnd ) / GetDpiForWindow( hwnd ));
-        EnterCriticalSection( &win_data_section );
+                                            (float)get_win_monitor_dpi( hwnd ) / NtUserGetDpiForWindow( hwnd ));
+        pthread_mutex_lock( &win_data_mutex );
         win_data_context[context_idx(hwnd)] = data;
     }
     return data;
@@ -142,9 +140,9 @@ static struct android_win_data *alloc_win_data( HWND hwnd )
 static void free_win_data( struct android_win_data *data )
 {
     win_data_context[context_idx( data->hwnd )] = NULL;
-    LeaveCriticalSection( &win_data_section );
+    pthread_mutex_unlock( &win_data_mutex );
     if (data->window) release_ioctl_window( data->window );
-    HeapFree( GetProcessHeap(), 0, data );
+    free( data );
 }
 
 
@@ -158,9 +156,9 @@ static struct android_win_data *get_win_data( HWND hwnd )
     struct android_win_data *data;
 
     if (!hwnd) return NULL;
-    EnterCriticalSection( &win_data_section );
+    pthread_mutex_lock( &win_data_mutex );
     if ((data = win_data_context[context_idx(hwnd)]) && data->hwnd == hwnd) return data;
-    LeaveCriticalSection( &win_data_section );
+    pthread_mutex_unlock( &win_data_mutex );
     return NULL;
 }
 
@@ -172,7 +170,7 @@ static struct android_win_data *get_win_data( HWND hwnd )
  */
 static void release_win_data( struct android_win_data *data )
 {
-    if (data) LeaveCriticalSection( &win_data_section );
+    if (data) pthread_mutex_unlock( &win_data_mutex );
 }
 
 
@@ -373,12 +371,12 @@ static void init_event_queue(void)
     if (pipe2( event_pipe, O_CLOEXEC | O_NONBLOCK ) == -1)
     {
         ERR( "could not create data\n" );
-        ExitProcess(1);
+        NtTerminateProcess( 0, 1 );
     }
     if (wine_server_fd_to_handle( event_pipe[0], GENERIC_READ | SYNCHRONIZE, 0, &handle ))
     {
         ERR( "Can't allocate handle for event fd\n" );
-        ExitProcess(1);
+        NtTerminateProcess( 0, 1 );
     }
     SERVER_START_REQ( set_queue_fd )
     {
@@ -389,9 +387,9 @@ static void init_event_queue(void)
     if (ret)
     {
         ERR( "Can't store handle for event fd %x\n", ret );
-        ExitProcess(1);
+        NtTerminateProcess( 0, 1 );
     }
-    CloseHandle( handle );
+    NtClose( handle );
     desktop_tid = GetCurrentThreadId();
 }
 
@@ -408,13 +406,13 @@ static void pull_events(void)
 
     for (;;)
     {
-        if (!(event = HeapAlloc( GetProcessHeap(), 0, sizeof(*event) ))) break;
+        if (!(event = malloc( sizeof(*event) ))) break;
 
         res = read( event_pipe[0], &event->data, sizeof(event->data) );
         if (res != sizeof(event->data)) break;
         list_add_tail( &event_queue, &event->entry );
     }
-    HeapFree( GetProcessHeap(), 0, event );
+    free( event );
 }
 
 
@@ -468,8 +466,8 @@ static int process_events( DWORD mask )
             screen_width = event->data.desktop.width;
             screen_height = event->data.desktop.height;
             init_monitors( screen_width, screen_height );
-            SetWindowPos( GetDesktopWindow(), 0, 0, 0, screen_width, screen_height,
-                          SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOREDRAW );
+            NtUserSetWindowPos( NtUserGetDesktopWindow(), 0, 0, 0, screen_width, screen_height,
+                                SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOREDRAW );
             SetThreadDpiAwarenessContext( context );
             break;
 
@@ -539,7 +537,7 @@ static int process_events( DWORD mask )
         default:
             FIXME( "got event %u\n", event->data.type );
         }
-        HeapFree( GetProcessHeap(), 0, event );
+        free( event );
         count++;
         /* next may have been removed by a recursive call, so reset it to the beginning of the list */
         next = LIST_ENTRY( event_queue.next, struct java_event, entry );
@@ -585,7 +583,7 @@ struct android_window_surface
     BYTE                  alpha;
     COLORREF              color_key;
     void                 *bits;
-    CRITICAL_SECTION      crit;
+    pthread_mutex_t       mutex;
     BITMAPINFO            info;   /* variable size, must be last */
 };
 
@@ -654,27 +652,27 @@ static void apply_line_region( DWORD *dst, int width, int x, int y, const RECT *
 /***********************************************************************
  *           android_surface_lock
  */
-static void CDECL android_surface_lock( struct window_surface *window_surface )
+static void android_surface_lock( struct window_surface *window_surface )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
 
-    EnterCriticalSection( &surface->crit );
+    pthread_mutex_lock( &surface->mutex );
 }
 
 /***********************************************************************
  *           android_surface_unlock
  */
-static void CDECL android_surface_unlock( struct window_surface *window_surface )
+static void android_surface_unlock( struct window_surface *window_surface )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
 
-    LeaveCriticalSection( &surface->crit );
+    pthread_mutex_unlock( &surface->mutex );
 }
 
 /***********************************************************************
  *           android_surface_get_bitmap_info
  */
-static void *CDECL android_surface_get_bitmap_info( struct window_surface *window_surface, BITMAPINFO *info )
+static void *android_surface_get_bitmap_info( struct window_surface *window_surface, BITMAPINFO *info )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
 
@@ -685,7 +683,7 @@ static void *CDECL android_surface_get_bitmap_info( struct window_surface *windo
 /***********************************************************************
  *           android_surface_get_bounds
  */
-static RECT *CDECL android_surface_get_bounds( struct window_surface *window_surface )
+static RECT *android_surface_get_bounds( struct window_surface *window_surface )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
 
@@ -695,7 +693,7 @@ static RECT *CDECL android_surface_get_bounds( struct window_surface *window_sur
 /***********************************************************************
  *           android_surface_set_region
  */
-static void CDECL android_surface_set_region( struct window_surface *window_surface, HRGN region )
+static void android_surface_set_region( struct window_surface *window_surface, HRGN region )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
 
@@ -704,13 +702,13 @@ static void CDECL android_surface_set_region( struct window_surface *window_surf
     window_surface->funcs->lock( window_surface );
     if (!region)
     {
-        if (surface->region) DeleteObject( surface->region );
+        if (surface->region) NtGdiDeleteObjectApp( surface->region );
         surface->region = 0;
     }
     else
     {
-        if (!surface->region) surface->region = CreateRectRgn( 0, 0, 0, 0 );
-        CombineRgn( surface->region, region, 0, RGN_COPY );
+        if (!surface->region) surface->region = NtGdiCreateRectRgn( 0, 0, 0, 0 );
+        NtGdiCombineRgn( surface->region, region, 0, RGN_COPY );
     }
     window_surface->funcs->unlock( window_surface );
     set_surface_region( &surface->header, (HRGN)1 );
@@ -719,7 +717,7 @@ static void CDECL android_surface_set_region( struct window_surface *window_surf
 /***********************************************************************
  *           android_surface_flush
  */
-static void CDECL android_surface_flush( struct window_surface *window_surface )
+static void android_surface_flush( struct window_surface *window_surface )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
     ANativeWindow_Buffer buffer;
@@ -730,7 +728,7 @@ static void CDECL android_surface_flush( struct window_surface *window_surface )
     window_surface->funcs->lock( window_surface );
     SetRect( &rect, 0, 0, surface->header.rect.right - surface->header.rect.left,
              surface->header.rect.bottom - surface->header.rect.top );
-    needs_flush = IntersectRect( &rect, &rect, &surface->bounds );
+    needs_flush = intersect_rect( &rect, &rect, &surface->bounds );
     reset_bounds( &surface->bounds );
     window_surface->funcs->unlock( window_surface );
     if (!needs_flush) return;
@@ -755,7 +753,7 @@ static void CDECL android_surface_flush( struct window_surface *window_surface )
         rect.top    = rc.top;
         rect.right  = rc.right;
         rect.bottom = rc.bottom;
-        IntersectRect( &rect, &rect, &surface->header.rect );
+        intersect_rect( &rect, &rect, &surface->header.rect );
 
         if (surface->region_data)
         {
@@ -802,19 +800,17 @@ static void CDECL android_surface_flush( struct window_surface *window_surface )
 /***********************************************************************
  *           android_surface_destroy
  */
-static void CDECL android_surface_destroy( struct window_surface *window_surface )
+static void android_surface_destroy( struct window_surface *window_surface )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
 
     TRACE( "freeing %p bits %p\n", surface, surface->bits );
 
-    surface->crit.DebugInfo->Spare[0] = 0;
-    DeleteCriticalSection( &surface->crit );
-    HeapFree( GetProcessHeap(), 0, surface->region_data );
-    if (surface->region) DeleteObject( surface->region );
+    free( surface->region_data );
+    if (surface->region) NtGdiDeleteObjectApp( surface->region );
     release_ioctl_window( surface->window );
-    HeapFree( GetProcessHeap(), 0, surface->bits );
-    HeapFree( GetProcessHeap(), 0, surface );
+    free( surface->bits );
+    free( surface );
 }
 
 static const struct window_surface_funcs android_surface_funcs =
@@ -874,30 +870,30 @@ static void set_surface_region( struct window_surface *window_surface, HRGN win_
 
     if (win_region == (HRGN)1)  /* hack: win_region == 1 means retrieve region from server */
     {
-        region = CreateRectRgn( 0, 0, win_data->window_rect.right - win_data->window_rect.left,
-                                win_data->window_rect.bottom - win_data->window_rect.top );
-        if (GetWindowRgn( surface->hwnd, region ) == ERROR && !surface->region) goto done;
+        region = NtGdiCreateRectRgn( 0, 0, win_data->window_rect.right - win_data->window_rect.left,
+                                     win_data->window_rect.bottom - win_data->window_rect.top );
+        if (NtUserGetWindowRgnEx( surface->hwnd, region, 0 ) == ERROR && !surface->region) goto done;
     }
 
-    OffsetRgn( region, offset_x, offset_y );
-    if (surface->region) CombineRgn( region, region, surface->region, RGN_AND );
+    NtGdiOffsetRgn( region, offset_x, offset_y );
+    if (surface->region) NtGdiCombineRgn( region, region, surface->region, RGN_AND );
 
-    if (!(size = GetRegionData( region, 0, NULL ))) goto done;
-    if (!(data = HeapAlloc( GetProcessHeap(), 0, size ))) goto done;
+    if (!(size = NtGdiGetRegionData( region, 0, NULL ))) goto done;
+    if (!(data = malloc( size ))) goto done;
 
-    if (!GetRegionData( region, size, data ))
+    if (!NtGdiGetRegionData( region, size, data ))
     {
-        HeapFree( GetProcessHeap(), 0, data );
+        free( data );
         data = NULL;
     }
 
 done:
     window_surface->funcs->lock( window_surface );
-    HeapFree( GetProcessHeap(), 0, surface->region_data );
+    free( surface->region_data );
     surface->region_data = data;
     *window_surface->funcs->get_bounds( window_surface ) = surface->header.rect;
     window_surface->funcs->unlock( window_surface );
-    if (region != win_region) DeleteObject( region );
+    if (region != win_region) NtGdiDeleteObjectApp( region );
 }
 
 /***********************************************************************
@@ -908,9 +904,9 @@ static struct window_surface *create_surface( HWND hwnd, const RECT *rect,
 {
     struct android_window_surface *surface;
     int width = rect->right - rect->left, height = rect->bottom - rect->top;
+    pthread_mutexattr_t attr;
 
-    surface = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY,
-                         FIELD_OFFSET( struct android_window_surface, info.bmiColors[3] ));
+    surface = calloc( 1, FIELD_OFFSET( struct android_window_surface, info.bmiColors[3] ));
     if (!surface) return NULL;
     set_color_info( &surface->info, src_alpha );
     surface->info.bmiHeader.biWidth       = width;
@@ -918,8 +914,10 @@ static struct window_surface *create_surface( HWND hwnd, const RECT *rect,
     surface->info.bmiHeader.biPlanes      = 1;
     surface->info.bmiHeader.biSizeImage   = get_dib_image_size( &surface->info );
 
-    InitializeCriticalSection( &surface->crit );
-    surface->crit.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": surface");
+    pthread_mutexattr_init( &attr );
+    pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
+    pthread_mutex_init( &surface->mutex, &attr );
+    pthread_mutexattr_destroy( &attr );
 
     surface->header.funcs = &android_surface_funcs;
     surface->header.rect  = *rect;
@@ -931,7 +929,7 @@ static struct window_surface *create_surface( HWND hwnd, const RECT *rect,
     set_surface_region( &surface->header, (HRGN)1 );
     reset_bounds( &surface->bounds );
 
-    if (!(surface->bits = HeapAlloc( GetProcessHeap(), 0, surface->info.bmiHeader.biSizeImage )))
+    if (!(surface->bits = malloc( surface->info.bmiHeader.biSizeImage )))
         goto failed;
 
     TRACE( "created %p hwnd %p %s bits %p-%p\n", surface, hwnd, wine_dbgstr_rect(rect),
@@ -976,15 +974,15 @@ static unsigned int *get_mono_icon_argb( HDC hdc, HBITMAP bmp, unsigned int *wid
     char *mask;
     unsigned int i, j, stride, mask_size, bits_size, *bits = NULL, *ptr;
 
-    if (!GetObjectW( bmp, sizeof(bm), &bm )) return NULL;
+    if (!NtGdiExtGetObjectW( bmp, sizeof(bm), &bm )) return NULL;
     stride = ((bm.bmWidth + 15) >> 3) & ~1;
     mask_size = stride * bm.bmHeight;
-    if (!(mask = HeapAlloc( GetProcessHeap(), 0, mask_size ))) return NULL;
-    if (!GetBitmapBits( bmp, mask_size, mask )) goto done;
+    if (!(mask = malloc( mask_size ))) return NULL;
+    if (!NtGdiGetBitmapBits( bmp, mask_size, mask )) goto done;
 
     bm.bmHeight /= 2;
     bits_size = bm.bmWidth * bm.bmHeight * sizeof(*bits);
-    if (!(bits = HeapAlloc( GetProcessHeap(), 0, bits_size ))) goto done;
+    if (!(bits = malloc( bits_size ))) goto done;
 
     ptr = bits;
     for (i = 0; i < bm.bmHeight; i++)
@@ -1005,7 +1003,7 @@ static unsigned int *get_mono_icon_argb( HDC hdc, HBITMAP bmp, unsigned int *wid
     *height = bm.bmHeight;
 
 done:
-    HeapFree( GetProcessHeap(), 0, mask );
+    free( mask );
     return bits;
 }
 
@@ -1027,7 +1025,7 @@ static unsigned int *get_bitmap_argb( HDC hdc, HBITMAP color, HBITMAP mask, unsi
 
     if (!color) return get_mono_icon_argb( hdc, mask, width, height );
 
-    if (!GetObjectW( color, sizeof(bm), &bm )) return NULL;
+    if (!NtGdiExtGetObjectW( color, sizeof(bm), &bm )) return NULL;
     info->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
     info->bmiHeader.biWidth = bm.bmWidth;
     info->bmiHeader.biHeight = -bm.bmHeight;
@@ -1039,9 +1037,10 @@ static unsigned int *get_bitmap_argb( HDC hdc, HBITMAP color, HBITMAP mask, unsi
     info->bmiHeader.biYPelsPerMeter = 0;
     info->bmiHeader.biClrUsed = 0;
     info->bmiHeader.biClrImportant = 0;
-    if (!(bits = HeapAlloc( GetProcessHeap(), 0, bm.bmWidth * bm.bmHeight * sizeof(unsigned int) )))
+    if (!(bits = malloc( bm.bmWidth * bm.bmHeight * sizeof(unsigned int) )))
+        goto failed;
+    if (!NtGdiGetDIBitsInternal( hdc, color, 0, bm.bmHeight, bits, info, DIB_RGB_COLORS, 0, 0 ))
         goto failed;
-    if (!GetDIBits( hdc, color, 0, bm.bmHeight, bits, info, DIB_RGB_COLORS )) goto failed;
 
     *width = bm.bmWidth;
     *height = bm.bmHeight;
@@ -1055,20 +1054,21 @@ static unsigned int *get_bitmap_argb( HDC hdc, HBITMAP color, HBITMAP mask, unsi
         /* generate alpha channel from the mask */
         info->bmiHeader.biBitCount = 1;
         info->bmiHeader.biSizeImage = width_bytes * bm.bmHeight;
-        if (!(mask_bits = HeapAlloc( GetProcessHeap(), 0, info->bmiHeader.biSizeImage ))) goto failed;
-        if (!GetDIBits( hdc, mask, 0, bm.bmHeight, mask_bits, info, DIB_RGB_COLORS )) goto failed;
+        if (!(mask_bits = malloc( info->bmiHeader.biSizeImage ))) goto failed;
+        if (!NtGdiGetDIBitsInternal( hdc, mask, 0, bm.bmHeight, mask_bits, info, DIB_RGB_COLORS, 0, 0 ))
+            goto failed;
         ptr = bits;
         for (i = 0; i < bm.bmHeight; i++)
             for (j = 0; j < bm.bmWidth; j++, ptr++)
                 if (!((mask_bits[i * width_bytes + j / 8] << (j % 8)) & 0x80)) *ptr |= 0xff000000;
-        HeapFree( GetProcessHeap(), 0, mask_bits );
+        free( mask_bits );
     }
 
     return bits;
 
 failed:
-    HeapFree( GetProcessHeap(), 0, bits );
-    HeapFree( GetProcessHeap(), 0, mask_bits );
+    free( bits );
+    free( mask_bits );
     *width = *height = 0;
     return NULL;
 }
@@ -1168,14 +1168,15 @@ static const struct
 static int get_cursor_system_id( const ICONINFOEXW *info )
 {
     const struct system_cursors *cursors;
+    const WCHAR *module;
     unsigned int i;
-    HMODULE module;
 
     if (info->szResName[0]) return 0;  /* only integer resources are supported here */
-    if (!(module = GetModuleHandleW( info->szModName ))) return 0;
 
+    if ((module = wcsrchr( info->szModName, '\\' ))) module++;
+    else module = info->szModName;
     for (i = 0; i < ARRAY_SIZE( module_cursors ); i++)
-        if (GetModuleHandleW( module_cursors[i].name ) == module) break;
+        if (!wcsicmp( module, module_cursors[i].name )) break;
     if (i == ARRAY_SIZE( module_cursors )) return 0;
 
     cursors = module_cursors[i].cursors;
@@ -1186,9 +1187,7 @@ static int get_cursor_system_id( const ICONINFOEXW *info )
 }
 
 
-static WNDPROC desktop_orig_wndproc;
-
-static LRESULT CALLBACK desktop_wndproc_wrapper( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
+LRESULT ANDROID_DesktopWindowProc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
 {
     switch (msg)
     {
@@ -1196,15 +1195,16 @@ static LRESULT CALLBACK desktop_wndproc_wrapper( HWND hwnd, UINT msg, WPARAM wp,
         if (LOWORD(wp) == WM_DESTROY) destroy_ioctl_window( (HWND)lp, FALSE );
         break;
     }
-    return desktop_orig_wndproc( hwnd, msg, wp, lp );
+    return NtUserMessageCall( hwnd, msg, wp, lp, 0, NtUserDefWindowProc, FALSE );
 }
 
 
 /***********************************************************************
  *           ANDROID_MsgWaitForMultipleObjectsEx
  */
-DWORD CDECL ANDROID_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *handles,
-                                                 DWORD timeout, DWORD mask, DWORD flags )
+NTSTATUS ANDROID_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *handles,
+                                              const LARGE_INTEGER *timeout,
+                                              DWORD mask, DWORD flags )
 {
     if (GetCurrentThreadId() == desktop_tid)
     {
@@ -1212,18 +1212,18 @@ DWORD CDECL ANDROID_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *hand
         if (current_event) mask = 0;
         if (process_events( mask )) return count - 1;
     }
-    return WaitForMultipleObjectsEx( count, handles, flags & MWMO_WAITALL,
-                                     timeout, flags & MWMO_ALERTABLE );
+    return NtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
+                                     !!(flags & MWMO_ALERTABLE), timeout );
 }
 
 /**********************************************************************
  *           ANDROID_CreateWindow
  */
-BOOL CDECL ANDROID_CreateWindow( HWND hwnd )
+BOOL ANDROID_CreateWindow( HWND hwnd )
 {
     TRACE( "%p\n", hwnd );
 
-    if (hwnd == GetDesktopWindow())
+    if (hwnd == NtUserGetDesktopWindow())
     {
         struct android_win_data *data;
 
@@ -1239,7 +1239,7 @@ BOOL CDECL ANDROID_CreateWindow( HWND hwnd )
 /***********************************************************************
  *           ANDROID_DestroyWindow
  */
-void CDECL ANDROID_DestroyWindow( HWND hwnd )
+void ANDROID_DestroyWindow( HWND hwnd )
 {
     struct android_win_data *data;
 
@@ -1263,11 +1263,11 @@ static struct android_win_data *create_win_data( HWND hwnd, const RECT *window_r
     struct android_win_data *data;
     HWND parent;
 
-    if (!(parent = GetAncestor( hwnd, GA_PARENT ))) return NULL;  /* desktop or HWND_MESSAGE */
+    if (!(parent = NtUserGetAncestor( hwnd, GA_PARENT ))) return NULL;  /* desktop or HWND_MESSAGE */
 
     if (!(data = alloc_win_data( hwnd ))) return NULL;
 
-    data->parent = (parent == GetDesktopWindow()) ? 0 : parent;
+    data->parent = (parent == NtUserGetDesktopWindow()) ? 0 : parent;
     data->whole_rect = data->window_rect = *window_rect;
     data->client_rect = *client_rect;
     return data;
@@ -1276,7 +1276,7 @@ static struct android_win_data *create_win_data( HWND hwnd, const RECT *window_r
 
 static inline BOOL get_surface_rect( const RECT *visible_rect, RECT *surface_rect )
 {
-    if (!IntersectRect( surface_rect, visible_rect, &virtual_screen_rect )) return FALSE;
+    if (!intersect_rect( surface_rect, visible_rect, &virtual_screen_rect )) return FALSE;
     OffsetRect( surface_rect, -visible_rect->left, -visible_rect->top );
     surface_rect->left &= ~31;
     surface_rect->top  &= ~31;
@@ -1289,20 +1289,20 @@ static inline BOOL get_surface_rect( const RECT *visible_rect, RECT *surface_rec
 /***********************************************************************
  *           ANDROID_WindowPosChanging
  */
-BOOL CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
-                                      const RECT *window_rect, const RECT *client_rect, RECT *visible_rect,
-                                      struct window_surface **surface )
+BOOL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
+                                const RECT *window_rect, const RECT *client_rect, RECT *visible_rect,
+                                struct window_surface **surface )
 {
     struct android_win_data *data = get_win_data( hwnd );
     RECT surface_rect;
     DWORD flags;
     COLORREF key;
     BYTE alpha;
-    BOOL layered = GetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_LAYERED;
+    BOOL layered = NtUserGetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_LAYERED;
 
     TRACE( "win %p window %s client %s style %08x flags %08x\n",
            hwnd, wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
-           GetWindowLongW( hwnd, GWL_STYLE ), swp_flags );
+           NtUserGetWindowLongW( hwnd, GWL_STYLE ), swp_flags );
 
     if (!data && !(data = create_win_data( hwnd, window_rect, client_rect ))) return TRUE;
 
@@ -1326,9 +1326,10 @@ BOOL CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_fla
             goto done;
         }
     }
-    if (!(swp_flags & SWP_SHOWWINDOW) && !(GetWindowLongW( hwnd, GWL_STYLE ) & WS_VISIBLE)) goto done;
+    if (!(swp_flags & SWP_SHOWWINDOW) && !(NtUserGetWindowLongW( hwnd, GWL_STYLE ) & WS_VISIBLE))
+        goto done;
 
-    if (!layered || !GetLayeredWindowAttributes( hwnd, &key, &alpha, &flags )) flags = 0;
+    if (!layered || !NtUserGetLayeredWindowAttributes( hwnd, &key, &alpha, &flags )) flags = 0;
     if (!(flags & LWA_ALPHA)) alpha = 255;
     if (!(flags & LWA_COLORKEY)) key = CLR_INVALID;
 
@@ -1344,13 +1345,13 @@ done:
 /***********************************************************************
  *           ANDROID_WindowPosChanged
  */
-void CDECL ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags,
-                                    const RECT *window_rect, const RECT *client_rect,
-                                    const RECT *visible_rect, const RECT *valid_rects,
-                                    struct window_surface *surface )
+void ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags,
+                               const RECT *window_rect, const RECT *client_rect,
+                               const RECT *visible_rect, const RECT *valid_rects,
+                               struct window_surface *surface )
 {
     struct android_win_data *data;
-    DWORD new_style = GetWindowLongW( hwnd, GWL_STYLE );
+    DWORD new_style = NtUserGetWindowLongW( hwnd, GWL_STYLE );
     HWND owner = 0;
 
     if (!(data = get_win_data( hwnd ))) return;
@@ -1365,10 +1366,10 @@ void CDECL ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flag
         if (data->surface) window_surface_release( data->surface );
         data->surface = surface;
     }
-    if (!data->parent) owner = GetWindow( hwnd, GW_OWNER );
+    if (!data->parent) owner = NtUserGetWindowRelative( hwnd, GW_OWNER );
     release_win_data( data );
 
-    if (!(swp_flags & SWP_NOZORDER)) insert_after = GetWindow( hwnd, GW_HWNDPREV );
+    if (!(swp_flags & SWP_NOZORDER)) insert_after = NtUserGetWindowRelative( hwnd, GW_HWNDPREV );
 
     TRACE( "win %p window %s client %s style %08x owner %p after %p flags %08x\n", hwnd,
            wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
@@ -1382,9 +1383,9 @@ void CDECL ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flag
 /***********************************************************************
  *           ANDROID_ShowWindow
  */
-UINT CDECL ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
+UINT ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
 {
-    if (!IsIconic( hwnd )) return swp;
+    if (!(NtUserGetWindowLongW( hwnd, GWL_STYLE ) & WS_MINIMIZE)) return swp;
     /* always hide icons off-screen */
     if (rect->left != -32000 || rect->top != -32000)
     {
@@ -1398,7 +1399,7 @@ UINT CDECL ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
 /*****************************************************************
  *	     ANDROID_SetParent
  */
-void CDECL ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
+void ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
 {
     struct android_win_data *data;
 
@@ -1407,8 +1408,8 @@ void CDECL ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
 
     TRACE( "win %p parent %p -> %p\n", hwnd, old_parent, parent );
 
-    data->parent = (parent == GetDesktopWindow()) ? 0 : parent;
-    ioctl_set_window_parent( hwnd, parent, (float)get_win_monitor_dpi( hwnd ) / GetDpiForWindow( hwnd ));
+    data->parent = (parent == NtUserGetDesktopWindow()) ? 0 : parent;
+    ioctl_set_window_parent( hwnd, parent, (float)get_win_monitor_dpi( hwnd ) / NtUserGetDpiForWindow( hwnd ));
     release_win_data( data );
 }
 
@@ -1416,25 +1417,47 @@ void CDECL ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
 /***********************************************************************
  *           ANDROID_SetCapture
  */
-void CDECL ANDROID_SetCapture( HWND hwnd, UINT flags )
+void ANDROID_SetCapture( HWND hwnd, UINT flags )
 {
     if (!(flags & (GUI_INMOVESIZE | GUI_INMENUMODE))) return;
     ioctl_set_capture( hwnd );
 }
 
 
+static BOOL get_icon_info( HICON handle, ICONINFOEXW *ret )
+{
+    UNICODE_STRING module, res_name;
+    ICONINFO info;
+
+    module.Buffer = ret->szModName;
+    module.MaximumLength = sizeof(ret->szModName) - sizeof(WCHAR);
+    res_name.Buffer = ret->szResName;
+    res_name.MaximumLength = sizeof(ret->szResName) - sizeof(WCHAR);
+    if (!NtUserGetIconInfo( handle, &info, &module, &res_name, NULL, 0 )) return FALSE;
+    ret->fIcon    = info.fIcon;
+    ret->xHotspot = info.xHotspot;
+    ret->yHotspot = info.yHotspot;
+    ret->hbmColor = info.hbmColor;
+    ret->hbmMask  = info.hbmMask;
+    ret->wResID   = res_name.Length ? 0 : LOWORD(res_name.Buffer);
+    ret->szModName[module.Length] = 0;
+    ret->szResName[res_name.Length] = 0;
+    return TRUE;
+}
+
+
 /***********************************************************************
  *           ANDROID_SetCursor
  */
-void CDECL ANDROID_SetCursor( HCURSOR handle )
+void ANDROID_SetCursor( HCURSOR handle )
 {
     static HCURSOR last_cursor;
     static DWORD last_cursor_change;
 
     if (InterlockedExchangePointer( (void **)&last_cursor, handle ) != handle ||
-        GetTickCount() - last_cursor_change > 100)
+        NtGetTickCount() - last_cursor_change > 100)
     {
-        last_cursor_change = GetTickCount();
+        last_cursor_change = NtGetTickCount();
 
         if (handle)
         {
@@ -1442,14 +1465,13 @@ void CDECL ANDROID_SetCursor( HCURSOR handle )
             ICONINFOEXW info;
             int id;
 
-            info.cbSize = sizeof(info);
-            if (!GetIconInfoExW( handle, &info )) return;
+            if (!get_icon_info( handle, &info )) return;
 
             if (!(id = get_cursor_system_id( &info )))
             {
-                HDC hdc = CreateCompatibleDC( 0 );
+                HDC hdc = NtGdiCreateCompatibleDC( 0 );
                 bits = get_bitmap_argb( hdc, info.hbmColor, info.hbmMask, &width, &height );
-                DeleteDC( hdc );
+                NtGdiDeleteObjectApp( hdc );
 
                 /* make sure hotspot is valid */
                 if (info.xHotspot >= width || info.yHotspot >= height)
@@ -1459,9 +1481,9 @@ void CDECL ANDROID_SetCursor( HCURSOR handle )
                 }
             }
             ioctl_set_cursor( id, width, height, info.xHotspot, info.yHotspot, bits );
-            HeapFree( GetProcessHeap(), 0, bits );
-            DeleteObject( info.hbmColor );
-            DeleteObject( info.hbmMask );
+            free( bits );
+            NtGdiDeleteObjectApp( info.hbmColor );
+            NtGdiDeleteObjectApp( info.hbmMask );
         }
         else ioctl_set_cursor( 0, 0, 0, 0, 0, NULL );
     }
@@ -1471,12 +1493,12 @@ void CDECL ANDROID_SetCursor( HCURSOR handle )
 /***********************************************************************
  *           ANDROID_SetWindowStyle
  */
-void CDECL ANDROID_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
+void ANDROID_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
 {
     struct android_win_data *data;
     DWORD changed = style->styleNew ^ style->styleOld;
 
-    if (hwnd == GetDesktopWindow()) return;
+    if (hwnd == NtUserGetDesktopWindow()) return;
     if (!(data = get_win_data( hwnd ))) return;
 
     if (offset == GWL_EXSTYLE && (changed & WS_EX_LAYERED)) /* changing WS_EX_LAYERED resets attributes */
@@ -1495,7 +1517,7 @@ void CDECL ANDROID_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
 /***********************************************************************
  *           ANDROID_SetWindowRgn
  */
-void CDECL ANDROID_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
+void ANDROID_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
 {
     struct android_win_data *data;
 
@@ -1511,7 +1533,7 @@ void CDECL ANDROID_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
 /***********************************************************************
  *	     ANDROID_SetLayeredWindowAttributes
  */
-void CDECL ANDROID_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alpha, DWORD flags )
+void ANDROID_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alpha, DWORD flags )
 {
     struct android_win_data *data;
 
@@ -1529,8 +1551,8 @@ void CDECL ANDROID_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alp
 /*****************************************************************************
  *           ANDROID_UpdateLayeredWindow
  */
-BOOL CDECL ANDROID_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO *info,
-                                        const RECT *window_rect )
+BOOL ANDROID_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO *info,
+                                  const RECT *window_rect )
 {
     struct window_surface *surface;
     struct android_win_data *data;
@@ -1576,27 +1598,28 @@ BOOL CDECL ANDROID_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO
 
     dst_bits = surface->funcs->get_info( surface, bmi );
 
-    if (!(dib = CreateDIBSection( info->hdcDst, bmi, DIB_RGB_COLORS, &src_bits, NULL, 0 ))) goto done;
-    if (!(hdc = CreateCompatibleDC( 0 ))) goto done;
+    if (!(dib = NtGdiCreateDIBSection( info->hdcDst, NULL, 0, bmi, DIB_RGB_COLORS, 0, 0, 0, &src_bits )))
+        goto done;
+    if (!(hdc = NtGdiCreateCompatibleDC( 0 ))) goto done;
 
-    SelectObject( hdc, dib );
+    NtGdiSelectBitmap( hdc, dib );
 
     surface->funcs->lock( surface );
 
     if (info->prcDirty)
     {
-        IntersectRect( &rect, &rect, info->prcDirty );
+        intersect_rect( &rect, &rect, info->prcDirty );
         memcpy( src_bits, dst_bits, bmi->bmiHeader.biSizeImage );
-        PatBlt( hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, BLACKNESS );
+        NtGdiPatBlt( hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, BLACKNESS );
     }
     src_rect = rect;
     if (info->pptSrc) OffsetRect( &src_rect, info->pptSrc->x, info->pptSrc->y );
-    DPtoLP( info->hdcSrc, (POINT *)&src_rect, 2 );
+    NtGdiTransformPoints( info->hdcSrc, (POINT *)&src_rect, (POINT *)&src_rect, 2, NtGdiDPtoLP );
 
-    ret = GdiAlphaBlend( hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
-                         info->hdcSrc, src_rect.left, src_rect.top,
-                         src_rect.right - src_rect.left, src_rect.bottom - src_rect.top,
-                         (info->dwFlags & ULW_ALPHA) ? *info->pblend : blend );
+    ret = NtGdiAlphaBlend( hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
+                           info->hdcSrc, src_rect.left, src_rect.top,
+                           src_rect.right - src_rect.left, src_rect.bottom - src_rect.top,
+                           (info->dwFlags & ULW_ALPHA) ? *info->pblend : blend, 0 );
     if (ret)
     {
         memcpy( dst_bits, src_bits, bmi->bmiHeader.biSizeImage );
@@ -1608,8 +1631,8 @@ BOOL CDECL ANDROID_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO
 
 done:
     window_surface_release( surface );
-    if (hdc) DeleteDC( hdc );
-    if (dib) DeleteObject( dib );
+    if (hdc) NtGdiDeleteObjectApp( hdc );
+    if (dib) NtGdiDeleteObjectApp( dib );
     return ret;
 }
 
@@ -1617,7 +1640,7 @@ done:
 /**********************************************************************
  *           ANDROID_WindowMessage
  */
-LRESULT CDECL ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
+LRESULT ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
 {
     struct android_win_data *data;
 
@@ -1649,13 +1672,10 @@ LRESULT CDECL ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
 
 
 /***********************************************************************
- *           ANDROID_create_desktop
+ *           android_create_desktop
  */
-BOOL CDECL ANDROID_create_desktop( UINT width, UINT height )
+NTSTATUS android_create_desktop( void *arg )
 {
-    desktop_orig_wndproc = (WNDPROC)SetWindowLongPtrW( GetDesktopWindow(), GWLP_WNDPROC,
-                                                       (LONG_PTR)desktop_wndproc_wrapper );
-
     /* wait until we receive the surface changed event */
     while (!screen_width)
     {
@@ -1666,5 +1686,5 @@ BOOL CDECL ANDROID_create_desktop( UINT width, UINT height )
         }
         process_events( QS_ALLINPUT );
     }
-    return TRUE;
+    return 0;
 }
