@@ -555,16 +555,16 @@ static struct base_device *find_device_from_syspath(const char *path)
 
 #define test_bit(arr,bit) (((BYTE*)(arr))[(bit)>>3]&(1<<((bit)&7)))
 
-static const BYTE* what_am_I(struct udev_device *dev)
+static const USAGE_AND_PAGE *what_am_I(struct udev_device *dev, int fd)
 {
-    static const BYTE Unknown[2]     = {HID_USAGE_PAGE_GENERIC, 0};
-    static const BYTE Mouse[2]       = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_MOUSE};
-    static const BYTE Keyboard[2]    = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_KEYBOARD};
-    static const BYTE Gamepad[2]     = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_GAMEPAD};
-    static const BYTE Keypad[2]      = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_KEYPAD};
-    static const BYTE Tablet[2]      = {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_PEN};
-    static const BYTE Touchscreen[2] = {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_TOUCH_SCREEN};
-    static const BYTE Touchpad[2]    = {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_TOUCH_PAD};
+    static const USAGE_AND_PAGE Unknown     = {.UsagePage = HID_USAGE_PAGE_GENERIC, .Usage = 0};
+    static const USAGE_AND_PAGE Mouse       = {.UsagePage = HID_USAGE_PAGE_GENERIC, .Usage = HID_USAGE_GENERIC_MOUSE};
+    static const USAGE_AND_PAGE Keyboard    = {.UsagePage = HID_USAGE_PAGE_GENERIC, .Usage = HID_USAGE_GENERIC_KEYBOARD};
+    static const USAGE_AND_PAGE Gamepad     = {.UsagePage = HID_USAGE_PAGE_GENERIC, .Usage = HID_USAGE_GENERIC_GAMEPAD};
+    static const USAGE_AND_PAGE Keypad      = {.UsagePage = HID_USAGE_PAGE_GENERIC, .Usage = HID_USAGE_GENERIC_KEYPAD};
+    static const USAGE_AND_PAGE Tablet      = {.UsagePage = HID_USAGE_PAGE_DIGITIZER, .Usage = HID_USAGE_DIGITIZER_PEN};
+    static const USAGE_AND_PAGE Touchscreen = {.UsagePage = HID_USAGE_PAGE_DIGITIZER, .Usage = HID_USAGE_DIGITIZER_TOUCH_SCREEN};
+    static const USAGE_AND_PAGE Touchpad    = {.UsagePage = HID_USAGE_PAGE_DIGITIZER, .Usage = HID_USAGE_DIGITIZER_TOUCH_PAD};
 
     struct udev_device *parent = dev;
 
@@ -572,23 +572,24 @@ static const BYTE* what_am_I(struct udev_device *dev)
     while (parent)
     {
         if (udev_device_get_property_value(parent, "ID_INPUT_MOUSE"))
-            return Mouse;
+            return &Mouse;
         else if (udev_device_get_property_value(parent, "ID_INPUT_KEYBOARD"))
-            return Keyboard;
+            return &Keyboard;
         else if (udev_device_get_property_value(parent, "ID_INPUT_JOYSTICK"))
-            return Gamepad;
+            return &Gamepad;
         else if (udev_device_get_property_value(parent, "ID_INPUT_KEY"))
-            return Keypad;
+            return &Keypad;
         else if (udev_device_get_property_value(parent, "ID_INPUT_TOUCHPAD"))
-            return Touchpad;
+            return &Touchpad;
         else if (udev_device_get_property_value(parent, "ID_INPUT_TOUCHSCREEN"))
-            return Touchscreen;
+            return &Touchscreen;
         else if (udev_device_get_property_value(parent, "ID_INPUT_TABLET"))
-            return Tablet;
+            return &Tablet;
 
         parent = udev_device_get_parent_with_subsystem_devtype(parent, "input", NULL);
     }
-    return Unknown;
+
+    return &Unknown;
 }
 
 static INT count_buttons(int device_fd, BYTE *map)
@@ -642,8 +643,8 @@ static NTSTATUS build_report_descriptor(struct unix_device *iface, struct udev_d
     USHORT count = 0;
     USAGE usages[16];
     INT i, button_count, abs_count, rel_count, hat_count;
-    const BYTE *device_usage = what_am_I(dev);
     struct lnxev_device *impl = lnxev_impl_from_unix_device(iface);
+    const USAGE_AND_PAGE device_usage = *what_am_I(dev, impl->base.device_fd);
 
     if (ioctl(impl->base.device_fd, EVIOCGBIT(EV_REL, sizeof(relbits)), relbits) == -1)
     {
@@ -661,10 +662,10 @@ static NTSTATUS build_report_descriptor(struct unix_device *iface, struct udev_d
         memset(ffbits, 0, sizeof(ffbits));
     }
 
-    if (!hid_device_begin_report_descriptor(iface, device_usage[0], device_usage[1]))
+    if (!hid_device_begin_report_descriptor(iface, &device_usage))
         return STATUS_NO_MEMORY;
 
-    if (!hid_device_begin_input_report(iface))
+    if (!hid_device_begin_input_report(iface, &device_usage))
         return STATUS_NO_MEMORY;
 
     abs_count = 0;
@@ -866,7 +867,8 @@ static void lnxev_device_read_report(struct unix_device *iface)
 }
 
 static NTSTATUS lnxev_device_haptics_start(struct unix_device *iface, UINT duration_ms,
-                                           USHORT rumble_intensity, USHORT buzz_intensity)
+                                           USHORT rumble_intensity, USHORT buzz_intensity,
+                                           USHORT left_intensity, USHORT right_intensity)
 {
     struct lnxev_device *impl = lnxev_impl_from_unix_device(iface);
     struct ff_effect effect =
@@ -876,8 +878,8 @@ static NTSTATUS lnxev_device_haptics_start(struct unix_device *iface, UINT durat
     };
     struct input_event event;
 
-    TRACE("iface %p, duration_ms %u, rumble_intensity %u, buzz_intensity %u.\n", iface,
-          duration_ms, rumble_intensity, buzz_intensity);
+    TRACE("iface %p, duration_ms %u, rumble_intensity %u, buzz_intensity %u, left_intensity %u, right_intensity %u.\n",
+          iface, duration_ms, rumble_intensity, buzz_intensity, left_intensity, right_intensity);
 
     effect.replay.length = duration_ms;
     effect.u.rumble.strong_magnitude = rumble_intensity;
@@ -1136,7 +1138,7 @@ static NTSTATUS lnxev_device_physical_effect_update(struct unix_device *iface, B
     case PID_USAGE_ET_SAWTOOTH_UP:
     case PID_USAGE_ET_SAWTOOTH_DOWN:
         effect.u.periodic.period = params->periodic.period;
-        effect.u.periodic.magnitude = params->periodic.magnitude;
+        effect.u.periodic.magnitude = (params->periodic.magnitude * params->gain_percent) / 100;
         effect.u.periodic.offset = params->periodic.offset;
         effect.u.periodic.phase = params->periodic.phase * 0x800 / 1125;
         effect.u.periodic.envelope.attack_length = params->envelope.attack_time;
@@ -1170,7 +1172,7 @@ static NTSTATUS lnxev_device_physical_effect_update(struct unix_device *iface, B
         break;
 
     case PID_USAGE_ET_CONSTANT_FORCE:
-        effect.u.constant.level = params->constant_force.magnitude;
+        effect.u.constant.level = (params->constant_force.magnitude * params->gain_percent) / 100;
         effect.u.constant.envelope.attack_length = params->envelope.attack_time;
         effect.u.constant.envelope.attack_level = params->envelope.attack_level;
         effect.u.constant.envelope.fade_length = params->envelope.fade_time;
@@ -1178,8 +1180,8 @@ static NTSTATUS lnxev_device_physical_effect_update(struct unix_device *iface, B
         break;
 
     case PID_USAGE_ET_RAMP:
-        effect.u.ramp.start_level = params->ramp_force.ramp_start;
-        effect.u.ramp.end_level = params->ramp_force.ramp_end;
+        effect.u.ramp.start_level = (params->ramp_force.ramp_start * params->gain_percent) / 100;
+        effect.u.ramp.end_level = (params->ramp_force.ramp_end * params->gain_percent) / 100;
         effect.u.ramp.envelope.attack_length = params->envelope.attack_time;
         effect.u.ramp.envelope.attack_level = params->envelope.attack_level;
         effect.u.ramp.envelope.fade_length = params->envelope.fade_time;
@@ -1771,7 +1773,7 @@ NTSTATUS udev_bus_init(void *args)
         goto error;
     }
 
-#if HAVE_SYS_INOTIFY_H
+#ifdef HAVE_SYS_INOTIFY_H
     if (options.disable_udevd) monitor_fd = create_inotify();
     if (monitor_fd < 0) options.disable_udevd = FALSE;
 #else
@@ -1796,7 +1798,7 @@ NTSTATUS udev_bus_init(void *args)
     poll_count = 2;
 
     if (!options.disable_udevd) build_initial_deviceset_udevd();
-#if HAVE_SYS_INOTIFY_H
+#ifdef HAVE_SYS_INOTIFY_H
     else build_initial_deviceset_direct();
 #endif
 
@@ -1839,7 +1841,7 @@ NTSTATUS udev_bus_wait(void *args)
         if (pfd[0].revents)
         {
             if (udev_monitor) process_monitor_event(udev_monitor);
-#if HAVE_SYS_INOTIFY_H
+#ifdef HAVE_SYS_INOTIFY_H
             else process_inotify_event(pfd[0].fd);
 #endif
         }
