@@ -18,8 +18,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
-#include "config.h"
-#include "wine/port.h"
+
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
@@ -47,7 +46,11 @@ ULONG CDECL wined3d_palette_decref(struct wined3d_palette *palette)
     TRACE("%p decreasing refcount to %u.\n", palette, refcount);
 
     if (!refcount)
+    {
+        wined3d_mutex_lock();
         wined3d_cs_destroy_object(palette->device->cs, wined3d_palette_destroy_object, palette);
+        wined3d_mutex_unlock();
+    }
 
     return refcount;
 }
