@@ -20,9 +20,6 @@
 #define __WINE_WINED3D_VK_H
 
 #define VK_NO_PROTOTYPES
-#ifndef USE_WIN32_VULKAN
-#define WINE_VK_HOST
-#endif
 #include "wine/vulkan.h"
 
 #define VK_INSTANCE_FUNCS() \
@@ -49,7 +46,9 @@
     VK_INSTANCE_PFN(vkGetPhysicalDeviceSurfacePresentModesKHR) \
     VK_INSTANCE_PFN(vkGetPhysicalDeviceSurfaceSupportKHR) \
     /* VK_KHR_win32_surface */ \
-    VK_INSTANCE_PFN(vkCreateWin32SurfaceKHR)
+    VK_INSTANCE_PFN(vkCreateWin32SurfaceKHR) \
+    /* VK_EXT_host_query_reset */ \
+    VK_INSTANCE_EXT_PFN(vkResetQueryPoolEXT)
 
 #define VK_DEVICE_FUNCS() \
     VK_DEVICE_PFN(vkAllocateCommandBuffers) \
@@ -210,6 +209,7 @@ enum wined3d_vk_extension
 
     WINED3D_VK_EXT_TRANSFORM_FEEDBACK,
     WINED3D_VK_KHR_SAMPLER_MIRROR_CLAMP_TO_EDGE,
+    WINED3D_VK_EXT_HOST_QUERY_RESET,
 
     WINED3D_VK_EXT_COUNT,
 };
@@ -222,9 +222,7 @@ struct wined3d_vk_info
     unsigned int api_version;
 
     BOOL supported[WINED3D_VK_EXT_COUNT];
-#ifdef USE_WIN32_VULKAN
     HMODULE vulkan_lib;
-#endif
 };
 
 #define VK_CALL(f) (vk_info->vk_ops.f)
