@@ -34,6 +34,7 @@ static unixlib_handle_t unix_handle;
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
 DEFINE_GUID(GUID_NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+DEFINE_GUID(MEDIASUBTYPE_VC1S,MAKEFOURCC('V','C','1','S'),0x0000,0x0010,0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71);
 
 bool array_reserve(void **elements, size_t *capacity, size_t count, size_t size)
 {
@@ -68,6 +69,8 @@ struct wg_parser *wg_parser_create(enum wg_parser_type type, bool unlimited_buff
     {
         .type = type,
         .unlimited_buffering = unlimited_buffering,
+        .err_on = ERR_ON(quartz),
+        .warn_on = WARN_ON(quartz),
     };
 
     TRACE("type %#x, unlimited_buffering %d.\n", type, unlimited_buffering);
@@ -450,6 +453,7 @@ static struct class_factory mpeg_audio_codec_cf = {{&class_factory_vtbl}, mpeg_a
 static struct class_factory mpeg_splitter_cf = {{&class_factory_vtbl}, mpeg_splitter_create};
 static struct class_factory wave_parser_cf = {{&class_factory_vtbl}, wave_parser_create};
 static struct class_factory wma_decoder_cf = {{&class_factory_vtbl}, wma_decoder_create};
+static struct class_factory wmv_decoder_cf = {{&class_factory_vtbl}, wmv_decoder_create};
 static struct class_factory resampler_cf = {{&class_factory_vtbl}, resampler_create};
 static struct class_factory color_convert_cf = {{&class_factory_vtbl}, color_convert_create};
 
@@ -478,6 +482,8 @@ HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, void **out)
         factory = &wave_parser_cf;
     else if (IsEqualGUID(clsid, &CLSID_WMADecMediaObject))
         factory = &wma_decoder_cf;
+    else if (IsEqualGUID(clsid, &CLSID_WMVDecoderMFT))
+        factory = &wmv_decoder_cf;
     else if (IsEqualGUID(clsid, &CLSID_CResamplerMediaObject))
         factory = &resampler_cf;
     else if (IsEqualGUID(clsid, &CLSID_CColorConvertDMO))
@@ -693,6 +699,31 @@ HRESULT WINAPI DllRegisterServer(void)
         {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_WMAUDIO3},
         {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_WMAUDIO_LOSSLESS},
     };
+    DMO_PARTIAL_MEDIATYPE wmv_decoder_output[11] =
+    {
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YV12},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YUY2},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_UYVY},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YVYU},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_NV11},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_NV12},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB32},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB24},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB565},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB555},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB8},
+    };
+    DMO_PARTIAL_MEDIATYPE wmv_decoder_input[8] =
+    {
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WMV1},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WMV2},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WMV3},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WMVA},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WVC1},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WMVP},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_WVP2},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_VC1S},
+    };
     DMO_PARTIAL_MEDIATYPE color_convert_input[20] =
     {
         {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YV12},
@@ -762,6 +793,9 @@ HRESULT WINAPI DllRegisterServer(void)
     if (FAILED(hr = DMORegister(L"WMAudio Decoder DMO", &CLSID_WMADecMediaObject, &DMOCATEGORY_AUDIO_DECODER,
             0, ARRAY_SIZE(wma_decoder_input), wma_decoder_input, ARRAY_SIZE(wma_decoder_output), wma_decoder_output)))
         return hr;
+    if (FAILED(hr = DMORegister(L"WMVideo Decoder DMO", &CLSID_WMVDecoderMFT, &DMOCATEGORY_VIDEO_DECODER,
+            0, ARRAY_SIZE(wmv_decoder_input), wmv_decoder_input, ARRAY_SIZE(wmv_decoder_output), wmv_decoder_output)))
+        return hr;
     if (FAILED(hr = DMORegister(L"Resampler DMO", &CLSID_CResamplerMediaObject, &DMOCATEGORY_AUDIO_EFFECT,
             0, ARRAY_SIZE(audio_convert_types), audio_convert_types, ARRAY_SIZE(audio_convert_types), audio_convert_types)))
         return hr;
@@ -800,6 +834,8 @@ HRESULT WINAPI DllUnregisterServer(void)
         return hr;
     if (FAILED(hr = DMOUnregister(&CLSID_WMADecMediaObject, &DMOCATEGORY_AUDIO_DECODER)))
         return hr;
+    if (FAILED(hr = DMOUnregister(&CLSID_WMVDecoderMFT, &DMOCATEGORY_VIDEO_DECODER)))
+        return hr;
 
     return S_OK;
 }
