@@ -363,6 +363,10 @@ static void start_pipeline(struct media_source *source, struct source_async_comm
             IMFMediaType_Release(current_mt);
             IMFMediaTypeHandler_Release(mth);
         }
+        else
+        {
+            wg_parser_stream_disable(stream->wg_stream);
+        }
 
         if (position->vt != VT_EMPTY)
             stream->eos = FALSE;
@@ -419,10 +423,7 @@ static void stop_pipeline(struct media_source *source)
     {
         struct media_stream *stream = source->streams[i];
         if (stream->state != STREAM_INACTIVE)
-        {
             IMFMediaEventQueue_QueueEventParamVar(stream->event_queue, MEStreamStopped, &GUID_NULL, S_OK, NULL);
-            wg_parser_stream_disable(stream->wg_stream);
-        }
     }
 
     IMFMediaEventQueue_QueueEventParamVar(source->event_queue, MESourceStopped, &GUID_NULL, S_OK, NULL);
@@ -878,6 +879,12 @@ static HRESULT media_stream_init_desc(struct media_stream *stream)
         IMFMediaType *base_type = mf_media_type_from_wg_format(&format);
         GUID base_subtype;
 
+        if (!base_type)
+        {
+            hr = MF_E_INVALIDMEDIATYPE;
+            goto done;
+        }
+
         IMFMediaType_GetGUID(base_type, &MF_MT_SUBTYPE, &base_subtype);
 
         stream_types[0] = base_type;
@@ -910,8 +917,8 @@ static HRESULT media_stream_init_desc(struct media_stream *stream)
             WG_AUDIO_FORMAT_F32LE,
         };
 
-        stream_types[0] = mf_media_type_from_wg_format(&format);
-        type_count = 1;
+        if ((stream_types[0] = mf_media_type_from_wg_format(&format)))
+            type_count = 1;
 
         for (i = 0; i < ARRAY_SIZE(audio_types); i++)
         {
@@ -920,7 +927,8 @@ static HRESULT media_stream_init_desc(struct media_stream *stream)
                 continue;
             new_format = format;
             new_format.u.audio.format = audio_types[i];
-            stream_types[type_count++] = mf_media_type_from_wg_format(&new_format);
+            if ((stream_types[type_count] = mf_media_type_from_wg_format(&new_format)))
+                type_count++;
         }
     }
     else
