@@ -20,6 +20,50 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(wmvcore);
 
+struct wm_stream
+{
+    struct wm_reader *reader;
+    struct wg_parser_stream *wg_stream;
+    struct wg_format format;
+    WMT_STREAM_SELECTION selection;
+    WORD index;
+    bool eos;
+    /* Note that we only pretend to read compressed samples, and instead output
+     * uncompressed samples regardless of whether we are configured to read
+     * compressed samples. Rather, the behaviour of the reader objects differs
+     * in nontrivial ways depending on this field. */
+    bool read_compressed;
+
+    IWMReaderAllocatorEx *output_allocator;
+    IWMReaderAllocatorEx *stream_allocator;
+};
+
+struct wm_reader
+{
+    IUnknown IUnknown_inner;
+    IWMSyncReader2 IWMSyncReader2_iface;
+    IWMHeaderInfo3 IWMHeaderInfo3_iface;
+    IWMLanguageList IWMLanguageList_iface;
+    IWMPacketSize2 IWMPacketSize2_iface;
+    IWMProfile3 IWMProfile3_iface;
+    IWMReaderPlaylistBurn IWMReaderPlaylistBurn_iface;
+    IWMReaderTimecode IWMReaderTimecode_iface;
+    IUnknown *outer;
+    LONG refcount;
+
+    CRITICAL_SECTION cs;
+    QWORD start_time;
+
+    IStream *source_stream;
+    HANDLE file;
+    HANDLE read_thread;
+    bool read_thread_shutdown;
+    struct wg_parser *wg_parser;
+
+    struct wm_stream *streams;
+    WORD stream_count;
+};
+
 static struct wm_stream *get_stream_by_output_number(struct wm_reader *reader, DWORD output)
 {
     if (output < reader->stream_count)
@@ -1487,23 +1531,6 @@ static struct wm_stream *wm_reader_get_stream_by_stream_number(struct wm_reader
     return NULL;
 }
 
-HRESULT wm_reader_get_output_props(struct wm_reader *reader, DWORD output, IWMOutputMediaProps **props)
-{
-    struct wm_stream *stream;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = get_stream_by_output_number(reader, output)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    *props = output_props_create(&stream->format);
-    LeaveCriticalSection(&reader->cs);
-    return *props ? S_OK : E_OUTOFMEMORY;
-}
-
 static const enum wg_video_format video_formats[] =
 {
     /* Try to prefer YUV formats over RGB ones. Most decoders output in the
@@ -1520,195 +1547,26 @@ static const enum wg_video_format video_formats[] =
     WG_VIDEO_FORMAT_RGB15,
 };
 
-HRESULT wm_reader_get_output_format_count(struct wm_reader *reader, DWORD output, DWORD *count)
-{
-    struct wm_stream *stream;
-    struct wg_format format;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = get_stream_by_output_number(reader, output)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    wg_parser_stream_get_preferred_format(stream->wg_stream, &format);
-    switch (format.major_type)
-    {
-        case WG_MAJOR_TYPE_VIDEO:
-            *count = ARRAY_SIZE(video_formats);
-            break;
-
-        case WG_MAJOR_TYPE_MPEG1_AUDIO:
-        case WG_MAJOR_TYPE_WMA:
-        case WG_MAJOR_TYPE_H264:
-            FIXME("Format %u not implemented!\n", format.major_type);
-            /* fallthrough */
-        case WG_MAJOR_TYPE_AUDIO:
-        case WG_MAJOR_TYPE_UNKNOWN:
-            *count = 1;
-            break;
-    }
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
-HRESULT wm_reader_get_output_format(struct wm_reader *reader, DWORD output,
-        DWORD index, IWMOutputMediaProps **props)
-{
-    struct wm_stream *stream;
-    struct wg_format format;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = get_stream_by_output_number(reader, output)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    wg_parser_stream_get_preferred_format(stream->wg_stream, &format);
-
-    switch (format.major_type)
-    {
-        case WG_MAJOR_TYPE_VIDEO:
-            if (index >= ARRAY_SIZE(video_formats))
-            {
-                LeaveCriticalSection(&reader->cs);
-                return NS_E_INVALID_OUTPUT_FORMAT;
-            }
-            format.u.video.format = video_formats[index];
-            break;
-
-        case WG_MAJOR_TYPE_AUDIO:
-            if (index)
-            {
-                LeaveCriticalSection(&reader->cs);
-                return NS_E_INVALID_OUTPUT_FORMAT;
-            }
-            format.u.audio.format = WG_AUDIO_FORMAT_S16LE;
-            break;
-
-        case WG_MAJOR_TYPE_MPEG1_AUDIO:
-        case WG_MAJOR_TYPE_WMA:
-        case WG_MAJOR_TYPE_H264:
-            FIXME("Format %u not implemented!\n", format.major_type);
-            break;
-        case WG_MAJOR_TYPE_UNKNOWN:
-            break;
-    }
-
-    LeaveCriticalSection(&reader->cs);
-
-    *props = output_props_create(&format);
-    return *props ? S_OK : E_OUTOFMEMORY;
-}
-
-HRESULT wm_reader_set_output_props(struct wm_reader *reader, DWORD output,
-        IWMOutputMediaProps *props_iface)
-{
-    struct output_props *props = unsafe_impl_from_IWMOutputMediaProps(props_iface);
-    struct wg_format format, pref_format;
-    struct wm_stream *stream;
-    HRESULT hr = S_OK;
-    int i;
-
-    strmbase_dump_media_type(&props->mt);
-
-    if (!amt_to_wg_format(&props->mt, &format))
-    {
-        ERR("Failed to convert media type to winegstreamer format.\n");
-        return E_FAIL;
-    }
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = get_stream_by_output_number(reader, output)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    wg_parser_stream_get_preferred_format(stream->wg_stream, &pref_format);
-    if (pref_format.major_type != format.major_type)
-    {
-        /* R.U.S.E sets the type of the wrong stream, apparently by accident. */
-        hr = NS_E_INCOMPATIBLE_FORMAT;
-    }
-    else switch (pref_format.major_type)
-    {
-        case WG_MAJOR_TYPE_AUDIO:
-            if (format.u.audio.format == WG_AUDIO_FORMAT_UNKNOWN)
-                hr = NS_E_AUDIO_CODEC_NOT_INSTALLED;
-            else if (format.u.audio.channels > pref_format.u.audio.channels)
-                hr = NS_E_AUDIO_CODEC_NOT_INSTALLED;
-            break;
-
-        case WG_MAJOR_TYPE_VIDEO:
-            for (i = 0; i < ARRAY_SIZE(video_formats); ++i)
-                if (format.u.video.format == video_formats[i])
-                    break;
-            if (i == ARRAY_SIZE(video_formats))
-                hr = NS_E_INVALID_OUTPUT_FORMAT;
-            else if (pref_format.u.video.width != format.u.video.width)
-                hr = NS_E_INVALID_OUTPUT_FORMAT;
-            else if (pref_format.u.video.height != format.u.video.height)
-                hr = NS_E_INVALID_OUTPUT_FORMAT;
-            break;
-
-        default:
-            hr = NS_E_INCOMPATIBLE_FORMAT;
-            break;
-    }
-
-    if (FAILED(hr))
-    {
-        WARN("Unsupported media type, returning %#lx.\n", hr);
-        LeaveCriticalSection(&reader->cs);
-        return hr;
-    }
-
-    stream->format = format;
-    wg_parser_stream_enable(stream->wg_stream, &format);
-
-    /* Re-decode any buffers that might have been generated with the old format.
-     *
-     * FIXME: Seeking in-place will cause some buffers to be dropped.
-     * Unfortunately, we can't really store the last received PTS and seek there
-     * either: since seeks are inexact and we aren't guaranteed to receive
-     * samples in order, some buffers might be duplicated or dropped anyway.
-     * In order to really seamlessly allow for format changes, we need
-     * cooperation from each individual GStreamer stream, to be able to tell
-     * upstream exactly which buffers they need resent...
-     *
-     * In all likelihood this function is being called not mid-stream but rather
-     * while setting the stream up, before consuming any events. Accordingly
-     * let's just seek back to the beginning. */
-    wg_parser_stream_seek(reader->streams[0].wg_stream, 1.0, reader->start_time, 0,
-            AM_SEEKING_AbsolutePositioning, AM_SEEKING_NoPositioning);
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
 static const char *get_major_type_string(enum wg_major_type type)
 {
     switch (type)
     {
         case WG_MAJOR_TYPE_AUDIO:
             return "audio";
+        case WG_MAJOR_TYPE_AUDIO_MPEG1:
+            return "mpeg1-audio";
+        case WG_MAJOR_TYPE_AUDIO_MPEG4:
+            return "mpeg4-audio";
+        case WG_MAJOR_TYPE_AUDIO_WMA:
+            return "wma";
         case WG_MAJOR_TYPE_VIDEO:
             return "video";
+        case WG_MAJOR_TYPE_VIDEO_CINEPAK:
+            return "cinepak";
+        case WG_MAJOR_TYPE_VIDEO_H264:
+            return "h264";
         case WG_MAJOR_TYPE_UNKNOWN:
             return "unknown";
-        case WG_MAJOR_TYPE_MPEG1_AUDIO:
-            return "mpeg1-audio";
-        case WG_MAJOR_TYPE_WMA:
-            return "wma";
-        case WG_MAJOR_TYPE_H264:
-            return "h264";
     }
     assert(0);
     return NULL;
@@ -1757,15 +1615,16 @@ static WORD get_earliest_buffer(struct wm_reader *reader, struct wg_parser_buffe
     return stream_number;
 }
 
-HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, IWMReaderCallbackAdvanced *callback_advanced, WORD stream_number,
+static HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, WORD stream_number,
         INSSBuffer **ret_sample, QWORD *pts, QWORD *duration, DWORD *flags, WORD *ret_stream_number)
 {
     struct wg_parser_stream *wg_stream;
     struct wg_parser_buffer wg_buffer;
     struct wm_stream *stream;
+    struct buffer *object;
     DWORD size, capacity;
     INSSBuffer *sample;
-    HRESULT hr;
+    HRESULT hr = S_OK;
     BYTE *data;
 
     for (;;)
@@ -1809,37 +1668,17 @@ HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, IWMReaderCallbackA
 
         TRACE("Got buffer for '%s' stream %p.\n", get_major_type_string(stream->format.major_type), stream);
 
-        if (callback_advanced && stream->read_compressed && stream->allocate_stream)
-        {
-            if (FAILED(hr = IWMReaderCallbackAdvanced_AllocateForStream(callback_advanced,
-                    stream->index + 1, wg_buffer.size, &sample, NULL)))
-            {
-                ERR("Failed to allocate stream sample of %u bytes, hr %#lx.\n", wg_buffer.size, hr);
-                wg_parser_stream_release_buffer(wg_stream);
-                return hr;
-            }
-        }
-        else if (callback_advanced && !stream->read_compressed && stream->allocate_output)
-        {
-            if (FAILED(hr = IWMReaderCallbackAdvanced_AllocateForOutput(callback_advanced,
-                    stream->index, wg_buffer.size, &sample, NULL)))
-            {
-                ERR("Failed to allocate output sample of %u bytes, hr %#lx.\n", wg_buffer.size, hr);
-                wg_parser_stream_release_buffer(wg_stream);
-                return hr;
-            }
-        }
+        if (!stream->read_compressed && stream->output_allocator)
+            hr = IWMReaderAllocatorEx_AllocateForOutputEx(stream->output_allocator, stream->index,
+                    wg_buffer.size, &sample, 0, 0, 0, NULL);
+        else if (stream->read_compressed && stream->stream_allocator)
+            hr = IWMReaderAllocatorEx_AllocateForStreamEx(stream->stream_allocator, stream->index + 1,
+                    wg_buffer.size, &sample, 0, 0, 0, NULL);
+        /* FIXME: Should these be pooled? */
+        else if (!(object = calloc(1, offsetof(struct buffer, data[wg_buffer.size]))))
+            hr = E_OUTOFMEMORY;
         else
         {
-            struct buffer *object;
-
-            /* FIXME: Should these be pooled? */
-            if (!(object = calloc(1, offsetof(struct buffer, data[wg_buffer.size]))))
-            {
-                wg_parser_stream_release_buffer(wg_stream);
-                return E_OUTOFMEMORY;
-            }
-
             object->INSSBuffer_iface.lpVtbl = &buffer_vtbl;
             object->refcount = 1;
             object->capacity = wg_buffer.size;
@@ -1848,6 +1687,13 @@ HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, IWMReaderCallbackA
             sample = &object->INSSBuffer_iface;
         }
 
+        if (FAILED(hr))
+        {
+            ERR("Failed to allocate sample of %u bytes, hr %#lx.\n", wg_buffer.size, hr);
+            wg_parser_stream_release_buffer(wg_stream);
+            return hr;
+        }
+
         if (FAILED(hr = INSSBuffer_GetBufferAndLength(sample, &data, &size)))
             ERR("Failed to get data pointer, hr %#lx.\n", hr);
         if (FAILED(hr = INSSBuffer_GetMaxLength(sample, &capacity)))
@@ -1886,123 +1732,6 @@ HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, IWMReaderCallbackA
     }
 }
 
-HRESULT wm_reader_set_streams_selected(struct wm_reader *reader, WORD count,
-        const WORD *stream_numbers, const WMT_STREAM_SELECTION *selections)
-{
-    struct wm_stream *stream;
-    WORD i;
-
-    if (!count)
-        return E_INVALIDARG;
-
-    EnterCriticalSection(&reader->cs);
-
-    for (i = 0; i < count; ++i)
-    {
-        if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_numbers[i])))
-        {
-            LeaveCriticalSection(&reader->cs);
-            WARN("Invalid stream number %u; returning NS_E_INVALID_REQUEST.\n", stream_numbers[i]);
-            return NS_E_INVALID_REQUEST;
-        }
-    }
-
-    for (i = 0; i < count; ++i)
-    {
-        stream = wm_reader_get_stream_by_stream_number(reader, stream_numbers[i]);
-        stream->selection = selections[i];
-        if (selections[i] == WMT_OFF)
-        {
-            TRACE("Disabling stream %u.\n", stream_numbers[i]);
-            wg_parser_stream_disable(stream->wg_stream);
-        }
-        else if (selections[i] == WMT_ON)
-        {
-            if (selections[i] != WMT_ON)
-                FIXME("Ignoring selection %#x for stream %u; treating as enabled.\n",
-                        selections[i], stream_numbers[i]);
-            TRACE("Enabling stream %u.\n", stream_numbers[i]);
-            wg_parser_stream_enable(stream->wg_stream, &stream->format);
-        }
-    }
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
-HRESULT wm_reader_get_stream_selection(struct wm_reader *reader,
-        WORD stream_number, WMT_STREAM_SELECTION *selection)
-{
-    struct wm_stream *stream;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    *selection = stream->selection;
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
-HRESULT wm_reader_set_allocate_for_output(struct wm_reader *reader, DWORD output, BOOL allocate)
-{
-    struct wm_stream *stream;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = get_stream_by_output_number(reader, output)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    stream->allocate_output = !!allocate;
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
-HRESULT wm_reader_set_allocate_for_stream(struct wm_reader *reader, WORD stream_number, BOOL allocate)
-{
-    struct wm_stream *stream;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    stream->allocate_stream = !!allocate;
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
-HRESULT wm_reader_get_max_stream_size(struct wm_reader *reader, WORD stream_number, DWORD *size)
-{
-    struct wm_stream *stream;
-
-    EnterCriticalSection(&reader->cs);
-
-    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
-    {
-        LeaveCriticalSection(&reader->cs);
-        return E_INVALIDARG;
-    }
-
-    *size = wg_format_get_max_size(&stream->format);
-
-    LeaveCriticalSection(&reader->cs);
-    return S_OK;
-}
-
 static struct wm_reader *impl_from_IUnknown(IUnknown *iface)
 {
     return CONTAINING_RECORD(iface, struct wm_reader, IUnknown_inner);
@@ -2145,11 +1874,25 @@ static HRESULT WINAPI reader_GetMaxOutputSampleSize(IWMSyncReader2 *iface, DWORD
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI reader_GetMaxStreamSampleSize(IWMSyncReader2 *iface, WORD stream, DWORD *max)
+static HRESULT WINAPI reader_GetMaxStreamSampleSize(IWMSyncReader2 *iface, WORD stream_number, DWORD *size)
 {
-    struct wm_reader *This = impl_from_IWMSyncReader2(iface);
-    FIXME("(%p)->(%d %p): stub!\n", This, stream, max);
-    return E_NOTIMPL;
+    struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+
+    TRACE("reader %p, stream_number %u, size %p.\n", reader, stream_number, size);
+
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    *size = wg_format_get_max_size(&stream->format);
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
 static HRESULT WINAPI reader_GetNextSample(IWMSyncReader2 *iface,
@@ -2168,7 +1911,7 @@ static HRESULT WINAPI reader_GetNextSample(IWMSyncReader2 *iface,
 
     EnterCriticalSection(&reader->cs);
 
-    hr = wm_reader_get_stream_sample(reader, NULL, stream_number, sample, pts, duration, flags, &stream_number);
+    hr = wm_reader_get_stream_sample(reader, stream_number, sample, pts, duration, flags, &stream_number);
     if (output_number && hr == S_OK)
         *output_number = stream_number - 1;
     if (ret_stream_number && (hr == S_OK || stream_number))
@@ -2194,19 +1937,96 @@ static HRESULT WINAPI reader_GetOutputFormat(IWMSyncReader2 *iface,
         DWORD output, DWORD index, IWMOutputMediaProps **props)
 {
     struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+    struct wg_format format;
 
     TRACE("reader %p, output %lu, index %lu, props %p.\n", reader, output, index, props);
 
-    return wm_reader_get_output_format(reader, output, index, props);
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = get_stream_by_output_number(reader, output)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    wg_parser_stream_get_preferred_format(stream->wg_stream, &format);
+
+    switch (format.major_type)
+    {
+        case WG_MAJOR_TYPE_VIDEO:
+            if (index >= ARRAY_SIZE(video_formats))
+            {
+                LeaveCriticalSection(&reader->cs);
+                return NS_E_INVALID_OUTPUT_FORMAT;
+            }
+            format.u.video.format = video_formats[index];
+            break;
+
+        case WG_MAJOR_TYPE_AUDIO:
+            if (index)
+            {
+                LeaveCriticalSection(&reader->cs);
+                return NS_E_INVALID_OUTPUT_FORMAT;
+            }
+            format.u.audio.format = WG_AUDIO_FORMAT_S16LE;
+            break;
+
+        case WG_MAJOR_TYPE_AUDIO_MPEG1:
+        case WG_MAJOR_TYPE_AUDIO_MPEG4:
+        case WG_MAJOR_TYPE_AUDIO_WMA:
+        case WG_MAJOR_TYPE_VIDEO_CINEPAK:
+        case WG_MAJOR_TYPE_VIDEO_H264:
+            FIXME("Format %u not implemented!\n", format.major_type);
+            break;
+        case WG_MAJOR_TYPE_UNKNOWN:
+            break;
+    }
+
+    LeaveCriticalSection(&reader->cs);
+
+    *props = output_props_create(&format);
+    return *props ? S_OK : E_OUTOFMEMORY;
 }
 
 static HRESULT WINAPI reader_GetOutputFormatCount(IWMSyncReader2 *iface, DWORD output, DWORD *count)
 {
     struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+    struct wg_format format;
 
     TRACE("reader %p, output %lu, count %p.\n", reader, output, count);
 
-    return wm_reader_get_output_format_count(reader, output, count);
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = get_stream_by_output_number(reader, output)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    wg_parser_stream_get_preferred_format(stream->wg_stream, &format);
+    switch (format.major_type)
+    {
+        case WG_MAJOR_TYPE_VIDEO:
+            *count = ARRAY_SIZE(video_formats);
+            break;
+
+        case WG_MAJOR_TYPE_AUDIO_MPEG1:
+        case WG_MAJOR_TYPE_AUDIO_MPEG4:
+        case WG_MAJOR_TYPE_AUDIO_WMA:
+        case WG_MAJOR_TYPE_VIDEO_CINEPAK:
+        case WG_MAJOR_TYPE_VIDEO_H264:
+            FIXME("Format %u not implemented!\n", format.major_type);
+            /* fallthrough */
+        case WG_MAJOR_TYPE_AUDIO:
+        case WG_MAJOR_TYPE_UNKNOWN:
+            *count = 1;
+            break;
+    }
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
 static HRESULT WINAPI reader_GetOutputNumberForStream(IWMSyncReader2 *iface,
@@ -2224,10 +2044,21 @@ static HRESULT WINAPI reader_GetOutputProps(IWMSyncReader2 *iface,
         DWORD output, IWMOutputMediaProps **props)
 {
     struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
 
     TRACE("reader %p, output %lu, props %p.\n", reader, output, props);
 
-    return wm_reader_get_output_props(reader, output, props);
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = get_stream_by_output_number(reader, output)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    *props = output_props_create(&stream->format);
+    LeaveCriticalSection(&reader->cs);
+    return *props ? S_OK : E_OUTOFMEMORY;
 }
 
 static HRESULT WINAPI reader_GetOutputSetting(IWMSyncReader2 *iface, DWORD output_num, const WCHAR *name,
@@ -2274,10 +2105,22 @@ static HRESULT WINAPI reader_GetStreamSelected(IWMSyncReader2 *iface,
         WORD stream_number, WMT_STREAM_SELECTION *selection)
 {
     struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
 
     TRACE("reader %p, stream_number %u, selection %p.\n", reader, stream_number, selection);
 
-    return wm_reader_get_stream_selection(reader, stream_number, selection);
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    *selection = stream->selection;
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
 static HRESULT WINAPI reader_Open(IWMSyncReader2 *iface, const WCHAR *filename)
@@ -2356,13 +2199,93 @@ static HRESULT WINAPI reader_OpenStream(IWMSyncReader2 *iface, IStream *stream)
     return hr;
 }
 
-static HRESULT WINAPI reader_SetOutputProps(IWMSyncReader2 *iface, DWORD output, IWMOutputMediaProps *props)
+static HRESULT WINAPI reader_SetOutputProps(IWMSyncReader2 *iface, DWORD output, IWMOutputMediaProps *props_iface)
 {
     struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct output_props *props = unsafe_impl_from_IWMOutputMediaProps(props_iface);
+    struct wg_format format, pref_format;
+    struct wm_stream *stream;
+    HRESULT hr = S_OK;
+    int i;
 
-    TRACE("reader %p, output %lu, props %p.\n", reader, output, props);
+    TRACE("reader %p, output %lu, props_iface %p.\n", reader, output, props_iface);
 
-    return wm_reader_set_output_props(reader, output, props);
+    strmbase_dump_media_type(&props->mt);
+
+    if (!amt_to_wg_format(&props->mt, &format))
+    {
+        ERR("Failed to convert media type to winegstreamer format.\n");
+        return E_FAIL;
+    }
+
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = get_stream_by_output_number(reader, output)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    wg_parser_stream_get_preferred_format(stream->wg_stream, &pref_format);
+    if (pref_format.major_type != format.major_type)
+    {
+        /* R.U.S.E sets the type of the wrong stream, apparently by accident. */
+        hr = NS_E_INCOMPATIBLE_FORMAT;
+    }
+    else switch (pref_format.major_type)
+    {
+        case WG_MAJOR_TYPE_AUDIO:
+            if (format.u.audio.format == WG_AUDIO_FORMAT_UNKNOWN)
+                hr = NS_E_AUDIO_CODEC_NOT_INSTALLED;
+            else if (format.u.audio.channels > pref_format.u.audio.channels)
+                hr = NS_E_AUDIO_CODEC_NOT_INSTALLED;
+            break;
+
+        case WG_MAJOR_TYPE_VIDEO:
+            for (i = 0; i < ARRAY_SIZE(video_formats); ++i)
+                if (format.u.video.format == video_formats[i])
+                    break;
+            if (i == ARRAY_SIZE(video_formats))
+                hr = NS_E_INVALID_OUTPUT_FORMAT;
+            else if (pref_format.u.video.width != format.u.video.width)
+                hr = NS_E_INVALID_OUTPUT_FORMAT;
+            else if (pref_format.u.video.height != format.u.video.height)
+                hr = NS_E_INVALID_OUTPUT_FORMAT;
+            break;
+
+        default:
+            hr = NS_E_INCOMPATIBLE_FORMAT;
+            break;
+    }
+
+    if (FAILED(hr))
+    {
+        WARN("Unsupported media type, returning %#lx.\n", hr);
+        LeaveCriticalSection(&reader->cs);
+        return hr;
+    }
+
+    stream->format = format;
+    wg_parser_stream_enable(stream->wg_stream, &format);
+
+    /* Re-decode any buffers that might have been generated with the old format.
+     *
+     * FIXME: Seeking in-place will cause some buffers to be dropped.
+     * Unfortunately, we can't really store the last received PTS and seek there
+     * either: since seeks are inexact and we aren't guaranteed to receive
+     * samples in order, some buffers might be duplicated or dropped anyway.
+     * In order to really seamlessly allow for format changes, we need
+     * cooperation from each individual GStreamer stream, to be able to tell
+     * upstream exactly which buffers they need resent...
+     *
+     * In all likelihood this function is being called not mid-stream but rather
+     * while setting the stream up, before consuming any events. Accordingly
+     * let's just seek back to the beginning. */
+    wg_parser_stream_seek(reader->streams[0].wg_stream, 1.0, reader->start_time, 0,
+            AM_SEEKING_AbsolutePositioning, AM_SEEKING_NoPositioning);
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
 static HRESULT WINAPI reader_SetOutputSetting(IWMSyncReader2 *iface, DWORD output,
@@ -2449,11 +2372,48 @@ static HRESULT WINAPI reader_SetStreamsSelected(IWMSyncReader2 *iface,
         WORD count, WORD *stream_numbers, WMT_STREAM_SELECTION *selections)
 {
     struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+    WORD i;
 
     TRACE("reader %p, count %u, stream_numbers %p, selections %p.\n",
             reader, count, stream_numbers, selections);
 
-    return wm_reader_set_streams_selected(reader, count, stream_numbers, selections);
+    if (!count)
+        return E_INVALIDARG;
+
+    EnterCriticalSection(&reader->cs);
+
+    for (i = 0; i < count; ++i)
+    {
+        if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_numbers[i])))
+        {
+            LeaveCriticalSection(&reader->cs);
+            WARN("Invalid stream number %u; returning NS_E_INVALID_REQUEST.\n", stream_numbers[i]);
+            return NS_E_INVALID_REQUEST;
+        }
+    }
+
+    for (i = 0; i < count; ++i)
+    {
+        stream = wm_reader_get_stream_by_stream_number(reader, stream_numbers[i]);
+        stream->selection = selections[i];
+        if (selections[i] == WMT_OFF)
+        {
+            TRACE("Disabling stream %u.\n", stream_numbers[i]);
+            wg_parser_stream_disable(stream->wg_stream);
+        }
+        else if (selections[i] == WMT_ON)
+        {
+            if (selections[i] != WMT_ON)
+                FIXME("Ignoring selection %#x for stream %u; treating as enabled.\n",
+                        selections[i], stream_numbers[i]);
+            TRACE("Enabling stream %u.\n", stream_numbers[i]);
+            wg_parser_stream_enable(stream->wg_stream, &stream->format);
+        }
+    }
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
 static HRESULT WINAPI reader_SetRangeByTimecode(IWMSyncReader2 *iface, WORD stream_num,
@@ -2473,32 +2433,103 @@ static HRESULT WINAPI reader_SetRangeByFrameEx(IWMSyncReader2 *iface, WORD strea
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI reader_SetAllocateForOutput(IWMSyncReader2 *iface, DWORD output_num, IWMReaderAllocatorEx *allocator)
+static HRESULT WINAPI reader_SetAllocateForOutput(IWMSyncReader2 *iface, DWORD output, IWMReaderAllocatorEx *allocator)
 {
-    struct wm_reader *This = impl_from_IWMSyncReader2(iface);
-    FIXME("(%p)->(%lu %p): stub!\n", This, output_num, allocator);
-    return E_NOTIMPL;
+    struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+
+    TRACE("reader %p, output %lu, allocator %p.\n", reader, output, allocator);
+
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = get_stream_by_output_number(reader, output)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    if (stream->output_allocator)
+        IWMReaderAllocatorEx_Release(stream->output_allocator);
+    if ((stream->output_allocator = allocator))
+        IWMReaderAllocatorEx_AddRef(stream->output_allocator);
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
-static HRESULT WINAPI reader_GetAllocateForOutput(IWMSyncReader2 *iface, DWORD output_num, IWMReaderAllocatorEx **allocator)
+static HRESULT WINAPI reader_GetAllocateForOutput(IWMSyncReader2 *iface, DWORD output, IWMReaderAllocatorEx **allocator)
 {
-    struct wm_reader *This = impl_from_IWMSyncReader2(iface);
-    FIXME("(%p)->(%lu %p): stub!\n", This, output_num, allocator);
-    return E_NOTIMPL;
+    struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+
+    TRACE("reader %p, output %lu, allocator %p.\n", reader, output, allocator);
+
+    if (!allocator)
+        return E_INVALIDARG;
+
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = get_stream_by_output_number(reader, output)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    stream = reader->streams + output;
+    if ((*allocator = stream->output_allocator))
+        IWMReaderAllocatorEx_AddRef(*allocator);
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
-static HRESULT WINAPI reader_SetAllocateForStream(IWMSyncReader2 *iface, DWORD stream_num, IWMReaderAllocatorEx *allocator)
+static HRESULT WINAPI reader_SetAllocateForStream(IWMSyncReader2 *iface, DWORD stream_number, IWMReaderAllocatorEx *allocator)
 {
-    struct wm_reader *This = impl_from_IWMSyncReader2(iface);
-    FIXME("(%p)->(%lu %p): stub!\n", This, stream_num, allocator);
-    return E_NOTIMPL;
+    struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+
+    TRACE("reader %p, stream_number %lu, allocator %p.\n", reader, stream_number, allocator);
+
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    if (stream->stream_allocator)
+        IWMReaderAllocatorEx_Release(stream->stream_allocator);
+    if ((stream->stream_allocator = allocator))
+        IWMReaderAllocatorEx_AddRef(stream->stream_allocator);
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
-static HRESULT WINAPI reader_GetAllocateForStream(IWMSyncReader2 *iface, DWORD stream_num, IWMReaderAllocatorEx **allocator)
+static HRESULT WINAPI reader_GetAllocateForStream(IWMSyncReader2 *iface, DWORD stream_number, IWMReaderAllocatorEx **allocator)
 {
-    struct wm_reader *This = impl_from_IWMSyncReader2(iface);
-    FIXME("(%p)->(%lu %p): stub!\n", This, stream_num, allocator);
-    return E_NOTIMPL;
+    struct wm_reader *reader = impl_from_IWMSyncReader2(iface);
+    struct wm_stream *stream;
+
+    TRACE("reader %p, stream_number %lu, allocator %p.\n", reader, stream_number, allocator);
+
+    if (!allocator)
+        return E_INVALIDARG;
+
+    EnterCriticalSection(&reader->cs);
+
+    if (!(stream = wm_reader_get_stream_by_stream_number(reader, stream_number)))
+    {
+        LeaveCriticalSection(&reader->cs);
+        return E_INVALIDARG;
+    }
+
+    if ((*allocator = stream->stream_allocator))
+        IWMReaderAllocatorEx_AddRef(*allocator);
+
+    LeaveCriticalSection(&reader->cs);
+    return S_OK;
 }
 
 static const IWMSyncReader2Vtbl reader_vtbl =
@@ -2535,11 +2566,6 @@ static const IWMSyncReader2Vtbl reader_vtbl =
     reader_GetAllocateForStream
 };
 
-struct wm_reader *wm_reader_from_sync_reader_inner(IUnknown *iface)
-{
-    return impl_from_IUnknown(iface);
-}
-
 HRESULT WINAPI winegstreamer_create_wm_sync_reader(IUnknown *outer, void **out)
 {
     struct wm_reader *object;
