@@ -162,8 +162,8 @@ static void macdrv_im_set_text(const macdrv_event *event)
 
     size = offsetof(struct ime_set_text_params, text[length]);
     if (!(params = malloc(size))) return;
-    params->hwnd = hwnd;
-    params->data = event->im_set_text.data;
+    params->hwnd = HandleToUlong(hwnd);
+    params->data = (UINT_PTR)event->im_set_text.data;
     params->cursor_pos = event->im_set_text.cursor_pos;
     params->complete = event->im_set_text.complete;
 
@@ -229,7 +229,7 @@ static BOOL query_drag_drop(macdrv_query *query)
         return FALSE;
     }
 
-    params.hwnd = hwnd;
+    params.hwnd = HandleToUlong(hwnd);
     params.effect = drag_operations_to_dropeffects(query->drag_drop.op);
     params.x = query->drag_drop.x + data->whole_rect.left;
     params.y = query->drag_drop.y + data->whole_rect.top;
@@ -244,7 +244,7 @@ static BOOL query_drag_drop(macdrv_query *query)
 static BOOL query_drag_exited(macdrv_query *query)
 {
     struct dnd_query_exited_params params;
-    params.hwnd = macdrv_get_window_hwnd(query->window);
+    params.hwnd = HandleToUlong(macdrv_get_window_hwnd(query->window));
     return macdrv_client_func(client_func_dnd_query_exited, &params, sizeof(params));
 }
 
@@ -265,7 +265,7 @@ static BOOL query_drag_operation(macdrv_query *query)
         return FALSE;
     }
 
-    params.hwnd = hwnd;
+    params.hwnd = HandleToUlong(hwnd);
     params.effect = drag_operations_to_dropeffects(query->drag_operation.offered_ops);
     params.x = query->drag_operation.x + data->whole_rect.left;
     params.y = query->drag_operation.y + data->whole_rect.top;
@@ -297,9 +297,9 @@ BOOL query_ime_char_rect(macdrv_query* query)
     TRACE_(imm)("win %p/%p himc %p range %ld-%ld\n", hwnd, query->window, himc, range->location,
                 range->length);
 
-    params.hwnd = hwnd;
-    params.data = himc;
-    params.result = &result;
+    params.hwnd = HandleToUlong(hwnd);
+    params.data = (UINT_PTR)himc;
+    params.result = (UINT_PTR)&result;
     params.location = range->location;
     params.length = range->length;
     ret = macdrv_client_func(client_func_ime_query_char_rect, &params, sizeof(params));
@@ -525,8 +525,8 @@ NTSTATUS macdrv_MsgWaitForMultipleObjectsEx(DWORD count, const HANDLE *handles,
     if (!data)
     {
         if (!count && timeout && !timeout->QuadPart) return WAIT_TIMEOUT;
-        return pNtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
-                                          !!(flags & MWMO_ALERTABLE), timeout );
+        return NtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
+                                         !!(flags & MWMO_ALERTABLE), timeout );
     }
 
     if (data->current_event && data->current_event->type != QUERY_EVENT &&
@@ -538,8 +538,8 @@ NTSTATUS macdrv_MsgWaitForMultipleObjectsEx(DWORD count, const HANDLE *handles,
     if (process_events(data->queue, event_mask)) ret = count - 1;
     else if (count || !timeout || timeout->QuadPart)
     {
-        ret = pNtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
-                                         !!(flags & MWMO_ALERTABLE), timeout );
+        ret = NtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
+                                        !!(flags & MWMO_ALERTABLE), timeout );
         if (ret == count - 1) process_events(data->queue, event_mask);
     }
     else ret = WAIT_TIMEOUT;
