@@ -1570,6 +1570,26 @@ BOOL macdrv_CreateDesktopWindow(HWND hwnd)
     return TRUE;
 }
 
+void macdrv_resize_desktop(void)
+{
+    HWND hwnd = NtUserGetDesktopWindow();
+    CGRect new_desktop_rect;
+    RECT current_desktop_rect;
+
+    macdrv_reset_device_metrics();
+    new_desktop_rect = macdrv_get_desktop_rect();
+    if (!NtUserGetWindowRect(hwnd, &current_desktop_rect) ||
+        !CGRectEqualToRect(cgrect_from_rect(current_desktop_rect), new_desktop_rect))
+    {
+        send_message_timeout(HWND_BROADCAST, WM_MACDRV_RESET_DEVICE_METRICS, 0, 0,
+                             SMTO_ABORTIFHUNG, 2000, NULL);
+        NtUserSetWindowPos(hwnd, 0, CGRectGetMinX(new_desktop_rect), CGRectGetMinY(new_desktop_rect),
+                           CGRectGetWidth(new_desktop_rect), CGRectGetHeight(new_desktop_rect),
+                           SWP_NOZORDER | SWP_NOACTIVATE | SWP_DEFERERASE);
+        send_message_timeout(HWND_BROADCAST, WM_MACDRV_DISPLAYCHANGE, 0, 0,
+                             SMTO_ABORTIFHUNG, 2000, NULL);
+    }
+}
 
 #define WM_WINE_NOTIFY_ACTIVITY WM_USER
 
@@ -1591,6 +1611,9 @@ LRESULT macdrv_DesktopWindowProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
 #pragma clang diagnostic pop
         break;
     }
+    case WM_DISPLAYCHANGE:
+        macdrv_resize_desktop();
+        break;
     }
     return NtUserMessageCall(hwnd, msg, wp, lp, 0, NtUserDefWindowProc, FALSE);
 }
@@ -1987,27 +2010,6 @@ LRESULT macdrv_WindowMessage(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
             release_win_data(data);
         }
         return 0;
-    case WM_MACDRV_UPDATE_DESKTOP_RECT:
-        if (hwnd == NtUserGetDesktopWindow())
-        {
-            CGRect new_desktop_rect;
-            RECT current_desktop_rect;
-
-            macdrv_reset_device_metrics();
-            new_desktop_rect = macdrv_get_desktop_rect();
-            if (!NtUserGetWindowRect(hwnd, &current_desktop_rect) ||
-                !CGRectEqualToRect(cgrect_from_rect(current_desktop_rect), new_desktop_rect))
-            {
-                send_message_timeout(HWND_BROADCAST, WM_MACDRV_RESET_DEVICE_METRICS, 0, 0,
-                                     SMTO_ABORTIFHUNG, 2000, NULL);
-                NtUserSetWindowPos(hwnd, 0, CGRectGetMinX(new_desktop_rect), CGRectGetMinY(new_desktop_rect),
-                                   CGRectGetWidth(new_desktop_rect), CGRectGetHeight(new_desktop_rect),
-                                   SWP_NOZORDER | SWP_NOACTIVATE | SWP_DEFERERASE);
-                send_message_timeout(HWND_BROADCAST, WM_MACDRV_DISPLAYCHANGE, 0, 0,
-                                     SMTO_ABORTIFHUNG, 2000, NULL);
-            }
-        }
-        return 0;
     case WM_MACDRV_RESET_DEVICE_METRICS:
         macdrv_reset_device_metrics();
         return 0;
