@@ -343,6 +343,7 @@ static void set_stream_volumes(ACImpl *This)
     params.master_volume = (This->session->mute ? 0.0f : This->session->master_vol);
     params.volumes = This->vols;
     params.session_volumes = This->session->channel_vols;
+    params.channel = 0;
     OSS_CALL(set_volumes, &params);
 }
 
@@ -772,7 +773,7 @@ static HRESULT WINAPI AudioClient_GetBufferSize(IAudioClient3 *iface,
         return AUDCLNT_E_NOT_INITIALIZED;
 
     params.stream = This->stream;
-    params.size = frames;
+    params.frames = frames;
 
     OSS_CALL(get_buffer_size, &params);
     TRACE("buffer size: %u\n", *frames);
