@@ -2,6 +2,7 @@
  * Copyright 2011-2012 Maarten Lankhorst
  * Copyright 2010-2011 Maarten Lankhorst for CodeWeavers
  * Copyright 2011 Andrew Eikum for CodeWeavers
+ * Copyright 2022 Huw Davies
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,6 +29,7 @@
 #include "winternl.h"
 #include "wine/debug.h"
 #include "wine/list.h"
+#include "wine/unixlib.h"
 
 #include "ole2.h"
 #include "mimeole.h"
@@ -44,24 +46,16 @@
 #include "endpointvolume.h"
 #include "audiopolicy.h"
 
-#include "unixlib.h"
+#include "../mmdevapi/unixlib.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(pulse);
 
 static unixlib_handle_t pulse_handle;
 
+#define MAX_PULSE_NAME_LEN 256
+
 #define NULL_PTR_ERR MAKE_HRESULT(SEVERITY_ERROR, FACILITY_WIN32, RPC_X_NULL_REF_POINTER)
 
-/* From <dlls/mmdevapi/mmdevapi.h> */
-enum DriverPriority {
-    Priority_Unavailable = 0,
-    Priority_Low,
-    Priority_Neutral,
-    Priority_Preferred
-};
-
-static struct pulse_config pulse_config;
-
 static HANDLE pulse_thread;
 static struct list g_sessions = LIST_INIT(g_sessions);
 static struct list g_devices_cache = LIST_INIT(g_devices_cache);
@@ -231,8 +225,8 @@ static void pulse_call(enum unix_funcs code, void *params)
 static void pulse_release_stream(stream_handle stream, HANDLE timer)
 {
     struct release_stream_params params;
-    params.stream = stream;
-    params.timer  = timer;
+    params.stream       = stream;
+    params.timer_thread = timer;
     pulse_call(release_stream, &params);
 }
 
@@ -376,6 +370,7 @@ static void set_stream_volumes(ACImpl *This)
     params.master_volume   = This->session->mute ? 0.0f : This->session->master_vol;
     params.volumes         = This->vol;
     params.session_volumes = This->session->channel_vols;
+    params.channel         = 0;
     pulse_call(set_volumes, &params);
 }
 
@@ -458,7 +453,7 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids_out, GUID **ke
 
     for (i = 0; i < params.num; i++) {
         WCHAR *name = (WCHAR *)((char *)params.endpoints + params.endpoints[i].name);
-        char *pulse_name = (char *)params.endpoints + params.endpoints[i].pulse_name;
+        char *pulse_name = (char *)params.endpoints + params.endpoints[i].device;
         unsigned int size = (wcslen(name) + 1) * sizeof(WCHAR);
 
         if (!(ids[i] = HeapAlloc(GetProcessHeap(), 0, size))) {
@@ -491,11 +486,10 @@ int WINAPI AUDDRV_GetPriority(void)
     struct test_connect_params params;
     char *name;
 
-    params.name   = name = get_application_name(FALSE);
-    params.config = &pulse_config;
+    params.name = name = get_application_name(FALSE);
     pulse_call(test_connect, &params);
     free(name);
-    return SUCCEEDED(params.result) ? Priority_Preferred : Priority_Unavailable;
+    return params.priority;
 }
 
 static BOOL get_pulse_name_by_guid(const GUID *guid, char pulse_name[MAX_PULSE_NAME_LEN], EDataFlow *flow)
@@ -878,11 +872,12 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient3 *iface,
     }
 
     params.name = name = get_application_name(TRUE);
-    params.pulse_name  = This->pulse_name;
-    params.dataflow = This->dataflow;
-    params.mode     = mode;
+    params.device   = This->pulse_name;
+    params.flow     = This->dataflow;
+    params.share    = mode;
     params.flags    = flags;
     params.duration = duration;
+    params.period   = period;
     params.fmt      = fmt;
     params.stream   = &stream;
     params.channel_count = &channel_count;
@@ -936,7 +931,7 @@ static HRESULT WINAPI AudioClient_GetBufferSize(IAudioClient3 *iface,
         return AUDCLNT_E_NOT_INITIALIZED;
 
     params.stream = This->pulse_stream;
-    params.size = out;
+    params.frames = out;
     pulse_call(get_buffer_size, &params);
     return params.result;
 }
@@ -1136,22 +1131,36 @@ static HRESULT WINAPI AudioClient_GetMixFormat(IAudioClient3 *iface,
         WAVEFORMATEX **pwfx)
 {
     ACImpl *This = impl_from_IAudioClient3(iface);
+    struct get_mix_format_params params;
 
     TRACE("(%p)->(%p)\n", This, pwfx);
 
     if (!pwfx)
         return E_POINTER;
+    *pwfx = NULL;
 
-    *pwfx = clone_format(&pulse_config.modes[This->dataflow == eCapture].format.Format);
-    if (!*pwfx)
+    params.device = This->pulse_name;
+    params.flow = This->dataflow;
+    params.fmt = CoTaskMemAlloc(sizeof(WAVEFORMATEXTENSIBLE));
+    if (!params.fmt)
         return E_OUTOFMEMORY;
-    dump_fmt(*pwfx);
-    return S_OK;
+
+    pulse_call(get_mix_format, &params);
+
+    if (SUCCEEDED(params.result)) {
+        *pwfx = &params.fmt->Format;
+        dump_fmt(*pwfx);
+    } else {
+        CoTaskMemFree(params.fmt);
+    }
+
+    return params.result;
 }
 
 static HRESULT WINAPI AudioClient_GetDevicePeriod(IAudioClient3 *iface,
         REFERENCE_TIME *defperiod, REFERENCE_TIME *minperiod)
 {
+    struct get_device_period_params params;
     ACImpl *This = impl_from_IAudioClient3(iface);
 
     TRACE("(%p)->(%p, %p)\n", This, defperiod, minperiod);
@@ -1159,12 +1168,14 @@ static HRESULT WINAPI AudioClient_GetDevicePeriod(IAudioClient3 *iface,
     if (!defperiod && !minperiod)
         return E_POINTER;
 
-    if (defperiod)
-        *defperiod = pulse_config.modes[This->dataflow == eCapture].def_period;
-    if (minperiod)
-        *minperiod = pulse_config.modes[This->dataflow == eCapture].min_period;
+    params.flow = This->dataflow;
+    params.device = This->pulse_name;
+    params.def_period = defperiod;
+    params.min_period = minperiod;
 
-    return S_OK;
+    pulse_call(get_device_period, &params);
+
+    return params.result;
 }
 
 static HRESULT WINAPI AudioClient_Start(IAudioClient3 *iface)
@@ -1549,7 +1560,7 @@ static HRESULT WINAPI AudioCaptureClient_GetBuffer(IAudioCaptureClient *iface,
     params.stream = This->pulse_stream;
     params.data   = data;
     params.frames = frames;
-    params.flags  = flags;
+    params.flags  = (UINT*)flags;
     params.devpos = devpos;
     params.qpcpos = qpcpos;
     pulse_call(get_capture_buffer, &params);
@@ -2007,7 +2018,7 @@ static HRESULT WINAPI AudioSessionControl_GetState(IAudioSessionControl2 *iface,
 
         params.stream = client->pulse_stream;
         pulse_call(is_started, &params);
-        if (params.started) {
+        if (params.result == S_OK) {
             *state = AudioSessionStateActive;
             goto out;
         }
@@ -2646,35 +2657,33 @@ HRESULT WINAPI AUDDRV_GetPropValue(GUID *guid, const PROPERTYKEY *prop, PROPVARI
 {
     struct get_prop_value_params params;
     char pulse_name[MAX_PULSE_NAME_LEN];
-    DWORD size;
+    unsigned int size = 0;
 
     TRACE("%s, (%s,%lu), %p\n", wine_dbgstr_guid(guid), wine_dbgstr_guid(&prop->fmtid), prop->pid, out);
 
     if (!get_pulse_name_by_guid(guid, pulse_name, &params.flow))
         return E_FAIL;
 
-    params.pulse_name = pulse_name;
+    params.device = pulse_name;
     params.guid = guid;
     params.prop = prop;
-    pulse_call(get_prop_value, &params);
+    params.value = out;
+    params.buffer = NULL;
+    params.buffer_size = &size;
 
-    if (params.result != S_OK)
-        return params.result;
+    while(1) {
+        pulse_call(get_prop_value, &params);
 
-    switch (params.vt) {
-    case VT_LPWSTR:
-        size = (wcslen(params.wstr) + 1) * sizeof(WCHAR);
-        if (!(out->pwszVal = CoTaskMemAlloc(size)))
+        if(params.result != E_NOT_SUFFICIENT_BUFFER)
+            break;
+
+        CoTaskMemFree(params.buffer);
+        params.buffer = CoTaskMemAlloc(*params.buffer_size);
+        if(!params.buffer)
             return E_OUTOFMEMORY;
-        memcpy(out->pwszVal, params.wstr, size);
-        break;
-    case VT_UI4:
-        out->ulVal = params.ulVal;
-        break;
-    default:
-        assert(0);
     }
-    out->vt = params.vt;
+    if(FAILED(params.result))
+        CoTaskMemFree(params.buffer);
 
-    return S_OK;
+    return params.result;
 }
