@@ -837,7 +837,8 @@ static void focus_out( Display *display , HWND hwnd )
         return;
     }
     if (hwnd != NtUserGetForegroundWindow()) return;
-    send_message( hwnd, WM_CANCELMODE, 0, 0 );
+    if (!(NtUserGetWindowLongW( hwnd, GWL_STYLE ) & WS_MINIMIZE))
+        send_message( hwnd, WM_CANCELMODE, 0, 0 );
 
     /* don't reset the foreground window, if the window which is
        getting the focus is a Wine window */
@@ -1802,7 +1803,7 @@ static void handle_xdnd_position_event( HWND hwnd, XClientMessageEvent *event )
     XClientMessageEvent e;
     DWORD effect;
 
-    params.hwnd = hwnd;
+    params.hwnd = HandleToUlong( hwnd );
     params.point = root_to_virtual_screen( event->data.l[2] >> 16, event->data.l[2] & 0xFFFF );
     params.effect = effect = xdnd_action_to_drop_effect( event->data.l[4] );
 
