@@ -144,6 +144,20 @@ static BOOL session_query_option( struct object_header *hdr, DWORD option, void
         *buflen = sizeof(DWORD);
         return TRUE;
 
+    case WINHTTP_OPTION_WEB_SOCKET_RECEIVE_BUFFER_SIZE:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
+        *(DWORD *)buffer = session->websocket_receive_buffer_size;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
+    case WINHTTP_OPTION_WEB_SOCKET_SEND_BUFFER_SIZE:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
+        *(DWORD *)buffer = session->websocket_send_buffer_size;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
     default:
         FIXME( "unimplemented option %lu\n", option );
         SetLastError( ERROR_INVALID_PARAMETER );
@@ -233,6 +247,38 @@ static BOOL session_set_option( struct object_header *hdr, DWORD option, void *b
         FIXME( "WINHTTP_OPTION_MAX_CONNS_PER_1_0_SERVER: %lu\n", *(DWORD *)buffer );
         return TRUE;
 
+    case WINHTTP_OPTION_WEB_SOCKET_RECEIVE_BUFFER_SIZE:
+    {
+        DWORD buffer_size;
+
+        if (buflen != sizeof(buffer_size))
+        {
+            SetLastError( ERROR_INSUFFICIENT_BUFFER );
+            return FALSE;
+        }
+
+        buffer_size = *(DWORD *)buffer;
+        TRACE( "%#lx\n", buffer_size );
+        session->websocket_receive_buffer_size = buffer_size;
+        return TRUE;
+    }
+
+    case WINHTTP_OPTION_WEB_SOCKET_SEND_BUFFER_SIZE:
+    {
+        DWORD buffer_size;
+
+        if (buflen != sizeof(buffer_size))
+        {
+            SetLastError( ERROR_INSUFFICIENT_BUFFER );
+            return FALSE;
+        }
+
+        buffer_size = *(DWORD *)buffer;
+        TRACE( "%#lx\n", buffer_size );
+        session->websocket_send_buffer_size = buffer_size;
+        return TRUE;
+    }
+
     default:
         FIXME( "unimplemented option %lu\n", option );
         SetLastError( ERROR_WINHTTP_INVALID_OPTION );
@@ -242,6 +288,7 @@ static BOOL session_set_option( struct object_header *hdr, DWORD option, void *b
 
 static const struct object_vtbl session_vtbl =
 {
+    NULL,
     session_destroy,
     session_query_option,
     session_set_option
@@ -269,6 +316,8 @@ HINTERNET WINAPI WinHttpOpen( LPCWSTR agent, DWORD access, LPCWSTR proxy, LPCWST
     session->send_timeout = DEFAULT_SEND_TIMEOUT;
     session->receive_timeout = DEFAULT_RECEIVE_TIMEOUT;
     session->receive_response_timeout = DEFAULT_RECEIVE_RESPONSE_TIMEOUT;
+    session->websocket_receive_buffer_size = 32768;
+    session->websocket_send_buffer_size = 32768;
     list_init( &session->cookie_cache );
     InitializeCriticalSection( &session->cs );
     session->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": session.cs");
@@ -382,6 +431,7 @@ static BOOL connect_query_option( struct object_header *hdr, DWORD option, void
 
 static const struct object_vtbl connect_vtbl =
 {
+    NULL,
     connect_destroy,
     connect_query_option,
     NULL
@@ -840,6 +890,20 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
         *buflen = sizeof(DWORD);
         return TRUE;
 
+    case WINHTTP_OPTION_WEB_SOCKET_RECEIVE_BUFFER_SIZE:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
+        *(DWORD *)buffer = request->websocket_receive_buffer_size;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
+    case WINHTTP_OPTION_WEB_SOCKET_SEND_BUFFER_SIZE:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
+        *(DWORD *)buffer = request->websocket_set_send_buffer_size;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
     default:
         FIXME( "unimplemented option %lu\n", option );
         SetLastError( ERROR_INVALID_PARAMETER );
@@ -1076,6 +1140,38 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
         SetLastError(ERROR_INVALID_PARAMETER);
         return FALSE;
 
+    case WINHTTP_OPTION_WEB_SOCKET_RECEIVE_BUFFER_SIZE:
+    {
+        DWORD buffer_size;
+
+        if (buflen != sizeof(buffer_size))
+        {
+            SetLastError( ERROR_INSUFFICIENT_BUFFER );
+            return FALSE;
+        }
+
+        buffer_size = *(DWORD *)buffer;
+        WARN( "Setting websocket receive buffer size currently has not effct, size %lu\n", buffer_size );
+        request->websocket_receive_buffer_size = buffer_size;
+        return TRUE;
+    }
+
+    case WINHTTP_OPTION_WEB_SOCKET_SEND_BUFFER_SIZE:
+    {
+        DWORD buffer_size;
+
+        if (buflen != sizeof(buffer_size))
+        {
+            SetLastError( ERROR_INSUFFICIENT_BUFFER );
+            return FALSE;
+        }
+
+        buffer_size = *(DWORD *)buffer;
+        request->websocket_set_send_buffer_size = buffer_size;
+        TRACE( "Websocket send buffer size %lu.\n", buffer_size);
+        return TRUE;
+    }
+
     default:
         FIXME( "unimplemented option %lu\n", option );
         SetLastError( ERROR_WINHTTP_INVALID_OPTION );
@@ -1085,6 +1181,7 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
 
 static const struct object_vtbl request_vtbl =
 {
+    NULL,
     request_destroy,
     request_query_option,
     request_set_option
@@ -1160,6 +1257,7 @@ HINTERNET WINAPI WinHttpOpenRequest( HINTERNET hconnect, const WCHAR *verb, cons
     request->hdr.notify_mask = connect->hdr.notify_mask;
     request->hdr.context = connect->hdr.context;
     request->hdr.redirect_policy = connect->hdr.redirect_policy;
+    init_queue( &request->queue );
 
     addref_object( &connect->hdr );
     request->connect = connect;
@@ -1170,6 +1268,9 @@ HINTERNET WINAPI WinHttpOpenRequest( HINTERNET hconnect, const WCHAR *verb, cons
     request->receive_timeout = connect->session->receive_timeout;
     request->receive_response_timeout = connect->session->receive_response_timeout;
     request->max_redirects = 10;
+    request->websocket_receive_buffer_size = connect->session->websocket_receive_buffer_size;
+    request->websocket_send_buffer_size = connect->session->websocket_send_buffer_size;
+    request->websocket_set_send_buffer_size = request->websocket_send_buffer_size;
 
     if (!verb || !verb[0]) verb = L"GET";
     if (!(request->verb = strdupW( verb ))) goto end;
@@ -1224,6 +1325,15 @@ static BOOL query_option( struct object_header *hdr, DWORD option, void *buffer,
 
     switch (option)
     {
+    case WINHTTP_OPTION_WORKER_THREAD_COUNT:
+    {
+        FIXME( "WINHTTP_OPTION_WORKER_THREAD_COUNT semi-stub.\n" );
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
+        *(DWORD *)buffer = 0;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+    }
     case WINHTTP_OPTION_CONTEXT_VALUE:
     {
         if (!validate_buffer( buffer, buflen, sizeof(DWORD_PTR) )) return FALSE;
