@@ -239,7 +239,7 @@ static void load_config( const WCHAR *key_name, struct console_config *config )
 
     HKEY key, app_key;
 
-    TRACE("loading %s registry settings.\n", wine_dbgstr_w( key_name ));
+    TRACE( "Loading default console settings\n" );
 
     memcpy( config->color_map, color_map, sizeof(color_map) );
     memset( config->face_name, 0, sizeof(config->face_name) );
@@ -265,13 +265,15 @@ static void load_config( const WCHAR *key_name, struct console_config *config )
     config->win_pos.Y    = 0;
     config->edition_mode = 0;
 
-    /* read global settings */
+    /* Load default console settings */
     if (!RegOpenKeyW( HKEY_CURRENT_USER, L"Console", &key ))
     {
         load_registry_key( key, config );
-        /* if requested, load part related to console title */
+
+        /* Load app-specific console settings (if any) */
         if (key_name && !RegOpenKeyW( key, key_name, &app_key ))
         {
+            TRACE( "Loading %s console settings\n", wine_dbgstr_w(key_name) );
             load_registry_key( app_key, config );
             RegCloseKey( app_key );
         }
@@ -280,76 +282,135 @@ static void load_config( const WCHAR *key_name, struct console_config *config )
     TRACE( "%s\n", debugstr_config( config ));
 }
 
-static void save_registry_key( HKEY key, const struct console_config *config )
+static void save_registry_key( HKEY key, const struct console_config *config, BOOL save_all )
 {
+    struct console_config default_config;
     DWORD val, width, height, i;
     WCHAR color_name[13];
 
     TRACE( "%s\n", debugstr_config( config ));
 
+    if (!save_all)
+        load_config( NULL, &default_config );
+
     for (i = 0; i < ARRAY_SIZE(config->color_map); i++)
     {
-        wsprintfW( color_name, L"ColorTable%02d", i );
-        val = config->color_map[i];
-        RegSetValueExW( key, color_name, 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+        if (save_all || config->color_map[i] != default_config.color_map[i])
+        {
+            wsprintfW( color_name, L"ColorTable%02d", i );
+            val = config->color_map[i];
+            RegSetValueExW( key, color_name, 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+        }
     }
 
-    val = config->cursor_size;
-    RegSetValueExW( key, L"CursorSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->cursor_size != default_config.cursor_size)
+    {
+        val = config->cursor_size;
+        RegSetValueExW( key, L"CursorSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->cursor_visible;
-    RegSetValueExW( key, L"CursorVisible", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->cursor_visible != default_config.cursor_visible)
+    {
+        val = config->cursor_visible;
+        RegSetValueExW( key, L"CursorVisible", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->edition_mode;
-    RegSetValueExW( key, L"EditionMode", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->edition_mode != default_config.edition_mode)
+    {
+        val = config->edition_mode;
+        RegSetValueExW( key, L"EditionMode", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    RegSetValueExW( key, L"FaceName", 0, REG_SZ, (BYTE *)&config->face_name,
-                    (lstrlenW(config->face_name) + 1) * sizeof(WCHAR) );
+    if (save_all || lstrcmpW( config->face_name, default_config.face_name ))
+    {
+        RegSetValueExW( key, L"FaceName", 0, REG_SZ, (BYTE *)&config->face_name,
+                        (lstrlenW(config->face_name) + 1) * sizeof(WCHAR) );
+    }
 
-    val = config->font_pitch_family;
-    RegSetValueExW( key, L"FontPitchFamily", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->font_pitch_family != default_config.font_pitch_family)
+    {
+        val = config->font_pitch_family;
+        RegSetValueExW( key, L"FontPitchFamily", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    width  = MulDiv( config->cell_width,  USER_DEFAULT_SCREEN_DPI, GetDpiForSystem() );
-    height = MulDiv( config->cell_height, USER_DEFAULT_SCREEN_DPI, GetDpiForSystem() );
-    val = MAKELONG( width, height );
-    RegSetValueExW( key, L"FontSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->cell_height != default_config.cell_height ||
+        config->cell_width != default_config.cell_width)
+    {
+        width  = MulDiv( config->cell_width,  USER_DEFAULT_SCREEN_DPI, GetDpiForSystem() );
+        height = MulDiv( config->cell_height, USER_DEFAULT_SCREEN_DPI, GetDpiForSystem() );
+        val = MAKELONG( width, height );
 
-    val = config->font_weight;
-    RegSetValueExW( key, L"FontWeight", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+        RegSetValueExW( key, L"FontSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->history_size;
-    RegSetValueExW( key, L"HistoryBufferSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->font_weight != default_config.font_weight)
+    {
+        val = config->font_weight;
+        RegSetValueExW( key, L"FontWeight", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->history_mode;
-    RegSetValueExW( key, L"HistoryNoDup", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->history_size != default_config.history_size)
+    {
+        val = config->history_size;
+        RegSetValueExW( key, L"HistoryBufferSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->insert_mode;
-    RegSetValueExW( key, L"InsertMode", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->history_mode != default_config.history_mode)
+    {
+        val = config->history_mode;
+        RegSetValueExW( key, L"HistoryNoDup", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->menu_mask;
-    RegSetValueExW( key, L"MenuMask", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->insert_mode != default_config.insert_mode)
+    {
+        val = config->insert_mode;
+        RegSetValueExW( key, L"InsertMode", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->popup_attr;
-    RegSetValueExW( key, L"PopupColors", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->menu_mask != default_config.menu_mask)
+    {
+        val = config->menu_mask;
+        RegSetValueExW( key, L"MenuMask", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->quick_edit;
-    RegSetValueExW( key, L"QuickEdit", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->popup_attr != default_config.popup_attr)
+    {
+        val = config->popup_attr;
+        RegSetValueExW( key, L"PopupColors", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = MAKELONG(config->sb_width, config->sb_height);
-    RegSetValueExW( key, L"ScreenBufferSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->quick_edit != default_config.quick_edit)
+    {
+        val = config->quick_edit;
+        RegSetValueExW( key, L"QuickEdit", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = config->attr;
-    RegSetValueExW( key, L"ScreenColors", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->sb_width != default_config.sb_width ||
+        config->sb_height != default_config.sb_height)
+    {
+        val = MAKELONG(config->sb_width, config->sb_height);
+        RegSetValueExW( key, L"ScreenBufferSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 
-    val = MAKELONG( config->win_width, config->win_height );
-    RegSetValueExW( key, L"WindowSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    if (save_all || config->attr != default_config.attr)
+    {
+        val = config->attr;
+        RegSetValueExW( key, L"ScreenColors", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
+
+    if (save_all || config->win_width != default_config.win_width ||
+        config->win_height != default_config.win_height)
+    {
+        val = MAKELONG( config->win_width, config->win_height );
+        RegSetValueExW( key, L"WindowSize", 0, REG_DWORD, (BYTE *)&val, sizeof(val) );
+    }
 }
 
 static void save_config( const WCHAR *key_name, const struct console_config *config )
 {
     HKEY key, app_key;
 
-    TRACE( "%s %s\n", debugstr_w( key_name ), debugstr_config( config ));
+    TRACE( "Saving %s console settings\n", key_name ? debugstr_w( key_name ) : "default" );
 
     if (RegCreateKeyW( HKEY_CURRENT_USER, L"Console", &key ))
     {
@@ -365,12 +426,11 @@ static void save_config( const WCHAR *key_name, const struct console_config *con
         }
         else
         {
-            /* FIXME: maybe only save the values different from the default value ? */
-            save_registry_key( app_key, config );
+            save_registry_key( app_key, config, FALSE );
             RegCloseKey( app_key );
         }
     }
-    else save_registry_key( key, config );
+    else save_registry_key( key, config, TRUE );
     RegCloseKey(key);
 }
 
@@ -601,8 +661,11 @@ static void update_window( struct console *console )
         if (console->win == GetFocus())
         {
             if (console->window->cursor_visible)
+            {
                 CreateCaret( console->win, console->window->cursor_bitmap,
                              console->active->font.width, console->active->font.height );
+                update_window_cursor( console );
+            }
             else
                 DestroyCaret();
         }
@@ -678,6 +741,7 @@ static BOOL set_console_font( struct console *console, const LOGFONTW *logfont )
     struct font_info *font_info = &console->active->font;
     HFONT font, old_font;
     TEXTMETRICW tm;
+    WCHAR face_name[LF_FACESIZE];
     CPINFO cpinfo;
     HDC dc;
 
@@ -703,6 +767,7 @@ static BOOL set_console_font( struct console *console, const LOGFONTW *logfont )
 
     old_font = SelectObject( dc, font );
     GetTextMetricsW( dc, &tm );
+    font_info->face_len = GetTextFaceW( dc, ARRAY_SIZE(face_name), face_name ) - 1;
     SelectObject( dc, old_font );
     ReleaseDC( console->win, dc );
 
@@ -712,9 +777,8 @@ static BOOL set_console_font( struct console *console, const LOGFONTW *logfont )
     font_info->weight = tm.tmWeight;
 
     free( font_info->face_name );
-    font_info->face_len = wcslen( logfont->lfFaceName );
     font_info->face_name = malloc( font_info->face_len * sizeof(WCHAR) );
-    memcpy( font_info->face_name, logfont->lfFaceName, font_info->face_len * sizeof(WCHAR) );
+    memcpy( font_info->face_name, face_name, font_info->face_len * sizeof(WCHAR) );
 
     /* FIXME: use maximum width for DBCS codepages since some chars take two cells */
     if (GetCPInfo( console->output_cp, &cpinfo ) && cpinfo.MaxCharSize == 2)
@@ -736,6 +800,8 @@ struct font_chooser
 {
     struct console *console;
     int             pass;
+    unsigned int    font_height;
+    unsigned int    font_width;
     BOOL            done;
 };
 
@@ -791,92 +857,83 @@ static BOOL validate_font( struct console *console, const LOGFONTW *lf, int pass
     return TRUE;
 }
 
-/* helper functions to get a decent font for the renderer */
-static int WINAPI get_first_font_sub_enum( const LOGFONTW *lf, const TEXTMETRICW *tm,
-                                           DWORD font_type, LPARAM lparam)
+static int CALLBACK enum_first_font_proc( const LOGFONTW *lf, const TEXTMETRICW *tm,
+                                          DWORD font_type, LPARAM lparam )
 {
     struct font_chooser *fc = (struct font_chooser *)lparam;
+    LOGFONTW mlf;
+
+    if (font_type != TRUETYPE_FONTTYPE) return 1;
+
+    TRACE( "%s\n", debugstr_logfont( lf, font_type ));
+
+    if (!validate_font( fc->console, lf, fc->pass ))
+        return 1;
 
     TRACE( "%s\n", debugstr_textmetric( tm, font_type ));
 
-    if (validate_font_metric( fc->console, tm, font_type, fc->pass ))
-    {
-        LOGFONTW mlf = *lf;
+    if (!validate_font_metric( fc->console, tm, font_type, fc->pass ))
+        return 1;
 
-        /* Use the default sizes for the font (this is needed, especially for
-         * TrueType fonts, so that we get a decent size, not the max size)
-         */
-        mlf.lfWidth  = fc->console->active->font.width;
-        mlf.lfHeight = fc->console->active->font.height;
-        if (!mlf.lfHeight)
-            mlf.lfHeight = MulDiv( 16, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI );
+    /* set default font size */
+    mlf = *lf;
+    mlf.lfHeight = fc->font_height;
+    mlf.lfWidth = fc->font_width;
 
-        if (set_console_font( fc->console, &mlf ))
-        {
-            struct console_config config;
+    if (!set_console_font( fc->console, &mlf ))
+        return 1;
 
-            fc->done = 1;
+    fc->done = TRUE;
 
-            /* since we've modified the current config with new font information,
-             * set this information as the new default.
-             */
-            load_config( fc->console->window->config_key, &config );
-            config.cell_width  = fc->console->active->font.width;
-            config.cell_height = fc->console->active->font.height;
-            memcpy( config.face_name, fc->console->active->font.face_name,
-                    fc->console->active->font.face_len * sizeof(WCHAR) );
-            config.face_name[fc->console->active->font.face_len] = 0;
-
-            /* Force also its writing back to the registry so that we can get it
-             * the next time.
-             */
-            save_config( fc->console->window->config_key, &config );
-            return 0;
-        }
-    }
-    return 1;
+    return 0;
 }
 
-static int WINAPI get_first_font_enum( const LOGFONTW *lf, const TEXTMETRICW *tm,
-                                       DWORD font_type, LPARAM lparam )
+static void set_first_font( struct console *console, struct console_config *config )
 {
-    struct font_chooser *fc = (struct font_chooser *)lparam;
+    LOGFONTW lf;
+    struct font_chooser fc;
 
-    TRACE("%s\n", debugstr_logfont( lf, font_type ));
+    TRACE("Looking for a suitable console font\n");
 
-    if (validate_font( fc->console, lf, fc->pass ))
+    memset( &lf, 0, sizeof(lf) );
+    lf.lfCharSet = DEFAULT_CHARSET;
+    lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
+
+    fc.console = console;
+    fc.font_height = config->cell_height;
+    fc.font_width = config->cell_width;
+    fc.done = FALSE;
+
+    for (fc.pass = 0; fc.pass <= 5; fc.pass++)
     {
-        EnumFontFamiliesW( fc->console->window->mem_dc, lf->lfFaceName,
-                           get_first_font_sub_enum, lparam );
-        return !fc->done; /* we just need the first matching one... */
+        EnumFontFamiliesExW( console->window->mem_dc, &lf, enum_first_font_proc, (LPARAM)&fc, 0);
+        if (fc.done) break;
     }
-    return 1;
+
+    if (fc.pass > 5)
+        ERR("Unable to find a valid console font\n");
+
+    /* Update active configuration */
+    config->cell_width  = console->active->font.width;
+    config->cell_height = console->active->font.height;
+    config->font_pitch_family = console->active->font.pitch_family;
+    memcpy( config->face_name, console->active->font.face_name,
+            console->active->font.face_len * sizeof(WCHAR) );
+    config->face_name[console->active->font.face_len] = 0;
+
+    /* Save default console configuration to the registry */
+    save_config( NULL, config );
 }
 
-
-/* sets logfont as the new font for the console */
+/* Sets the font specified in the LOGFONT as the new console font */
 void update_console_font( struct console *console, const WCHAR *face_name, size_t face_name_size,
                           unsigned int height, unsigned int weight )
 {
-    struct font_chooser fc;
     LOGFONTW lf;
 
-    if (face_name[0] && height && weight)
-    {
-        fill_logfont( &lf, face_name, face_name_size, height, weight );
-        if (set_console_font( console, &lf )) return;
-    }
+    fill_logfont( &lf, face_name, face_name_size, height, weight );
 
-    /* try to find an acceptable font */
-    WARN( "Couldn't match the font from registry, trying to find one\n" );
-    fc.console = console;
-    fc.done = FALSE;
-    for (fc.pass = 0; fc.pass <= 5; fc.pass++)
-    {
-        EnumFontFamiliesW( console->window->mem_dc, NULL, get_first_font_enum, (LPARAM)&fc );
-        if (fc.done) return;
-    }
-    ERR( "Couldn't find a decent font\n" );
+    set_console_font( console, &lf );
 }
 
 /* get a cell from a relative coordinate in window (takes into account the scrolling) */
@@ -1206,13 +1263,6 @@ struct dialog_info
     struct console        *console;
     struct console_config  config;
     HWND                   dialog;      /* handle to active propsheet */
-    int                    font_count;  /* number of fonts */
-    struct dialog_font_info
-    {
-        unsigned int  height;
-        unsigned int  weight;
-        WCHAR         faceName[LF_FACESIZE];
-    } *font;  /* array of fonts */
 };
 
 /* dialog proc for the option property sheet */
@@ -1352,12 +1402,10 @@ static LRESULT WINAPI font_preview_proc( HWND hwnd, UINT msg, WPARAM wparam, LPA
             struct dialog_info *di;
             HFONT font, old_font;
             PAINTSTRUCT ps;
-            int size_idx;
 
             di = (struct dialog_info *)GetWindowLongPtrW( GetParent( hwnd ), DWLP_USER );
             BeginPaint( hwnd, &ps );
 
-            size_idx = SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_GETCURSEL, 0, 0 );
             font = (HFONT)GetWindowLongPtrW( hwnd, 0 );
             if (font)
             {
@@ -1373,7 +1421,7 @@ static LRESULT WINAPI font_preview_proc( HWND hwnd, UINT msg, WPARAM wparam, LPA
                 SetTextColor( ps.hdc, get_color( di, IDC_FNT_COLOR_FG ));
                 len = LoadStringW( GetModuleHandleW(NULL), IDS_FNT_PREVIEW, buf, ARRAY_SIZE(buf) );
                 if (len) TextOutW( ps.hdc, 0, 0, buf, len );
-                TextOutW( ps.hdc, 0, di->font[size_idx].height, ascii, ARRAY_SIZE(ascii) - 1 );
+                TextOutW( ps.hdc, 0, di->config.cell_height, ascii, ARRAY_SIZE(ascii) - 1 );
                 SelectObject( ps.hdc, old_font );
             }
             EndPaint( hwnd, &ps );
@@ -1461,146 +1509,42 @@ static LRESULT WINAPI color_preview_proc( HWND hwnd, UINT msg, WPARAM wparam, LP
     return 0;
 }
 
-/* enumerates all the font names with at least one valid font */
-static int WINAPI font_enum_size2( const LOGFONTW *lf, const TEXTMETRICW *tm,
-                                   DWORD font_type, LPARAM lparam )
-{
-    struct dialog_info *di = (struct dialog_info *)lparam;
-    TRACE( "%s\n", debugstr_textmetric( tm, font_type ));
-    if (validate_font_metric( di->console, tm, font_type, 0 )) di->font_count++;
-    return 1;
-}
-
-static int WINAPI font_enum( const LOGFONTW *lf, const TEXTMETRICW *tm,
-                             DWORD font_type, LPARAM lparam )
-{
-    struct dialog_info *di = (struct dialog_info *)lparam;
-
-    TRACE( "%s\n", debugstr_logfont( lf, font_type ));
-
-    if (validate_font( di->console, lf, 0 ))
-    {
-        if (font_type & RASTER_FONTTYPE)
-        {
-            di->font_count = 0;
-            EnumFontFamiliesW( di->console->window->mem_dc, lf->lfFaceName,
-                               font_enum_size2, (LPARAM)di );
-        }
-        else
-            di->font_count = 1;
-
-        if (di->font_count)
-            SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_ADDSTRING,
-                                 0, (LPARAM)lf->lfFaceName );
-    }
-    return 1;
-}
-
-static int WINAPI font_enum_size( const LOGFONTW *lf, const TEXTMETRICW *tm,
-                                  DWORD font_type, LPARAM lparam )
-{
-    struct dialog_info *di = (struct dialog_info *)lparam;
-    WCHAR buf[32];
-
-    TRACE( "%s\n", debugstr_textmetric( tm, font_type ));
-
-    if (di->font_count == 0 && !(font_type & RASTER_FONTTYPE))
-    {
-        static const int sizes[] = {8,9,10,11,12,14,16,18,20,22,24,26,28,36,48,72};
-        int i;
-
-        di->font_count = ARRAY_SIZE(sizes);
-        di->font = malloc( di->font_count * sizeof(di->font[0]) );
-        for (i = 0; i < di->font_count; i++)
-        {
-            /* drop sizes where window size wouldn't fit on screen */
-            if (sizes[i] * di->config.win_height > GetSystemMetrics( SM_CYSCREEN ))
-            {
-                di->font_count = i;
-                break;
-            }
-            di->font[i].height = sizes[i];
-            di->font[i].weight = 400;
-            lstrcpyW( di->font[i].faceName, lf->lfFaceName );
-            wsprintfW( buf, L"%d", sizes[i] );
-            SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_INSERTSTRING, i, (LPARAM)buf );
-        }
-        /* don't need to enumerate other */
-        return 0;
-    }
-
-    if (validate_font_metric( di->console, tm, font_type, 0 ))
-    {
-        int idx = 0;
-
-        /* we want the string to be sorted with a numeric order, not a lexicographic...
-         * do the job by hand... get where to insert the new string
-         */
-        while (idx < di->font_count && tm->tmHeight > di->font[idx].height)
-            idx++;
-        while (idx < di->font_count &&
-               tm->tmHeight == di->font[idx].height &&
-               tm->tmWeight > di->font[idx].weight)
-            idx++;
-        if (idx == di->font_count ||
-            tm->tmHeight != di->font[idx].height ||
-            tm->tmWeight < di->font[idx].weight)
-        {
-            /* here we need to add the new entry */
-            wsprintfW( buf, L"%d", tm->tmHeight );
-            SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_INSERTSTRING, idx, (LPARAM)buf );
-
-            /* now grow our arrays and insert the values at the same index than in the list box */
-            if (di->font_count)
-            {
-                di->font = realloc( di->font, sizeof(*di->font) * (di->font_count + 1) );
-                if (idx != di->font_count)
-                    memmove( &di->font[idx + 1], &di->font[idx],
-                             (di->font_count - idx) * sizeof(*di->font) );
-            }
-            else
-                di->font = malloc( sizeof(*di->font) );
-            di->font[idx].height = tm->tmHeight;
-            di->font[idx].weight = tm->tmWeight;
-            lstrcpyW( di->font[idx].faceName, lf->lfFaceName );
-            di->font_count++;
-        }
-    }
-    return 1;
-}
-
 static BOOL select_font( struct dialog_info *di )
 {
-    struct console_config config;
     int font_idx, size_idx;
+    WCHAR face_name[LF_FACESIZE], height_buf[4];
+    size_t len;
+    unsigned int font_height;
+    LOGFONTW lf;
     HFONT font, old_font;
     DWORD_PTR args[2];
     WCHAR buf[256];
     WCHAR fmt[128];
-    LOGFONTW lf;
 
     font_idx = SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_GETCURSEL, 0, 0 );
     size_idx = SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_GETCURSEL, 0, 0 );
 
-    if (font_idx < 0 || size_idx < 0 || size_idx >= di->font_count)
+    if (font_idx < 0 || size_idx < 0)
         return FALSE;
 
-    fill_logfont( &lf, di->font[size_idx].faceName,
-                  wcslen(di->font[size_idx].faceName) * sizeof(WCHAR),
-                  di->font[size_idx].height, di->font[size_idx].weight );
-    font = select_font_config( &config, di->console->output_cp, di->console->win, &lf );
+    len = SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_GETTEXT, font_idx, (LPARAM)&face_name );
+    SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_GETTEXT, size_idx, (LPARAM)&height_buf );
+    font_height = _wtoi( height_buf );
+
+    fill_logfont( &lf, face_name, len * sizeof(WCHAR), font_height, FW_NORMAL );
+    font = select_font_config( &di->config, di->console->output_cp, di->console->win, &lf );
     if (!font) return FALSE;
 
-    if (config.cell_height != di->font[size_idx].height)
-        TRACE( "mismatched heights (%u<>%u)\n", config.cell_height, di->font[size_idx].height );
+    if (di->config.cell_height != font_height)
+        TRACE( "mismatched heights (%u<>%u)\n", di->config.cell_height, font_height );
 
     old_font = (HFONT)SendDlgItemMessageW( di->dialog, IDC_FNT_PREVIEW, WM_GETFONT, 0, 0 );
     SendDlgItemMessageW( di->dialog, IDC_FNT_PREVIEW, WM_SETFONT, (WPARAM)font, TRUE );
     if (old_font) DeleteObject( old_font );
 
     LoadStringW( GetModuleHandleW(NULL), IDS_FNT_DISPLAY, fmt, ARRAY_SIZE(fmt) );
-    args[0] = config.cell_width;
-    args[1] = config.cell_height;
+    args[0] = di->config.cell_width;
+    args[1] = di->config.cell_height;
     FormatMessageW( FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
                     fmt, 0, 0, buf, ARRAY_SIZE(buf), (__ms_va_list*)args );
 
@@ -1608,52 +1552,62 @@ static BOOL select_font( struct dialog_info *di )
     return TRUE;
 }
 
-/* fills the size list box according to selected family in font LB */
 static BOOL fill_list_size( struct dialog_info *di, BOOL init )
 {
-    WCHAR face_name[LF_FACESIZE];
-    int idx = 0;
-
-    idx = SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_GETCURSEL, 0, 0 );
-    if (idx < 0) return FALSE;
-
-    SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_GETTEXT, idx, (LPARAM)face_name );
-    SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_RESETCONTENT, 0, 0 );
-    free( di->font );
-    di->font_count = 0;
-    di->font = NULL;
-
-    EnumFontFamiliesW( di->console->window->mem_dc, face_name, font_enum_size, (LPARAM)di );
-
     if (init)
     {
-        int ref = -1;
-        for (idx = 0; idx < di->font_count; idx++)
+        static const int sizes[] = {8,9,10,11,12,14,16,18,20,22,24,26,28,36,48,72};
+        unsigned int i, idx = 4;
+        WCHAR buf[4];
+
+        for (i = 0; i < ARRAY_SIZE(sizes); i++)
         {
-            if (!lstrcmpW( di->font[idx].faceName, di->config.face_name ) &&
-                di->font[idx].height == di->config.cell_height &&
-                di->font[idx].weight == di->config.font_weight)
-            {
-                if (ref == -1) ref = idx;
-                else TRACE("Several matches found: ref=%d idx=%d\n", ref, idx);
-            }
+            wsprintfW( buf, L"%u", sizes[i] );
+            SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_INSERTSTRING, -1, (LPARAM)buf );
+
+            if (di->config.cell_height == sizes[i]) idx = i;
         }
-        idx = (ref == -1) ? 0 : ref;
+
+        SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_SETCURSEL, idx, 0 );
     }
 
-    SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_SIZE, LB_SETCURSEL, idx, 0 );
     select_font( di );
+
     return TRUE;
 }
 
+static int CALLBACK enum_list_font_proc( const LOGFONTW *lf, const TEXTMETRICW *tm,
+                                         DWORD font_type, LPARAM lparam )
+{
+    struct dialog_info *di = (struct dialog_info *)lparam;
+
+    if (font_type != TRUETYPE_FONTTYPE) return 1;
+
+    TRACE( "%s\n", debugstr_logfont( lf, font_type ));
+
+    if (validate_font( di->console, lf, 0 ))
+        SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_ADDSTRING, 0, (LPARAM)lf->lfFaceName );
+
+    return 1;
+}
+
 static BOOL fill_list_font( struct dialog_info *di )
 {
-    SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_RESETCONTENT, 0, 0 );
-    EnumFontFamiliesW( di->console->window->mem_dc, NULL, font_enum, (LPARAM)di );
+    LOGFONTW lf;
+
+    memset( &lf, 0, sizeof(lf) );
+    lf.lfCharSet = DEFAULT_CHARSET;
+    lf.lfFaceName[0] = 0;
+    lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
+
+    EnumFontFamiliesExW( di->console->window->mem_dc, &lf, enum_list_font_proc, (LPARAM)di, 0 );
+
     if (SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_SELECTSTRING,
                              -1, (LPARAM)di->config.face_name ) == LB_ERR)
         SendDlgItemMessageW( di->dialog, IDC_FNT_LIST_FONT, LB_SETCURSEL, 0, 0 );
+
     fill_list_size( di, TRUE );
+
     return TRUE;
 }
 
@@ -1668,7 +1622,7 @@ static INT_PTR WINAPI font_dialog_proc( HWND dialog, UINT msg, WPARAM wparam, LP
         di = (struct dialog_info *)((PROPSHEETPAGEA*)lparam)->lParam;
         di->dialog = dialog;
         SetWindowLongPtrW( dialog, DWLP_USER, (DWORD_PTR)di );
-        /* remove dialog from this control, font will be reset when listboxes are filled */
+        /* use default system font until user-selected font is applied */
         SendDlgItemMessageW( dialog, IDC_FNT_PREVIEW, WM_SETFONT, 0, 0 );
         fill_list_font( di );
         SetWindowLongW( GetDlgItem( dialog, IDC_FNT_COLOR_BK ), 0, (di->config.attr >> 4) & 0x0F );
@@ -1702,18 +1656,6 @@ static INT_PTR WINAPI font_dialog_proc( HWND dialog, UINT msg, WPARAM wparam, LP
                 di->dialog = dialog;
                 break;
             case PSN_APPLY:
-                val = SendDlgItemMessageW( dialog, IDC_FNT_LIST_SIZE, LB_GETCURSEL, 0, 0 );
-                if (val < di->font_count)
-                {
-                    LOGFONTW lf;
-
-                    fill_logfont( &lf, di->font[val].faceName,
-                                  wcslen(di->font[val].faceName) * sizeof(WCHAR),
-                                  di->font[val].height, di->font[val].weight );
-                    DeleteObject( select_font_config( &di->config, di->console->output_cp,
-                                                      di->console->win, &lf ));
-                }
-
                 val = (GetWindowLongW( GetDlgItem( dialog, IDC_FNT_COLOR_BK ), 0 ) << 4) |
                     GetWindowLongW( GetDlgItem( dialog, IDC_FNT_COLOR_FG ), 0 );
                 di->config.attr = val;
@@ -1819,34 +1761,6 @@ static INT_PTR WINAPI config_dialog_proc( HWND dialog, UINT msg, WPARAM wparam,
     return TRUE;
 }
 
-/* dialog proc for choosing how to handle modification to the console settings */
-static INT_PTR WINAPI save_dialog_proc( HWND dialog, UINT msg, WPARAM wparam, LPARAM lparam )
-{
-    switch (msg)
-    {
-    case WM_INITDIALOG:
-        SendMessageW( dialog, WM_NEXTDLGCTL, (WPARAM)GetDlgItem( dialog, IDC_SAV_SESSION ), TRUE );
-        SendDlgItemMessageW( dialog, IDC_SAV_SESSION, BM_SETCHECK, BST_CHECKED, 0 );
-        return FALSE;
-
-    case WM_COMMAND:
-        switch (LOWORD(wparam))
-        {
-        case IDOK:
-            EndDialog( dialog,
-                       (IsDlgButtonChecked(dialog, IDC_SAV_SAVE) == BST_CHECKED) ?
-                       IDC_SAV_SAVE : IDC_SAV_SESSION );
-            break;
-        case IDCANCEL:
-            EndDialog( dialog, IDCANCEL ); break;
-        }
-        break;
-    default:
-        return FALSE;
-    }
-    return TRUE;
-}
-
 static void apply_config( struct console *console, const struct console_config *config )
 {
     if (console->active->width != config->sb_width || console->active->height != config->sb_height)
@@ -1964,22 +1878,18 @@ static BOOL config_dialog( struct console *console, BOOL current )
     PROPSHEETPAGEW psp;
     WNDCLASSW wndclass;
     WCHAR buff[256];
-    BOOL modify_session = FALSE;
-    BOOL save = FALSE;
 
     InitCommonControls();
 
     memset( &di, 0, sizeof(di) );
     di.console = console;
-    if (!current)
-    {
+
+    if (current)
+        current_config( console, &di.config );
+    else
         load_config( NULL, &di.config );
-        save = TRUE;
-    }
-    else current_config( console, &di.config );
+
     prev_config = di.config;
-    di.font_count = 0;
-    di.font = NULL;
 
     wndclass.style         = 0;
     wndclass.lpfnWndProc   = font_preview_proc;
@@ -2036,42 +1946,22 @@ static BOOL config_dialog( struct console *console, BOOL current )
     header.hwndParent = console->win;
     header.u3.phpage  = pages;
     header.dwFlags    = PSH_NOAPPLYNOW;
-    PropertySheetW( &header );
+    if (PropertySheetW( &header ) < 1)
+        return TRUE;
 
     if (!memcmp( &prev_config, &di.config, sizeof(prev_config) ))
         return TRUE;
 
     TRACE( "%s\n", debugstr_config(&di.config) );
 
-    if (!save)
-    {
-        switch (DialogBoxW( GetModuleHandleW( NULL ), MAKEINTRESOURCEW(IDD_SAVE_SETTINGS),
-                            console->win, save_dialog_proc ))
-        {
-        case IDC_SAV_SAVE:
-            save = TRUE;
-            modify_session = TRUE;
-            break;
-        case IDC_SAV_SESSION:
-            modify_session = TRUE;
-            break;
-        default:
-            ERR( "dialog failed\n" );
-            /* fall through */
-        case IDCANCEL:
-            modify_session = FALSE;
-            save = FALSE;
-            break;
-        }
-    }
-
-    if (modify_session)
+    if (current)
     {
         apply_config( console, &di.config );
         update_window( di.console );
     }
-    if (save)
-        save_config( current ? console->window->config_key : NULL, &di.config );
+
+    save_config( current ? console->window->config_key : NULL, &di.config );
+
     return TRUE;
 }
 
@@ -2555,6 +2445,9 @@ BOOL init_window( struct console *console )
                         0, 0, 0, 0, wndclass.hInstance, console ))
         return FALSE;
 
+    if (!config.face_name[0])
+        set_first_font( console, &config );
+
     apply_config( console, &config );
     return TRUE;
 }
