@@ -416,7 +416,8 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
         if (opts->unix_lib)
         {
             strarray_add( &flags, "-install_name" );
-            strarray_add( &flags, strmake( "@loader_path/%s.so", output_name ) );
+            strarray_add( &flags, strmake( "@rpath/%s.so", output_name ) );
+            strarray_add( &flags, "-Wl,-rpath,@loader_path/" );
         }
         strarray_addall( &link_args, flags );
         return link_args;
@@ -659,7 +660,7 @@ static void init_argv0_dir( const char *argv0 )
     static int pathname[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
     size_t path_size = PATH_MAX;
     char *path = xmalloc( path_size );
-    if (!sysctl( pathname, sizeof(pathname)/sizeof(pathname[0]), path, &path_size, NULL, 0 ))
+    if (!sysctl( pathname, ARRAY_SIZE(pathname), path, &path_size, NULL, 0 ))
         dir = realpath( path, NULL );
     free( path );
 #else
