@@ -356,11 +356,10 @@ static void init_nls_info( struct nls_info *info, unsigned short *ptr )
 
 static const struct nls_info *get_nls_info( unsigned int codepage )
 {
-    struct stat st;
     unsigned short *data;
     char *path;
     unsigned int i;
-    int fd;
+    size_t size;
 
     for (i = 0; i < ARRAY_SIZE(nlsinfo) && nlsinfo[i].codepage; i++)
         if (nlsinfo[i].codepage == codepage) return &nlsinfo[i];
@@ -370,18 +369,15 @@ static const struct nls_info *get_nls_info( unsigned int codepage )
     for (i = 0; nlsdirs[i]; i++)
     {
         path = strmake( "%s/c_%03u.nls", nlsdirs[i], codepage );
-        if ((fd = open( path, O_RDONLY )) != -1) break;
+        if ((data = read_file( path, &size )))
+        {
+            free( path );
+            init_nls_info( &nlsinfo[i], data );
+            return &nlsinfo[i];
+        }
         free( path );
     }
-    if (!nlsdirs[i]) return NULL;
-
-    fstat( fd, &st );
-    data = xmalloc( st.st_size );
-    if (read( fd, data, st.st_size ) != st.st_size) error( "failed to load %s\n", path );
-    close( fd );
-    free( path );
-    init_nls_info( &nlsinfo[i], data );
-    return &nlsinfo[i];
+    return NULL;
 }
 
 int is_valid_codepage(int cp)
@@ -433,70 +429,6 @@ WCHAR *codepage_to_unicode( int codepage, const char *src, int srclen, int *dstl
 
 #endif  /* _WIN32 */
 
-/*******************************************************************
- *         buffer management
- *
- * Function for writing to a memory buffer.
- */
-
 unsigned char *output_buffer;
 size_t output_buffer_pos;
 size_t output_buffer_size;
-
-static void check_output_buffer_space( size_t size )
-{
-    if (output_buffer_pos + size >= output_buffer_size)
-    {
-        output_buffer_size = max( output_buffer_size * 2, output_buffer_pos + size );
-        output_buffer = xrealloc( output_buffer, output_buffer_size );
-    }
-}
-
-void init_output_buffer(void)
-{
-    output_buffer_size = 1024;
-    output_buffer_pos = 0;
-    output_buffer = xmalloc( output_buffer_size );
-}
-
-void flush_output_buffer( const char *name )
-{
-    int fd = open( name, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666 );
-    if (fd == -1) error( "Error creating %s\n", name );
-    if (write( fd, output_buffer, output_buffer_pos ) != output_buffer_pos)
-        error( "Error writing to %s\n", name );
-    close( fd );
-    free( output_buffer );
-}
-
-void put_byte( unsigned char val )
-{
-    check_output_buffer_space( 1 );
-    output_buffer[output_buffer_pos++] = val;
-}
-
-void put_word( unsigned short val )
-{
-    check_output_buffer_space( 2 );
-    output_buffer[output_buffer_pos++] = val;
-    output_buffer[output_buffer_pos++] = val >> 8;
-}
-
-void put_dword( unsigned int val )
-{
-    check_output_buffer_space( 4 );
-    output_buffer[output_buffer_pos++] = val;
-    output_buffer[output_buffer_pos++] = val >> 8;
-    output_buffer[output_buffer_pos++] = val >> 16;
-    output_buffer[output_buffer_pos++] = val >> 24;
-}
-
-void align_output( unsigned int align )
-{
-    size_t size = align - (output_buffer_pos % align);
-
-    if (size == align) return;
-    check_output_buffer_space( size );
-    memset( output_buffer + output_buffer_pos, 0, size );
-    output_buffer_pos += size;
-}
