@@ -36,7 +36,6 @@
 #include "../tools.h"
 #include "wrc.h"
 #include "utils.h"
-#include "dumpres.h"
 #include "newstruc.h"
 #include "parser.h"
 #include "wpp_private.h"
@@ -110,8 +109,8 @@ int extensions = 1;
 /*
  * Language setting for resources (-l option)
  */
-static language_t *defaultlanguage;
-language_t *currentlanguage = NULL;
+static language_t defaultlanguage;
+language_t currentlanguage = 0;
 
 /*
  * Set when extra warnings should be generated (-W option)
@@ -132,8 +131,6 @@ int utf8_input = 0;
 
 int check_utf8 = 1;  /* whether to check for valid utf8 */
 
-static int verify_translations_mode;
-
 static char *output_name;	/* The name given by the -o option */
 const char *input_name = NULL;	/* The name given on the command-line */
 static char *temp_name = NULL;	/* Temporary file for preprocess pipe */
@@ -167,7 +164,6 @@ enum long_options_values
     LONG_OPT_VERSION,
     LONG_OPT_DEBUG,
     LONG_OPT_PEDANTIC,
-    LONG_OPT_VERIFY_TRANSL
 };
 
 static const char short_options[] =
@@ -194,7 +190,6 @@ static const struct long_option long_options[] = {
 	{ "undefine", 1, 'U' },
 	{ "use-temp-file", 0, LONG_OPT_TMPFILE },
 	{ "verbose", 0, 'v' },
-	{ "verify-translations", 0, LONG_OPT_VERIFY_TRANSL },
 	{ "version", 0, LONG_OPT_VERSION },
 	{ NULL }
 };
@@ -276,7 +271,7 @@ static int load_file( const char *input_name, const char *output_name )
     }
 
     /* Reset the language */
-    currentlanguage = dup_language( defaultlanguage );
+    currentlanguage = defaultlanguage;
     check_utf8 = 1;
 
     /* Go from .rc to .res */
@@ -293,7 +288,6 @@ static int load_file( const char *input_name, const char *output_name )
         unlink( temp_name );
         temp_name = NULL;
     }
-    free( currentlanguage );
     return ret;
 }
 
@@ -308,7 +302,7 @@ static void init_argv0_dir( const char *argv0 )
     static int pathname[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
     size_t path_size = PATH_MAX;
     char *path = xmalloc( path_size );
-    if (!sysctl( pathname, sizeof(pathname)/sizeof(pathname[0]), path, &path_size, NULL, 0 ))
+    if (!sysctl( pathname, ARRAY_SIZE(pathname), path, &path_size, NULL, 0 ))
         dir = realpath( path, NULL );
     free( path );
 #else
@@ -358,9 +352,6 @@ static void option_callback( int optc, char *optarg )
     case LONG_OPT_PEDANTIC:
         pedantic = 1;
         break;
-    case LONG_OPT_VERIFY_TRANSL:
-        verify_translations_mode = 1;
-        break;
     case 'D':
         wpp_add_cmdline_define(optarg);
         break;
@@ -384,14 +375,10 @@ static void option_callback( int optc, char *optarg )
         else if (strcmp(optarg, "rc")) error("Output format %s not supported.\n", optarg);
         break;
     case 'l':
-    {
-        int lan;
-        lan = strtol(optarg, NULL, 0);
-        if (get_language_codepage(PRIMARYLANGID(lan), SUBLANGID(lan)) == -1)
-            error("Language %04x is not supported\n", lan);
-        defaultlanguage = new_language(PRIMARYLANGID(lan), SUBLANGID(lan));
-    }
-    break;
+        defaultlanguage = strtol(optarg, NULL, 0);
+        if (get_language_codepage( defaultlanguage ) == -1)
+            error("Language %04x is not supported\n", defaultlanguage);
+        break;
     case 'm':
         if (!strcmp( optarg, "16" )) win32 = 0;
         else win32 = 1;
@@ -482,10 +469,6 @@ int main(int argc,char *argv[])
                        (debuglevel & DEBUGLEVEL_PPTRACE) != 0,
                        (debuglevel & DEBUGLEVEL_PPMSG) != 0 );
 
-	/* Check if the user set a language, else set default */
-	if(!defaultlanguage)
-		defaultlanguage = new_language(0, 0);
-
 	atexit(cleanup_files);
 
         for (i = 0; i < input_files.count; i++)
@@ -496,14 +479,6 @@ int main(int argc,char *argv[])
 	/* stdin special case. NULL means "stdin" for wpp. */
         if (input_files.count == 0 && load_file( NULL, output_name )) exit(1);
 
-	if(debuglevel & DEBUGLEVEL_DUMP)
-		dump_resources(resource_top);
-
-	if(verify_translations_mode)
-	{
-		verify_translations(resource_top);
-		exit(0);
-	}
         if (!po_mode && output_name)
         {
             if (strendswith( output_name, ".po" )) po_mode = 1;
